/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 08.10.2009 - [JR] - creation
 * 28.10.2009 - [JR] - open MyBets as default screen
 */
package com.sibvisions.apps.packung;

import javax.rad.application.IWorkScreen;
import javax.rad.application.genui.UILauncher;
import javax.rad.genui.UIColor;
import javax.rad.genui.UIFont;
import javax.rad.genui.UIImage;
import javax.rad.genui.celleditor.UILinkedCellEditor;
import javax.rad.genui.component.UIIcon;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIDesktopPanel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.IDataBook;
import javax.rad.model.IDataRow;
import javax.rad.model.event.DataRowEvent;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;
import javax.rad.remote.IConnectionConstants;
import javax.rad.remote.MasterConnection;
import javax.rad.ui.IAlignmentConstants;
import javax.rad.ui.container.IInternalFrame;
import javax.rad.ui.event.UIActionEvent;

import com.sibvisions.apps.components.NavigationTable;
import com.sibvisions.apps.packung.auth.ActivationAuthenticator;
import com.sibvisions.apps.packung.screens.InformationWorkScreen;
import com.sibvisions.apps.packung.screens.MyBetsWorkScreen;
import com.sibvisions.apps.packung.screens.RegistrationWorkScreen;
import com.sibvisions.apps.packung.screens.UnregisterWorkScreen;
import com.sibvisions.apps.projx.Login;
import com.sibvisions.apps.projx.Menu;
import com.sibvisions.apps.projx.ProjX;
import com.sibvisions.rad.model.remote.RemoteDataBook;
import com.sibvisions.rad.model.remote.RemoteDataSource;
import com.sibvisions.util.ArrayUtil;

/**
 * The <code>PackungApplication</code> is the application class for Packung!.
 * 
 * @author Ren Jahn
 */
public class PackungApplication extends ProjX
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the connection for the anonymous user. */
	private MasterConnection conAnonymous 		= null;
	
	/** the data source. */
	protected RemoteDataSource dataSource 		= null;

	/** the user defaults. */
	private RemoteDataBook rdbUserDefaults 		= null;
	/** the competitions lookup. */
	private RemoteDataBook rdbCompetitionsLook 	= null;
	/** the competitions lookup. */
	private RemoteDataBook rdbLanguagesLook 	= null;
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new instance of <code>PackungApplication</code> for a specific
	 * {@link UILauncher}.
	 * 
	 * @param pLauncher the application launcher
	 * @throws Throwable if the database access has errors
	 */
	public PackungApplication(UILauncher pLauncher) throws Throwable
	{
		super(pLauncher);
		
		UIImage.setImageMapping(UIImage.ADD_SMALL, "/com/sibvisions/apps/packung/images/16x16/new.png");
		UIImage.setImageMapping(UIImage.ADD_LARGE, null);

		UIImage.setImageMapping(UIImage.REMOVE_SMALL, "/com/sibvisions/apps/packung/images/16x16/delete.png");
		UIImage.setImageMapping(UIImage.REMOVE_LARGE, null);

		UIImage.setImageMapping(UIImage.MOVE_UP_SMALL, "/com/sibvisions/apps/packung/images/16x16/up.png");

		UIImage.setImageMapping(UIImage.MOVE_DOWN_SMALL, "/com/sibvisions/apps/packung/images/16x16/down.png");
		
		UIImage.setImageMapping(UIImage.EXPORT_SMALL, "/com/sibvisions/apps/packung/images/16x16/export.png");
		UIImage.setImageMapping(UIImage.EXPORT_LARGE, null);
		
		UIImage.setImageMapping(UIImage.IMPORT_SMALL, "/com/sibvisions/apps/packung/images/16x16/import.png");
		UIImage.setImageMapping(UIImage.IMPORT_LARGE, null);
				
		NavigationTable.setDefaultButtonSize(21);
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Overwritten methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void configureDesktopPane(UIDesktopPanel pDesktop)
	{
		super.configureDesktopPane(pDesktop);
		
		pDesktop.add(new UIIcon(UIImage.getImage("/com/sibvisions/apps/packung/images/worldplayers.gif")), UIBorderLayout.NORTH);
	}
	
	/**
	 * Enables the lost password feature.
	 * 
	 * @param pLogin {@inheritDoc}
	 */
	@Override
	protected void configureLogin(Login pLogin)
	{
		super.configureLogin(pLogin);
		
		pLogin.setLostPasswordEnabled(true);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public PackungMenu createMenu()
	{
		return new PackungMenu(this);
	}
	
	/**
	 * Closes the anonymous user connection and call the super method.
	 */
	@Override
	protected void afterLogin()
	{
		if (conAnonymous != null)
		{
			try
			{
				conAnonymous.close();
			}
			catch (Throwable th)
			{
				//nothing to be done
			}
			finally
			{
				conAnonymous = null;
			}
		}
		
		super.afterLogin();
		
		//show MyBets work screen if available
		try
		{
			if (hasRole("user"))
			{
				IWorkScreen screen = openWorkScreen(MyBetsWorkScreen.class.getName());
				
				IInternalFrame frame = getFrame(screen);
				
				if (frame != null)
				{
					frame.setLocationRelativeTo(getDesktopPane());
				}
			}
		}
		catch (Throwable th)
		{
			debug(th);
		}
		
		//special handling for security manager property!
		try
		{
			if (getConnection().getProperty(IConnectionConstants.PREFIX_CLIENT + "activated") != null)
			{
				showInformation(this, "Your account was successful activated!");
			}
		}
		catch (Throwable th)
		{
			debug(th);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void initUser() throws Throwable
	{
		super.initUser();
		
		AbstractConnection con = getConnection();
		
		try
		{
			if (con.getProperty(IConnectionConstants.PREFIX_SERVER + IConnectionConstants.PREFIX_SESSION + "REGISTERED") != null)
			{
				getMenu().addItem(PackungMenu.HELP_UNREGISTER, 
						  "doShowUnregister", 
						  null, 
						  Menu.HELP, 
						  null, 
						  null,
					      "Unregister", 
					      "/com/sibvisions/apps/packung/images/16x16/unregister.png", 
					      null, 
					      null, 
					      true);
			}
		}
		catch (Throwable th)
		{
			//nothing to be done
		}

		dataSource = new RemoteDataSource(con);
		dataSource.open();
		
		rdbCompetitionsLook = new RemoteDataBook();
		rdbCompetitionsLook.setDataSource(dataSource);
		rdbCompetitionsLook.setName("competitionsLook");
		rdbCompetitionsLook.open();

		UILinkedCellEditor competitionsLook = new UILinkedCellEditor();
		competitionsLook.setLinkReference(new ReferenceDefinition(
				new String[] {"COMP_ID", "COMPETITION", "COMP_IMAGE"},
				rdbCompetitionsLook,
				new String[] {"ID", "COMPETITION", "IMAGE"}));

		rdbLanguagesLook = new RemoteDataBook();
		rdbLanguagesLook.setDataSource(dataSource);
		rdbLanguagesLook.setName("languagesLook");
		rdbLanguagesLook.open();

		UILinkedCellEditor languagesLook = new UILinkedCellEditor();
		languagesLook.setLinkReference(new ReferenceDefinition(
				new String[] {"LANG_ID", "LANG_CODE", "LANGUAGE"},
				rdbLanguagesLook,
				new String[] {"ID", "CODE_ID", "LANGUAGE"}));

		rdbUserDefaults = new RemoteDataBook();
		rdbUserDefaults.setDataSource(dataSource);
		rdbUserDefaults.setName("userDefaults");
		rdbUserDefaults.open();
		rdbUserDefaults.eventValuesChanged().addListener(this, "doCheckCompId");
		doCheckCompId(null);
		
		rdbUserDefaults.getRowDefinition().getColumnDefinition("USERNAME").setReadOnly(true);
		rdbUserDefaults.getRowDefinition().getColumnDefinition("COMPETITION").getDataType().setCellEditor(competitionsLook);
		rdbUserDefaults.getRowDefinition().getColumnDefinition("LANGUAGE").getDataType().setCellEditor(languagesLook);

		UILabel label = new UILabel();
		label.setFont(UIFont.getDefaultFont().deriveFont(UIFont.BOLD));
		label.setForeground(UIColor.white);
		label.setText("Choose Competition");
		label.setVerticalAlignment(IAlignmentConstants.ALIGN_BOTTOM);
		
		UIEditor editor = new UIEditor();
		editor.setDataRow(rdbUserDefaults);
		editor.setColumnName("COMPETITION");
		editor.setMinimumSize(150, 0);
		
		UIFormLayout layPanel = new UIFormLayout();
		layPanel.setMargins(9, 0, 5, 0);
		layPanel.setVerticalGap(7);
		
		UIPanel panel = new UIPanel();
		panel.setBackground(null);
		panel.setLayout(layPanel);
		panel.add(label, layPanel.getConstraints(0, 0));
		panel.add(editor, layPanel.getConstraints(0, 1));
		
		((PackungMenu)getMenu()).setRightComponent(panel);
		
		//Update the language with the user-defined or default database language
		setLanguage((String)rdbUserDefaults.getValue("LANG_CODE"));
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void afterLogout()
	{
		//to be sure that we are able to create an anonymous connection!
		try
		{
			super.afterLogout();

			((PackungMenu)getMenu()).setRightComponent(null);
			
			try
			{
				dataSource.close();
			}
			catch (Exception ex)
			{
				// Silent close.
			}
			dataSource = null; 
			
			rdbUserDefaults = null;
			rdbCompetitionsLook = null;
			rdbLanguagesLook = null;
		}
		finally
		{
			createAnonymousConnection();
		}
	}
	
	/**
	 * Opens the user registration screen.
	 * 
	 * @param pEvent {@inheritDoc}
	 * @throws Throwable {@inheritDoc}
	 */
	@Override
	public void doRegister(UIActionEvent pEvent) throws Throwable
	{
		RegistrationWorkScreen rwsRegister = new RegistrationWorkScreen(this, conAnonymous);
		
		openContent(this, "Registration", true, rwsRegister);
	}
	
	/**
	 * Sends a new password to a specific user. The username will be taken
	 * from the login screen.
	 * 
	 * @param pEvent {@inheritDoc}
	 * @throws Throwable {@inheritDoc}
	 */
	@Override
	public void doLostPassword(UIActionEvent pEvent) throws Throwable
	{
		conAnonymous.call("registration", "setNewPassword", getLogin().getUserName(), getLanguage());
		
		showInformation(this, "Password successful sent!\n\nCheck your mailbox!");
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void doReload() throws Exception
	{
		dataSource.reloadAllDataBooks();
		
		super.doReload();
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	

	/**
	 * Creates a new anonymous connection if not already created!
	 */
	private void createAnonymousConnection()
	{
		if (conAnonymous == null && !isConnected())
		{
			try
			{
				conAnonymous = new MasterConnection(createConnection());
				
				conAnonymous.setApplicationName(getLauncher().getParameter(PARAM_LOGIN_APPLICATION));
				conAnonymous.setProperty(IConnectionConstants.PREFIX_CLIENT + "login.anonymous", "true");
				conAnonymous.setTimeout(-1);
				conAnonymous.setAliveInterval(-1);
				
				setAdditionalConnectionProperties(conAnonymous);
				
				conAnonymous.open();
			}
			catch (Throwable th)
			{
				// don't show anything to the user!
				conAnonymous = null;
			}
		}
	}
	
	/**
	 * Checks if the application is allowed to login through an activation key.
	 * 
	 * @return <code>true</code> if the authentication was successful, otherwise <code>false</code>
	 */
	public boolean checkActivation()
	{
		ArrayUtil<String> auAuth = new ArrayUtil<String>();
		
		auAuth.add(ActivationAuthenticator.class.getName());
		
		processAutoLogin(auAuth);
		
		if (getConnection() != null)
		{
			afterLogin();
			
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Gets the user defaults.
	 * @return the user defaults.
	 */
	public IDataBook getUserDefaults()
	{
		return rdbUserDefaults;
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Saves the immediate after value changed.
	 * 
	 * @param pDataRowEvent the data row event
	 * @throws Exception throws the exception with the problem.
	 */
	public void doCheckCompId(DataRowEvent pDataRowEvent) throws Exception
	{
		if (pDataRowEvent == null || pDataRowEvent.isChangedColumnName("COMP_ID"))
		{
			if (rdbUserDefaults.getValue("COMP_ID") == null)
			{
				IDataRow competition = null;
				if (pDataRowEvent == null)
				{
					competition = rdbCompetitionsLook.getDataRow(0);
				}
				else
				{
					competition = (IDataBook)pDataRowEvent.getOriginalDataRow();
				}
				if (competition != null)
				{
					rdbUserDefaults.setValues(new String[] {"COMP_ID", "COMPETITION", "COMP_IMAGE"},
							competition.getValues(new String[] {"ID", "COMPETITION", "IMAGE"}));
				}
			}
			rdbUserDefaults.saveSelectedRow();
			super.doReload();
		}
	}
	
	/**
	 * Chesk user unregistration. 
	 * 
	 * @throws Throwable if the check can not be started
	 */
	public void doShowUnregister() throws Throwable
	{
		UnregisterWorkScreen iwsInfo = new UnregisterWorkScreen(this, getConnection());
		
		openContent(this, "Unregister", true, iwsInfo);
	}
	
	/**
	 * Deletes the current user and performs a logs out.
	 * 
	 * @throws Throwable if the user can not be deleted
	 */
	public void doUnregister() throws Throwable
	{
		getConnection().call("registration", "delete", getConnection().getUserName());
		
		logout();
	}
	
	/**
	 * Shows the application information texts (disclaimer, rules, ...).
	 * 
	 * @throws Throwable if the database access causes an error
	 */
	public void doShowInformation() throws Throwable
	{
		InformationWorkScreen iwsInfo = new InformationWorkScreen(this, conAnonymous != null ? conAnonymous : getConnection());
		
		openContent(this, "Information", true, iwsInfo);
	}

}	// PackungApplication
