/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 11.10.2009 - [JR] - creation
 */
package com.sibvisions.apps.packung;

import javax.rad.application.IApplication;
import javax.rad.genui.UIComponent;
import javax.rad.genui.UIImage;
import javax.rad.genui.UIInsets;
import javax.rad.genui.component.AbstractUIButton;
import javax.rad.genui.component.UIButton;
import javax.rad.genui.component.UIIcon;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.genui.menu.UIMenu;
import javax.rad.genui.menu.UIMenuItem;
import javax.rad.ui.IAlignmentConstants;
import javax.rad.ui.IComponent;
import javax.rad.ui.IContainer;
import javax.rad.ui.component.IIcon;
import javax.rad.ui.layout.IFormLayout.IConstraints;

import com.sibvisions.apps.projx.Menu;
import com.sibvisions.util.type.FileUtil;

/**
 * The <code>PackungMenu</code> uses a "graphical toolbar" for the application instead of
 * the standard toolbar.
 * 
 * @author Ren Jahn
 */
public class PackungMenu extends Menu
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the constant value for the mapped 'unregister' menuitem. */
	public static final String HELP_UNREGISTER 	= "Help/Unregister";
	
	/** the constant value for the mapped 'information' menuitem. */
	public static final String HELP_INFORMATION = "Help/Information";
	
	/** the layout for the toolbar. */
	private UIFormLayout folToolBar;
	/** the toolbar panel under the menu. */
	private UIPanel panToolBar;	
	
	/** the current right component. */
	private UIComponent<?> rightComponent;	
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new instance of <code>PackungMenu</code>.
	 * 
	 * @param pApplication the application
	 */
	public PackungMenu(IApplication pApplication)
	{
		super(pApplication);
	}

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Overwritten methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void configureApplication()
	{
		//add the MenuBar 
		IContainer conApp = application.getApplicationPane();
		
		if (!(conApp.getLayout() instanceof UIBorderLayout))
		{
			conApp.setLayout(new UIBorderLayout());
		}
		
		UIPanel panMain = new UIPanel();
		panMain.setLayout(new UIBorderLayout());
		

		UIPanel panTop = new UIPanel();
		panTop.setLayout(new UIBorderLayout());
		
		IIcon icoTopLeft = new UIIcon();
		icoTopLeft.setImage(UIImage.getImage("/com/sibvisions/apps/packung/images/top_left.jpg"));
		icoTopLeft.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		icoTopLeft.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);

		IIcon icoTopRight = new UIIcon();
		icoTopRight.setImage(UIImage.getImage("/com/sibvisions/apps/packung/images/top_right.jpg"));
		icoTopRight.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		icoTopRight.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);
		
		folToolBar = new UIFormLayout();
		folToolBar.setNewlineCount(20);
		folToolBar.setHorizontalGap(0);
		folToolBar.setMargins(new UIInsets(3, 3, 0, 6));
		folToolBar.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);
		folToolBar.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		
		panToolBar = new UIPanel();
		panToolBar.setLayout(folToolBar);
		
		panToolBar.setBackgroundImage(UIImage.getImage("/com/sibvisions/apps/packung/images/top.jpg"));
		
		panTop.add(icoTopLeft, UIBorderLayout.WEST);
		panTop.add(panToolBar, UIBorderLayout.CENTER);
		panTop.add(icoTopRight, UIBorderLayout.EAST);

		panMain.add(panTop, UIBorderLayout.NORTH);
		panMain.add(application.getContentPane(), UIBorderLayout.CENTER);
		
		conApp.add(panMain);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createStandardMenuBar()
	{
		super.createStandardMenuBar();
		
		UIMenuItem miInformation = createMenuItem("doShowInformation", 
									              null, 
									              "Information", 
									              UIImage.getImage("/com/sibvisions/apps/packung/images/toolbar/16x16/information.png"));
	
		IComponent[] comp = get(HELP);
		
		UIMenu menu = (UIMenu)comp[0];
		
		menu.add(miInformation, 2);
		
		put(HELP_INFORMATION, miInformation);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createStandardToolBar()
	{
		panToolBar.removeAll();
		
		UIButton butReload = createToolBarButton("Reload", "doReload", null, "/com/sibvisions/apps/packung/images/toolbar/reload.png");
		UIButton butSave = createToolBarButton("Save", "doSave", null, "/com/sibvisions/apps/packung/images/toolbar/save.png");
		UIButton butInfo = createToolBarButton("Information", "doShowInformation", null, "/com/sibvisions/apps/packung/images/toolbar/information.png");
		
		panToolBar.add(butInfo);
		panToolBar.add(new UILabel("      "));
		panToolBar.add(butSave);
		panToolBar.add(butReload);
		panToolBar.add(new UILabel("      "));

		//update mappings
		add(EDIT, butSave, butReload);
		add(EDIT_SAVE, butSave);
		add(EDIT_RELOAD, butReload);
		
		add(HELP_INFORMATION, butInfo);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void addToolBarButton(String pId,
						            String pGroupId, 
						            String pAction, 
						            String pActionCommand, 
						            String pText,
						            String pImage)
	{
		if (pImage != null)
		{
			if (!HELP_CREATE.equals(pId))
			{
				UIButton but = createToolBarButton(pText, pAction, pActionCommand, pImage);
				
				panToolBar.add(but);
				
				add(pGroupId, but);
				add(pId, but);
			}
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected <T extends AbstractUIButton<?>> T configureToolBarButton(T pButton, String pAction, String pActionCommand, String pText, UIImage pImage)
	{
		pButton.setImage(pImage);
		pButton.eventAction().addListener(application, pAction);
		pButton.setActionCommand(pActionCommand);
		pButton.setVerticalAlignment(IAlignmentConstants.ALIGN_CENTER);
		pButton.setHorizontalAlignment(IAlignmentConstants.ALIGN_CENTER);
		pButton.setFocusable(false);
		pButton.setBorderPainted(false);
		pButton.setMargins(0, 0, 0, 0);
		
		return pButton;
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	

	/**
	 * Creates a toolbar button with moue over and pressed image.
	 * 
	 * @param pToolTip the tooltip to show
	 * @param pAction the action to call when pressed
	 * @param pActionCommand the action command
	 * @param pImage the image to show
	 * @return the preconfigured button
	 */
	private UIButton createToolBarButton(String pToolTip, String pAction, String pActionCommand, String pImage)
	{
		String sExt   = FileUtil.getExtension(pImage);
		String sImage = FileUtil.removeExtension(pImage);

		UIButton but = createToolBarButton(pAction, pActionCommand, null, UIImage.getImage(pImage));
		but.setMouseOverImage(UIImage.getImage(sImage + "_over." + sExt));
		but.setPressedImage(UIImage.getImage(sImage + "_pressed." + sExt));
		
		but.setToolTipText(pToolTip);

		return but;
	}
	
	/**
	 * Gets the right component.
	 * 
	 * @return the right component.
	 */
	public UIComponent<?> getRightComponent()
	{
		return rightComponent;
	}
	
	/**
	 * Sets the right component.
	 * 
	 * @param pRightComponent the right component.
	 */
	public void setRightComponent(UIComponent<?> pRightComponent)
	{
		if (rightComponent != null)
		{
			panToolBar.remove(rightComponent);
		}
		
		rightComponent = pRightComponent;

		if (rightComponent != null)
		{
			IConstraints cons = folToolBar.getConstraints(-1, 0);
			
			cons.setTopAnchor(folToolBar.createAnchor(cons.getTopAnchor()));
			cons.setBottomAnchor(folToolBar.createAnchor(cons.getBottomAnchor()));
			
			panToolBar.add(rightComponent, cons);
			
		}
	}
	
}	// PackungMenu
