/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [MH] - creation
 * 06.11.2009 - [JR] - checkAndResizeImage: used ImageUtil
 */
package com.sibvisions.apps.packung;

import java.io.IOException;
import java.io.OutputStream;

import javax.rad.genui.UIColor;
import javax.rad.genui.UIImage;
import javax.rad.genui.celleditor.UIChoiceCellEditor;
import javax.rad.genui.celleditor.UIDateCellEditor;
import javax.rad.genui.celleditor.UIImageViewer;
import javax.rad.genui.celleditor.UINumberCellEditor;
import javax.rad.genui.celleditor.UITextCellEditor;
import javax.rad.genui.control.UICellFormat;
import javax.rad.genui.control.UITable;
import javax.rad.io.FileHandle;
import javax.rad.io.IFileHandle;
import javax.rad.model.ColumnDefinition;
import javax.rad.model.IDataBook;
import javax.rad.model.IDataPage;
import javax.rad.model.IDataRow;
import javax.rad.model.IRowDefinition;
import javax.rad.model.ModelException;
import javax.rad.model.event.DataRowEvent;
import javax.rad.ui.IAlignmentConstants;
import javax.rad.ui.control.ICellFormat;

import com.sibvisions.util.type.FileUtil;
import com.sibvisions.util.type.ImageUtil;

/**
 * The <code>PackungUtil</code> is a utility class and provides flags, cell editors, format definitions and
 * helper methods.
 * 
 * @author Martin Handsteiner
 */
public final class PackungUtil
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Constants
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/** the default image when no image was found. */
	public static final String IMAGE_NO_COMPETITION = "/com/sibvisions/apps/packung/images/nocompetition.png";
	
	/** the default image when no image was found. */
	public static final String IMAGE_NO_FLAG 		= "/flag_white.png";
	
	/** the default image when no image was found. */
	public static final String IMAGE_NOBODY 		= "/com/sibvisions/apps/packung/images/nobody.png";

	/** the whistle image. */
	public static final String IMAGE_WHISTLE 		= "/com/sibvisions/apps/packung/images/toolbar/16x16/whistle.png";
	
	/** the glock image. */
	public static final String IMAGE_CLOCK 			= "/com/sibvisions/apps/packung/images/12x12/clock.png";
	
	/** the information off image. */
	public static final String INFO_OFF				= "/com/sibvisions/apps/packung/images/16x16/info.png";
	/** the information off image. */
	public static final String PROFILE				= "/com/sibvisions/apps/packung/images/16x16/profile.png";

	/** Rote Zelle. */
	public static final UICellFormat RED_CELL 		= new UICellFormat(UIColor.red);
	/** Orange Zelle. */
	public static final UICellFormat ORANGE_CELL 	= new UICellFormat(UIColor.orange);
	/** Yellow Zelle. */
	public static final UICellFormat YELLOW_CELL 	= new UICellFormat(UIColor.yellow);
	/** Blaue Zelle. */
	public static final UICellFormat BLUE_CELL 		= new UICellFormat(new UIColor(50, 150, 255));
	/** Grne Zelle. */
	public static final UICellFormat GREEN_CELL 	= new UICellFormat(UIColor.green);
	
	/** Passwordfield. */
	public static final UITextCellEditor PASSWORD_EDITOR 		= new UITextCellEditor(UITextCellEditor.TEXT_PLAIN_PASSWORD);
	/** Passwordfield. */
	public static final UITextCellEditor CENTERED_TEXT_EDITOR	= new UITextCellEditor(IAlignmentConstants.ALIGN_CENTER);
	/** Passwordfield. */
	public static final UINumberCellEditor CENTERED_NUMBER_EDITOR	= new UINumberCellEditor(IAlignmentConstants.ALIGN_CENTER);
	/** Multiline field. */
	public static final UITextCellEditor MULTILINE_EDITOR 		= new UITextCellEditor(UITextCellEditor.TEXT_PLAIN_WRAPPEDMULTILINE);

	/** Date editor without time. */	
	public static final UIDateCellEditor DATE_EDITOR 			= new UIDateCellEditor("dd.MMMM.yyyy");
	/** Datetime editor without time. */	
	public static final UIDateCellEditor DATETIME_EDITOR 		= new UIDateCellEditor("dd.MMMM.yyyy HH:mm");

	/** Yes/ No Choice editor. */
	public static final UIChoiceCellEditor YESNO_EDITOR 		= new UIChoiceCellEditor(new Object [] {"Y", "N"},
	                                                                                     new String [] {UIImage.CHECK_YES_SMALL,
			                                                                                            UIImage.CHECK_SMALL},
			                                                                                            UIImage.CHECK_SMALL);
	
	/** Info On/Off Choice editor. */
	public static final UIChoiceCellEditor INFO_EDITOR 			= new UIChoiceCellEditor(new Object [] {"Y", "N"},
	                                                                                     new String [] {INFO_OFF,
			                                                                                            INFO_OFF},
			                                                                                            INFO_OFF);

	/** Positive, negative Choice editor. */
	public static final UIChoiceCellEditor POS_NEG_EDITOR 		= new UIChoiceCellEditor(new Object [] {"Y", "N"},
	                                                                                     new String [] {UIImage.CHECK_YES_SMALL,
			                                                                                            UIImage.CHECK_NO_SMALL},
			                                                                                            UIImage.CHECK_NO_SMALL);
		
	/** Temporary Choice editor. */
	public static final UIChoiceCellEditor TEMP_EDITOR 			= new UIChoiceCellEditor(new Object [] {"Y", "N"},
	                                                                                     new String [] {IMAGE_CLOCK,
			                                                                                            null});
		
	/** The default Imageviewer. */
	public static final UIImageViewer IMAGEVIEWER 				= new UIImageViewer(IMAGE_NO_FLAG);

	/** The Imageviewer for the competition image. */
	public static final UIImageViewer COMPETITION_IMAGEVIEWER 	= new UIImageViewer(IMAGE_NO_COMPETITION, IAlignmentConstants.ALIGN_CENTER, IAlignmentConstants.ALIGN_TOP);
	
	/** The Imageviewer for the competition image. */
	public static final UIImageViewer USER_IMAGEVIEWER 	= new UIImageViewer(IMAGE_NOBODY);

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** Singleton Listener. */
	private static PackungUtil packungUtil = null;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Invisible constructor because <code>PackungUtil</code> is a utility class.
	 */
	private PackungUtil()
	{
	}
	
	/**
	 * Gets the singleton instance of PackungUtil.
	 * 
	 * @return the singleton instance of PackungUtil.
	 */
	private static PackungUtil getInstance()
	{
		if (packungUtil == null)
		{
			packungUtil = new PackungUtil();
		}
		return packungUtil;
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	

	/**
	 * Saves the dataBook immediate.
	 * 
	 * @param pDataBook the dataBook.
	 */
	public static void saveImmediate(IDataBook pDataBook)
	{
		pDataBook.eventValuesChanged().addListener(getInstance(), "doSaveImmediate");
	}
	
	/**
	 * Formats empty goals red.
	 * 
	 * @param pTable the table to format.
	 */
	public static void formatMatches(UITable pTable)
	{
		pTable.setCellFormatter(getInstance(), "formatMatches");
	}
	
	/**
	 * Configures the team columns.
	 * 
	 * @param pDataBook the team columns.
	 * @throws ModelException if columns can't be initialized.
	 */
	public static void configureTeamColumns(IDataBook pDataBook) throws ModelException
	{
		IRowDefinition rowDefinition = pDataBook.getRowDefinition();
		
		String[] columns = rowDefinition.getColumnNames();
		
		boolean isMatchTable = false;
		
		for (int i = 0; i < columns.length; i++)
		{
			String column = columns[i];
			if (column.startsWith("IMAGENAME"))
			{
				configureImageColumn(rowDefinition.getColumnDefinition(column));
			}
			else if (column.equals("FINISHED"))
			{
				rowDefinition.getColumnDefinition(column).getDataType().setCellEditor(PackungUtil.YESNO_EDITOR);
			}
			else if (column.equals("TEMP"))
			{
				rowDefinition.getColumnDefinition(column).setLabel("");
				rowDefinition.getColumnDefinition(column).setWidth(10);
				rowDefinition.getColumnDefinition(column).getDataType().setCellEditor(PackungUtil.TEMP_EDITOR);
			}
			else if (column.endsWith("GOALS1") || column.endsWith("GOALS2"))
			{
				isMatchTable = true;
				rowDefinition.getColumnDefinition(column).getDataType().setCellEditor(CENTERED_NUMBER_EDITOR);
			}
			else if (column.equals("SCORE") || column.equals("POINTS") || column.equals("RANK")
					 || column.equals("WON") || column.equals("DRAWN") || column.equals("LOST")
					 || column.equals("GOALS_FOR") || column.equals("GOALS_AGAINST") || column.equals("PLAYED"))
			{
				rowDefinition.getColumnDefinition(column).getDataType().setCellEditor(CENTERED_NUMBER_EDITOR);
			}
			else if (column.equals("MATCHBEGIN") || column.equals("RESULT") || column.equals("BET"))
			{
				rowDefinition.getColumnDefinition(column).getDataType().setCellEditor(CENTERED_TEXT_EDITOR);
			}
		}
		
		if (isMatchTable)
		{
			for (int i = 0; i < columns.length; i++)
			{
				String column = columns[i];
				if (!column.endsWith("GOALS1") && !column.endsWith("GOALS2") && !"STAT".equals(column))
				{
					rowDefinition.getColumnDefinition(column).setReadOnly(true);
				}
			}
			pDataBook.setDeleteEnabled(false);
			pDataBook.setInsertEnabled(false);
		}
	}
	
	/**
	 * Configures the column.
	 * 
	 * @param pColumnDefinition the team column.
	 * @throws ModelException if columns can't be initialized.
	 */
	public static void configureImageColumn(ColumnDefinition pColumnDefinition) throws ModelException
	{
		pColumnDefinition.setLabel("");
		pColumnDefinition.setReadOnly(true);
		pColumnDefinition.getDataType().setCellEditor(IMAGEVIEWER);
	}
	
	/**
	 * Saves the immediate after value changed.
	 * 
	 * @param pDataRowEvent the data row event
	 * @throws ModelException throws the exception with the problem.
	 */
	public void doSaveImmediate(DataRowEvent pDataRowEvent) throws ModelException
	{
		IDataBook dataBook = (IDataBook)pDataRowEvent.getChangedDataRow();
		
		try
		{
			dataBook.saveSelectedRow();
		}
		catch (ModelException pModelException)
		{
			if (!dataBook.isInserting())
			{
				// Wenn ein Feld gelscht wurde, wird nicht restored, damit man die Mglichkeit hat, mehrere Fehler zu beheben.
				if (pDataRowEvent.getChangedColumnNames().length != 1
					|| dataBook.getValue(pDataRowEvent.getChangedColumnNames()[0]) != null)
				{
					dataBook.restoreSelectedRow();
				}
				throw pModelException;
			}
		}
	}

	/**
	 * Formats the bets table.
	 *
	 * @param pDataBook the bets databook
	 * @param pDataPage the current datapage
	 * @param pDataRow the current row
	 * @param pColumnName the current column name
	 * @param pRow the current row
	 * @param pColumn the current column
	 * @return the cell format for the bets table.
	 * @throws ModelException if an error occurs during format detection
	 */
	public ICellFormat formatMatches(IDataBook pDataBook, 
	                                 IDataPage pDataPage, 
	                                 IDataRow pDataRow, 
	                                 String pColumnName, 
	                                 int pRow, 
	                                 int pColumn) throws ModelException
	{
		if ("GOALS1".equals(pColumnName) || "GOALS2".equals(pColumnName))
	    {
			if (pDataRow.getValue(pColumnName) == null)
			{
				return RED_CELL;
			}
	    }
	    
	    return null;
	}
	
	/**
	 * Checks image format and resizes the image to the given maximum size, if needed.
	 * 
	 * @param pFileHandle the file handle.
	 * @param pMaxWidth the max width.
	 * @param pMaxHeight the max height.
	 * @return the remote file handle for the image.
	 * @throws IOException if the image has wrong format or cannot be converted.
	 */
	public static IFileHandle checkAndResizeImage(IFileHandle pFileHandle, int pMaxWidth, int pMaxHeight) throws IOException
	{
		String sFormat = FileUtil.getExtension(pFileHandle.getFileName().toLowerCase());
		
		if ("png".equals(sFormat) 
			|| "jpg".equals(sFormat)
			|| "gif".equals(sFormat))
		{
			FileHandle fileHandle = new FileHandle(pFileHandle.getFileName()); 
			OutputStream stream = fileHandle.getOutputStream();
			
			ImageUtil.createScaledImage(pFileHandle.getInputStream(), 
					                    pMaxWidth, 
					                    pMaxHeight, 
					                    true, 
					                    stream, 
					                    sFormat);
			
			stream.close();
			
			return fileHandle;
		}
		else
		{
			throw new IOException("Image format '" + sFormat + "' not supported. Use 'png', 'jpg' or 'gif'!");
		}
	}
	
}	// PackungUtil
