/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 13.10.2009 - [JR] - creation
 */
package com.sibvisions.apps.packung.auth;

import java.util.Hashtable;

import javax.rad.application.ILauncher;
import javax.rad.remote.IConnectionConstants;

import com.sibvisions.apps.auth.AutoLoginAuthenticator;

/**
 * The <code>ActivationAuthenticator</code> handles the automatic activation of new users. It
 * checks if there is a parameter called <code>activationkey</code> and sends the value as
 * connection parameter to the server (used from the security manager).
 * 
 * @author Ren Jahn
 */
public class ActivationAuthenticator extends AutoLoginAuthenticator
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/** the application parameter name for the activation key. */
	public static final String PARAM_ACTIVATIONKEY = "activationkey";
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Overwritten methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public Hashtable<String, String> getCredentials(ILauncher pLauncher)
	{
		Hashtable<String, String> htCred = super.getCredentials(pLauncher);
		
		String sActivationKey = pLauncher.getParameter(PARAM_ACTIVATIONKEY);

		
		if (sActivationKey != null)
		{
			if (htCred == null)
			{
				htCred = new Hashtable<String, String>();
				
				String sValue = pLauncher.getParameter("Application.Login.application");
				
				if (sValue != null)
				{
					htCred.put(APPLICATION, sValue);
				}
			}
			
			htCred.put(IConnectionConstants.PREFIX_CLIENT + PARAM_ACTIVATIONKEY, sActivationKey);
		}
			
		return htCred;
	}
	
}	// ActivationAuthenticator
