/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 19.10.2009 - [JR] - creation
 */
package com.sibvisions.apps.packung.components;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIImage;
import javax.rad.genui.component.UIIcon;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;
import javax.rad.ui.IComponent;
import javax.rad.ui.ILayout;

import com.sibvisions.apps.projx.screens.DataSourceWorkScreen;

/**
 * The <code>AbstractStyledWorkScreen</code> is the default work-screen for all application screens. It
 * shows a "goal" as border around the content.
 * 
 * @author Ren Jahn
 */
public abstract class AbstractStyledWorkScreen extends DataSourceWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the frame content panel. */
	private UIPanel panContent;
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new instance of <code>AbstractStyledWorkScreen</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public AbstractStyledWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);
		
		initializeUI();
	}
	
	/**
	 * Initializes the UI.
	 */
	private void initializeUI()
	{
		UIFormLayout folMain = new UIFormLayout();
		folMain.setMargins(25, 25, 20, 25);
		
		super.setLayout(folMain);
		
		panContent = new UIPanel();
		
		UIIcon icoGras = new UIIcon(UIImage.getImage("/com/sibvisions/apps/packung/images/gras.png"));
		icoGras.setPreferredSize(0, 20);

		UIIcon icoGoalPostLeft = new UIIcon(UIImage.getImage("/com/sibvisions/apps/packung/images/goalpost_left.png"));
		icoGoalPostLeft.setPreferredSize(35, 0);
		icoGoalPostLeft.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		icoGoalPostLeft.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);

		UIIcon icoGoalPostTop = new UIIcon(UIImage.getImage("/com/sibvisions/apps/packung/images/goalpost_top.png"));
		icoGoalPostTop.setPreferredSize(0, 35);
		icoGoalPostTop.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		icoGoalPostTop.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);

		UIIcon icoGoalPostRight = new UIIcon(UIImage.getImage("/com/sibvisions/apps/packung/images/goalpost_right.png"));
		icoGoalPostRight.setPreferredSize(35, 0);
		icoGoalPostRight.setHorizontalAlignment(IAlignmentConstants.ALIGN_RIGHT);
		icoGoalPostRight.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);

		super.add(panContent, folMain.getConstraints(0, 0, -1, -1), -1);
		super.add(icoGras, folMain.getConstraints(folMain.getBottomMarginAnchor(), folMain.getLeftAnchor(), null, folMain.getRightAnchor()), -1);
		super.add(icoGoalPostLeft, folMain.getConstraints(folMain.getTopAnchor(), folMain.getLeftAnchor(), folMain.getBottomAnchor(), null), -1);
		super.add(icoGoalPostTop, folMain.getConstraints(folMain.getTopAnchor(), folMain.getLeftMarginAnchor(), null, folMain.getRightMarginAnchor()), -1);
		super.add(icoGoalPostRight, folMain.getConstraints(folMain.getTopAnchor(), null, folMain.getBottomAnchor(), folMain.getRightAnchor()), -1);
		
		setLayout(new UIBorderLayout());
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Overwritten methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setLayout(ILayout pLayout)
	{
		if (panContent != null)
		{
			panContent.setLayout(pLayout);
		}
		else
		{
			super.setLayout(pLayout);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public ILayout getLayout()
	{
		if (panContent != null)
		{
			return panContent.getLayout();
		}
		else
		{
			return super.getLayout();
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void add(IComponent pComponent, Object pConstraints, int pIndex)
	{
		if (panContent != null)
		{
			panContent.add(pComponent, pConstraints, pIndex);
		}
		else
		{
			super.add(pComponent, pConstraints, pIndex);
		}
	}
	
}	// AbstractStyledWorkScreen
