/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.celleditor.UITextCellEditor;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.control.UITable;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.IDataBook;
import javax.rad.model.ModelException;
import javax.rad.model.condition.Equals;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;

import com.sibvisions.apps.packung.PackungApplication;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;

/**
 * The <code>CompetitionOverviewWorkScreen</code> shows all standings in the groups and 
 * the corresponding results. 
 * 
 * @author Martin Handsteiner
 */
public class CompetitionOverviewWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** User Defaults DataBook. */
	private IDataBook		rdbUserDefaults;
	/** Stages DataBook. */
	private	RemoteDataBook	rdbCompetitionDetails	= new RemoteDataBook();
	/** Stages DataBook. */
	private	RemoteDataBook	rdbStages				= new RemoteDataBook();
	/** Groups DataBook. */
	private	RemoteDataBook	rdbGroups				= new RemoteDataBook();
	/** Groupteams DataBook. */
	private	RemoteDataBook	rdbGroupteams			= new RemoteDataBook();
	/** Matches DataBook. */
	private	RemoteDataBook	rdbMatches				= new RemoteDataBook();

	/** Matches navigation table. */
	private UITable navStages 				= new UITable();
	/** Matches navigation table. */
	private UITable navGroups 				= new UITable();
	/** Matches navigation table. */
	private UITable navGroupteams 			= new UITable();
	/** Matches navigation table. */
	private UITable navMatches 				= new UITable();

	/** the competition image. */
	private UIEditor editCompImage			= new UIEditor();
	/** the first match label. */
	private UILabel labFirstMatch			= new UILabel();
	/** the first match editor. */
	private UIEditor editFirstMatch			= new UIEditor();
	/** the first match label. */
	private UILabel labLastMatch			= new UILabel();
	/** the last match editor. */
	private UIEditor editLastMatch			= new UIEditor();
	/** the number of matches label. */
	private UILabel labMatches				= new UILabel();
	/** the number of matches editor. */
	private UIEditor editMatches			= new UIEditor();
	/** the number of matches played label. */
	private UILabel labMatchesPlayed		= new UILabel();
	/** the number of matches played editor. */
	private UIEditor editMatchesPlayed		= new UIEditor();
	/** the temporary editor. */
	private UIEditor editTemp				= new UIEditor();
	/** the number of goals label. */
	private UILabel labGoals				= new UILabel();
	/** the number of goals editor. */
	private UIEditor editGoals				= new UIEditor();
	/** the number of goals per match label. */
	private UILabel labGoalsPerMatch		= new UILabel();
	/** the number of goals per match editor. */
	private UIEditor editGoalsPerMatch		= new UIEditor();

	/** the layout for details. */
	private UIFormLayout layDetails			= new UIFormLayout();
	/** the panel for details. */
	private UIGroupPanel panDetails			= new UIGroupPanel();

	/** the layout for stages and groups. */
	private UIFormLayout layStagesGroups	= new UIFormLayout();
	/** the layout for stages and groups. */
	private UIGroupPanel panStagesGroups	= new UIGroupPanel();

	/** the layout for group teams and matches. */
	private UIFormLayout layGroupteams		= new UIFormLayout();
	/** the layout for group teams and matches. */
	private UIGroupPanel panGroupteams		= new UIGroupPanel();
	/** the layout for group teams and matches. */
	private UIFormLayout layMatches			= new UIFormLayout();
	/** the layout for group teams and matches. */
	private UIGroupPanel panMatches			= new UIGroupPanel();

	/** the layout for this component. */
	private UIFormLayout layThis			= new UIFormLayout();
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * It creates the <code>CompetitionOverviewWorkScreen</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public CompetitionOverviewWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}
		
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{		
		rdbUserDefaults = ((PackungApplication)getApplication()).getUserDefaults();
		
		rdbCompetitionDetails.setDataSource(dataSource);
		rdbCompetitionDetails.setName("competitionDetails");
		rdbCompetitionDetails.open();
		rdbCompetitionDetails.setReadOnly(true);
		
		rdbStages.setDataSource(dataSource);
		rdbStages.setName("stages");
		rdbStages.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID"},
				rdbCompetitionDetails,
				new String[] {"COMP_ID"}));
		rdbStages.open();
		rdbStages.setReadOnly(true);
		
		rdbGroups.setDataSource(dataSource);
		rdbGroups.setName("groups");
		rdbGroups.setMasterReference(new ReferenceDefinition(
				new String[] {"STAG_ID"},
				rdbStages,
				new String[] {"ID"}));
		rdbGroups.open();
		rdbGroups.setReadOnly(true);
		rdbGroups.eventAfterRowSelected().addListener(this, "doSelectMatch");
		
		rdbGroupteams.setDataSource(dataSource);
		rdbGroupteams.setName("groupteams");
		rdbGroupteams.setMasterReference(new ReferenceDefinition(
				new String[] {"GROU_ID"},
				rdbGroups,
				new String[] {"ID"}));
		rdbGroupteams.open();
		rdbGroupteams.setReadOnly(true);
		
		PackungUtil.configureTeamColumns(rdbGroupteams);
		
		rdbMatches.setDataSource(dataSource);
		rdbMatches.setName("matches");
		rdbMatches.setMasterReference(new ReferenceDefinition(
				new String[] {"STAG_ID", "MATCHES_GROU_ID"},
				rdbGroups,
				new String[] {"STAG_ID", "MATCHES_GROU_ID"}));
		rdbMatches.open();
		rdbMatches.setReadOnly(true);
		rdbMatches.eventAfterRowSelected().addListener(this, "doSelectGroup");

		PackungUtil.configureTeamColumns(rdbMatches);
	}

	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		editCompImage.setCellEditor(PackungUtil.COMPETITION_IMAGEVIEWER);
		editCompImage.setDataRow(rdbUserDefaults);
		editCompImage.setColumnName("COMP_IMAGE");

		labFirstMatch.setText("First Match:");
		editFirstMatch.setCellEditor(new UITextCellEditor());
		editFirstMatch.setMinimumSize(140, 0);
		editFirstMatch.setDataRow(rdbCompetitionDetails);
		editFirstMatch.setColumnName("FIRST_MATCH");

		labLastMatch.setText("Final Match:");
		editLastMatch.setCellEditor(new UITextCellEditor());
		editLastMatch.setDataRow(rdbCompetitionDetails);
		editLastMatch.setColumnName("LAST_MATCH");

		labMatches.setText("Matches:");
		editMatches.setDataRow(rdbCompetitionDetails);
		editMatches.setColumnName("MATCHES");

		labMatchesPlayed.setText("Matches played:");
		editMatchesPlayed.setDataRow(rdbCompetitionDetails);
		editMatchesPlayed.setColumnName("MATCHES_PLAYED");
		editTemp.setCellEditor(PackungUtil.TEMP_EDITOR);
		editTemp.setDataRow(rdbCompetitionDetails);
		editTemp.setColumnName("TEMP");

		labGoals.setText("Goals overall:");
		editGoals.setDataRow(rdbCompetitionDetails);
		editGoals.setColumnName("GOALS");

		labGoalsPerMatch.setText("Goals per Match:");
		editGoalsPerMatch.setDataRow(rdbCompetitionDetails);
		editGoalsPerMatch.setColumnName("GOALS_PER_MATCH");

		layDetails.setMargins(5, 5, 5, 5);
		layDetails.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		layDetails.setHorizontalGap(20);
		panDetails.setText("Competition Details");
		panDetails.setLayout(layDetails);
		panDetails.add(editCompImage, layDetails.getConstraints(0, 0, 0, -1));
		panDetails.add(labFirstMatch, layDetails.getConstraints(1, 0));
		layDetails.setHorizontalGap(5);
		panDetails.add(editFirstMatch, layDetails.getConstraints(2, 0));
		panDetails.add(labLastMatch, layDetails.getConstraints(1, 1));
		panDetails.add(editLastMatch, layDetails.getConstraints(2, 1));
		layDetails.setHorizontalGap(20);
		panDetails.add(labMatches, layDetails.getConstraints(3, 0));
		layDetails.setHorizontalGap(5);
		panDetails.add(editMatches, layDetails.getConstraints(4, 0));
		panDetails.add(labMatchesPlayed, layDetails.getConstraints(3, 1));
		panDetails.add(editMatchesPlayed, layDetails.getConstraints(4, 1));
		panDetails.add(editTemp, layDetails.getConstraints(5, 1));
		panDetails.add(labGoals, layDetails.getConstraints(3, 2));
		panDetails.add(editGoals, layDetails.getConstraints(4, 2));
		panDetails.add(labGoalsPerMatch, layDetails.getConstraints(3, 3));
		panDetails.add(editGoalsPerMatch, layDetails.getConstraints(4, 3));
		
		navStages.setDataBook(rdbStages);
		navStages.setMinimumSize(100, 0);
		navStages.setTableHeaderVisible(false);
		navStages.setShowGridLines(false);

		navGroups.setDataBook(rdbGroups);
		navGroups.setTableHeaderVisible(false);
		navGroups.setShowGridLines(false);

		layStagesGroups.setMargins(5, 5, 5, 5);
		layStagesGroups.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);
		layStagesGroups.setVerticalGap(20);
		panStagesGroups.setText("Stages & Groups");
		panStagesGroups.setLayout(layStagesGroups);
		panStagesGroups.add(navStages, layStagesGroups.getConstraints(0, 0));
		panStagesGroups.add(navGroups, layStagesGroups.getConstraints(0, 1, 0, -1));
		
		navGroupteams.setDataBook(rdbGroupteams);
		navGroupteams.setShowGridLines(false);

		layGroupteams.setMargins(5, 5, 5, 5);
		panGroupteams.setText("Teams in Group");
		panGroupteams.setLayout(layGroupteams);
		panGroupteams.add(navGroupteams, layGroupteams.getConstraints(0, 0, -1, -1));
		
		navMatches.setDataBook(rdbMatches);
		navMatches.setShowGridLines(false);
		navMatches.setMinimumSize(0, 190);
		PackungUtil.formatMatches(navMatches);

		layMatches.setMargins(5, 5, 5, 5);
		panMatches.setText("Matches in Group");
		panMatches.setLayout(layMatches);
		panMatches.add(navMatches, layMatches.getConstraints(0, 0, -1, -1));
		
		layThis.setMargins(0, 0, 0, 0);
		setMinimumSize(635, 0);
		setLayout(layThis);
		add(panDetails, layThis.getConstraints(0, 0, -1, 0));
		add(panStagesGroups, layThis.getConstraints(0, 1, 0, -1));
		add(panGroupteams, layThis.getConstraints(1, 1, -1, 1));
		add(panMatches, layThis.getConstraints(1, 2, -1, -1));
	}
		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	
	/**
	 * Searches for the given team.
	 * 
	 * @param pStagId the stage id.
	 * @param pGroupId the group id.
	 * @param pTeamId the team id.
	 * 
	 * @throws ModelException if an exception occurs.
	 */
	public void search(Object pStagId, Object pGroupId, Object pTeamId) throws ModelException
	{
		int index = rdbStages.searchNext(new Equals("ID", pStagId));
		if (index < 0)
		{
			return;
		}
		rdbStages.setSelectedRow(index);
			
		index = rdbGroups.searchNext(new Equals("ID", pGroupId));
		if (index < 0)
		{
			return;
		}
		rdbGroups.setSelectedRow(index);

		index = rdbGroupteams.searchNext(new Equals("TEAM_ID", pGroupId));
		if (index >= 0)
		{
			return;
		}
		rdbGroupteams.setSelectedRow(index);
	}
	
	/**
	 * Selects the correct group for the current match, if it is not in group stage.
	 * 
	 * @throws Exception if the selection fails.
	 */
	public void doSelectGroup() throws Exception
	{
		if (rdbMatches.getValue("MATCHES_GROU_ID") == null)
		{
			int index = rdbGroups.searchNext(new Equals("ID", rdbMatches.getValue("GROU_ID")));
			if (index >= 0)
			{
				rdbGroups.setSelectedRow(index);
			}
		}
	}
		
	/**
	 * Selects the correct group for the current match, if it is not in group stage.
	 * 
	 * @throws Exception if the selection fails.
	 */
	public void doSelectMatch() throws Exception
	{
		if (rdbGroups.getValue("MATCHES_GROU_ID") == null)
		{
			int index = rdbMatches.searchNext(new Equals("GROU_ID", rdbGroups.getValue("ID")));
			if (index >= 0)
			{
				rdbMatches.setSelectedRow(index);
			}
		}
	}
		
}	// CompetitionOverviewWorkScreen
