/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import java.math.BigDecimal;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIImage;
import javax.rad.genui.celleditor.UILinkedCellEditor;
import javax.rad.genui.component.UIButton;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.container.UITabsetPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.io.IFileHandle;
import javax.rad.model.reference.ColumnMapping;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;

import com.sibvisions.apps.components.NavigationTable;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;
import com.sibvisions.util.ArrayUtil;
import com.sibvisions.util.type.FileUtil;

/**
 * The <code>CompetitionsWorkScreen</code> manages the competitions, stages, groups, 
 * teams in the group. In this screen can be organized all game days and matches for the 
 * competitions. 
 * 
 * @author Martin Handsteiner
 */
public class CompetitionsWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** Competitions DataBook. */
	private	RemoteDataBook	rdbCompetitions			= new RemoteDataBook();
	/** Stages DataBook. */
	private	RemoteDataBook	rdbStages				= new RemoteDataBook();
	/** Groups DataBook. */
	private	RemoteDataBook	rdbGroups				= new RemoteDataBook();
	/** Groupteams DataBook. */
	private	RemoteDataBook	rdbGroupteams			= new RemoteDataBook();
	/** Matches DataBook. */
	private	RemoteDataBook	rdbMatches				= new RemoteDataBook();
	/** Match places DataBook. */
	private	RemoteDataBook	rdbMatchplaces			= new RemoteDataBook();
	/** Match days DataBook. */
	private	RemoteDataBook	rdbMatchdays			= new RemoteDataBook();
	/** CompetitionTypes Lookup DataBook. */
	private	RemoteDataBook	rdbCompetitionTypesLook	= new RemoteDataBook();
	/** Teams Lookup DataBook. */
	private	RemoteDataBook	rdbTeamsLook			= new RemoteDataBook();
	/** Fillin groups Lookup DataBook. */
	private	RemoteDataBook	rdbFillinRanksLook 		= new RemoteDataBook();
	/** Fillin groups Lookup DataBook. */
	private	RemoteDataBook	rdbFillinGroupsLook 	= new RemoteDataBook();
	/** Match day Lookup DataBook. */
	private	RemoteDataBook	rdbMatchdaysLook 		= new RemoteDataBook();
	/** Match places Lookup DataBook. */
	private	RemoteDataBook	rdbMatchplacesLook 		= new RemoteDataBook();
	/** Groupteams Lookup DataBook. */
	private	RemoteDataBook	rdbGroupteamsLook		= new RemoteDataBook();

	/** Competitions navigation table. */
	private NavigationTable navCompetitions = new NavigationTable();
	/** Stages navigation table. */
	private NavigationTable navStages 		= new NavigationTable();
	/** Groups navigation table. */
	private NavigationTable navGroups 		= new NavigationTable();
	/** Groupteams navigation table. */
	private NavigationTable navGroupteams 	= new NavigationTable();
	/** Matches navigation table. */
	private NavigationTable navMatches 		= new NavigationTable();
	/** Match days navigation table. */
	private NavigationTable navMatchdays 	= new NavigationTable();
	/** Match places navigation table. */
	private NavigationTable navMatchplaces 	= new NavigationTable();

	/** top area - "competition details" */
	/** Competition name label. */
	private UILabel			lCompetitionName		= new UILabel();
	/** Competition name editor. */
	private UIEditor		edCompetitionName		= new UIEditor();
	/** Competition typ label. */
	private UILabel			lCompetitionType		= new UILabel();
	/** Competition typ editor. */
	private UIEditor		edCompetitionType		= new UIEditor();
	/** Competition deadline label. */
	private UILabel			lDeadline				= new UILabel();
	/** Competition deadline editor. */
	private UIEditor		edDeadline				= new UIEditor();
	/** Competition deadline unit label. */
	private UILabel			lDeadlineUnit			= new UILabel();
	/** Competition active label. */
	private UILabel			lActive					= new UILabel();
	/** Competition active editor. */
	private UIEditor		edActive				= new UIEditor();
	/** Competitions image. */
	private UIEditor		edImage					= new UIEditor();
	/** Competitions image upload Button. */
	private UIButton		butImageUpload			= new UIButton();

	/** Layout for Competitions. */
	private	UIFormLayout	layCompetitions			= new UIFormLayout();
	/** Competitions panel. */
	private	UIGroupPanel	groupCompetitions		= new UIGroupPanel();

	/** Layout for Stages, groups, teams. */
	private	UIFormLayout	layStagesGroups				= new UIFormLayout();
	/** Stages, groups, teams panel. */
	private	UIGroupPanel	groupStagesGroups			= new UIGroupPanel();
	/** Layout for matches. */
	private	UIFormLayout	layGroupteams				= new UIFormLayout();
	/** Matches panel. */
	private	UIGroupPanel	groupGroupteams				= new UIGroupPanel();
	/** Layout for matches. */
	private	UIFormLayout	layMatches					= new UIFormLayout();
	/** Matches panel. */
	private	UIGroupPanel	groupMatches				= new UIGroupPanel();
	/** Layout for Stages, groups, teams, matches. */
	private	UIFormLayout	layStagesGroupTeamsMatches	= new UIFormLayout();
	/** Stages, groups, teams, matches panel. */
	private	UIPanel			panStagesGroupTeamsMatches	= new UIPanel();
	
	/** Layout for Stages, groups, teams, matches. */
	private	UIFormLayout	layMatchplaces			= new UIFormLayout();
	/** Match places group panel. */
	private	UIGroupPanel	groupMatchplaces		= new UIGroupPanel();
	/** Layout for Stages, groups, teams, matches. */
	private	UIFormLayout	layMatchdays			= new UIFormLayout();
	/** Match days group panel. */
	private	UIGroupPanel	groupMatchdays 			= new UIGroupPanel();
	/** Layout for Stages, groups, teams, matches. */
	private	UIFormLayout	layMatchdayMatchplaces	= new UIFormLayout();
	/** Match days/places panel. */
	private	UIPanel			panMatchdayMatchplaces	= new UIPanel();
	
	/** Competitions tabset panel. */
	private UITabsetPanel 	tabCompetitions			= new UITabsetPanel();
	
	/** Layout for this component. */
	private	UIFormLayout	layThis					= new UIFormLayout();

	/** Stores the default group team column names. */
	private String[] defaultGroupTeamColumnNames;
	/** Stores the default group team column names. */
	private String[] teamGroupTeamColumnNames;
	/** Stores the default group team column names. */
	private String[] groupGroupTeamColumnNames;
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * It creates the <code>CompetitionsWorkScreen</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public CompetitionsWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}
		
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{
		// Lookup Definitions
		rdbCompetitionTypesLook.setDataSource(dataSource);
		rdbCompetitionTypesLook.setName("competitionTypesLook");
		rdbCompetitionTypesLook.open();
		UILinkedCellEditor competitionTypesLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"COTY_ID", "COMPETITION_TYPE"},
				rdbCompetitionTypesLook,
				new String[] {"ID", "COMPETITION_TYPE"}));
		
		rdbTeamsLook.setDataSource(dataSource);
		rdbTeamsLook.setName("teamsLook");
		rdbTeamsLook.open();
		PackungUtil.configureTeamColumns(rdbTeamsLook);
		UILinkedCellEditor teamsLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"TEAM_ID", "IMAGENAME", "TEAM"},
				rdbTeamsLook,
				new String[] {"ID", "IMAGENAME", "TEAM"}));
		teamsLook.setSearchColumnMapping(new ColumnMapping(new String[] {"COMP_ID"}));
		
		rdbFillinGroupsLook.setDataSource(dataSource);
		rdbFillinGroupsLook.setName("fillinGroupsLook");
		rdbFillinGroupsLook.open();
		UILinkedCellEditor fillinGroupsLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"FILLIN_GROU_ID", "FILLIN_GROUPNAME"},
				rdbFillinGroupsLook,
				new String[] {"ID", "GROUPNAME"}));
		fillinGroupsLook.setSearchColumnMapping(new ColumnMapping(new String[] {"COMP_ID", "STAG_ID"}));
		
		rdbFillinRanksLook.setDataSource(dataSource);
		rdbFillinRanksLook.setName("fillinRanksLook");
		rdbFillinRanksLook.open();
		UILinkedCellEditor fillinRanksLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"FILLIN_RANK_ID", "FILLIN_RANK"},
				rdbFillinRanksLook,
				new String[] {"ID", "RANK"}));
		fillinRanksLook.setSearchColumnMapping(new ColumnMapping(new String[] {"FILLIN_GROU_ID"}));
		
		rdbMatchdaysLook.setDataSource(dataSource);
		rdbMatchdaysLook.setName("matchdays");
		rdbMatchdaysLook.open();
		UILinkedCellEditor matchdaysLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"MADA_ID", "MATCHDAY"},
				rdbMatchdaysLook,
				new String[] {"ID", "MATCHDAY"}));
		matchdaysLook.setSearchColumnMapping(new ColumnMapping(new String[] {"COMP_ID"}));

		rdbMatchplacesLook.setDataSource(dataSource);
		rdbMatchplacesLook.setName("matchplaces");
		rdbMatchplacesLook.open();
		UILinkedCellEditor matchplacesLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"MAPL_ID", "PLACE"},
				rdbMatchplacesLook,
				new String[] {"ID", "PLACE"}));
		matchplacesLook.setSearchColumnMapping(new ColumnMapping(new String[] {"COMP_ID"}));
		
		rdbGroupteamsLook.setDataSource(dataSource);
		rdbGroupteamsLook.setName("groupteams");
		rdbGroupteamsLook.open();
		rdbGroupteamsLook.getRowDefinition().setTableColumnNames(new String[] {"TEAM_PLACEHOLDER"});
		UILinkedCellEditor groupteam1Look = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"GRTE1_ID", "TEAM_1"},
				rdbGroupteamsLook,
				new String[] {"ID", "TEAM_PLACEHOLDER"}));
		groupteam1Look.setSearchColumnMapping(new ColumnMapping(new String[] {"COMP_ID", "STAG_ID", "GROU_ID"}));
		UILinkedCellEditor groupteam2Look = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"GRTE2_ID", "TEAM_2"},
				rdbGroupteamsLook,
				new String[] {"ID", "TEAM_PLACEHOLDER"}));
		groupteam2Look.setSearchColumnMapping(new ColumnMapping(new String[] {"COMP_ID", "STAG_ID", "GROU_ID"}));
		
		rdbCompetitions.setDataSource(dataSource);
		rdbCompetitions.setName("competitions");
		rdbCompetitions.open();
		rdbCompetitions.eventBeforeInserting().addListener(this, "doInsertingFirstRow");

		rdbCompetitions.getRowDefinition().setTableColumnNames(new String[] {"COMPETITION"});
		rdbCompetitions.getRowDefinition().getColumnDefinition("COMPETITION_TYPE").getDataType().setCellEditor(competitionTypesLook);
		
		// init stages DataBook
		rdbStages.setDataSource(dataSource);
		rdbStages.setName("stages");
		rdbStages.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID"},
                rdbCompetitions,
                new String[] {"ID"}));				
		rdbStages.open();
		rdbStages.eventAfterRowSelected().addListener(this, "doHandleGroupTeamColumns");
		rdbStages.eventAfterInserted().addListener(this, "doHandleGroupTeamColumns");

		// init groups DataBook
		rdbGroups.setDataSource(dataSource);
		rdbGroups.setName("groups");
		rdbGroups.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID", "STAG_ID"},
				rdbStages,
                new String[] {"COMP_ID", "ID"}));		
		rdbGroups.open();		
		
		// init groupteams DataBook
		rdbGroupteams.setDataSource(dataSource);
		rdbGroupteams.setName("groupteams");
		rdbGroupteams.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID", "STAG_ID", "GROU_ID"},
				rdbGroups,
                new String[] {"COMP_ID", "STAG_ID", "ID"}));		
		rdbGroupteams.open();
		rdbGroupteams.eventValuesChanged().addListener(this, "doEnableFillinRank");
		rdbGroupteams.eventAfterRowSelected().addListener(this, "doEnableFillinRank");
		
		defaultGroupTeamColumnNames = rdbGroupteams.getRowDefinition().getTableColumnNames();
		teamGroupTeamColumnNames = ArrayUtil.remove(ArrayUtil.remove(defaultGroupTeamColumnNames, "FILLIN_GROUPNAME"), "FILLIN_RANK");
		groupGroupTeamColumnNames = ArrayUtil.remove(ArrayUtil.remove(defaultGroupTeamColumnNames, "IMAGENAME"), "TEAM");
		
		PackungUtil.configureTeamColumns(rdbGroupteams);
		rdbGroupteams.getRowDefinition().getColumnDefinition("TEAM").getDataType().setCellEditor(teamsLook);
		rdbGroupteams.getRowDefinition().getColumnDefinition("TEAM").setWidth(250);
		rdbGroupteams.getRowDefinition().getColumnDefinition("FILLIN_RANK").getDataType().setCellEditor(fillinRanksLook);
		rdbGroupteams.getRowDefinition().getColumnDefinition("FILLIN_GROUPNAME").getDataType().setCellEditor(fillinGroupsLook);
		
		// init matches DataBook
		rdbMatches.setDataSource(dataSource);
		rdbMatches.setName("matches");
		rdbMatches.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID", "STAG_ID", "GROU_ID"},
				rdbGroups,
                new String[] {"COMP_ID", "STAG_ID", "ID"}));		
		rdbMatches.open();
		rdbMatches.eventBeforeInserting().addListener(this, "doCheckMatchdaysMatchplaces");
		rdbMatches.getRowDefinition().getColumnDefinition("MATCHDAY").getDataType().setCellEditor(matchdaysLook);
		rdbMatches.getRowDefinition().getColumnDefinition("PLACE").getDataType().setCellEditor(matchplacesLook);
		rdbMatches.getRowDefinition().getColumnDefinition("TEAM_1").getDataType().setCellEditor(groupteam1Look);
		rdbMatches.getRowDefinition().getColumnDefinition("TEAM_2").getDataType().setCellEditor(groupteam2Look);
		
		// init match places DataBook
		rdbMatchplaces.setDataSource(dataSource);
		rdbMatchplaces.setName("matchplaces");
		rdbMatchplaces.setMasterReference(new ReferenceDefinition(new String[] {"COMP_ID"},
				rdbCompetitions,
                new String[] {"ID"}));		
		rdbMatchplaces.open();

		// init match days DataBook
		rdbMatchdays.setDataSource(dataSource);
		rdbMatchdays.setName("matchdays");
		rdbMatchdays.setMasterReference(new ReferenceDefinition(new String[] {"COMP_ID"},
				rdbCompetitions,
                new String[] {"ID"}));		
		rdbMatchdays.open();
	}
    
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		// Top Area Competitions
		navCompetitions.setDataBook(rdbCompetitions);
		navCompetitions.setShowGridLines(false);
	    navCompetitions.setPreferredSize(250, 80);
		
		lCompetitionName.setText("Competition:");
		edCompetitionName.setDataRow(rdbCompetitions);
		edCompetitionName.setColumnName("COMPETITION");
		
		lCompetitionType.setText("Typ:");
		edCompetitionType.setDataRow(rdbCompetitions);
		edCompetitionType.setColumnName("COMPETITION_TYPE");

		lDeadline.setText("Deadline:");
		edDeadline.setMaximumSize(30, Integer.MAX_VALUE);
		edDeadline.setDataRow(rdbCompetitions);
		edDeadline.setColumnName("DEADLINE");
		lDeadlineUnit.setText("min");
		
		lActive.setText("Competition is active");
		edActive.setCellEditor(PackungUtil.YESNO_EDITOR);
		edActive.setDataRow(rdbCompetitions);
		edActive.setColumnName("ACTIVE");
		
		edImage.setCellEditor(PackungUtil.COMPETITION_IMAGEVIEWER);
		edImage.setDataRow(rdbCompetitions);
		edImage.setColumnName("IMAGE");
		
		butImageUpload.setText("Set Image");
		butImageUpload.setImage(UIImage.getImage(UIImage.IMPORT_SMALL));
		butImageUpload.eventAction().addListener(this, "doUpload");
		
		layCompetitions.setMargins(5, 5, 5, 5);
		groupCompetitions.setText("Competitions");
	    groupCompetitions.setLayout(layCompetitions);
	    groupCompetitions.add(navCompetitions, layCompetitions.getConstraints(0, 0, 0, -1));
	    groupCompetitions.add(lCompetitionName,   layCompetitions.getConstraints(1, 0));
	    groupCompetitions.add(edCompetitionName,  layCompetitions.getConstraints(2, 0, -2, 0));
	    groupCompetitions.add(lCompetitionType,  layCompetitions.getConstraints(1, 1));
	    groupCompetitions.add(edCompetitionType, layCompetitions.getConstraints(2, 1, -2, 1));
	    groupCompetitions.add(lDeadline,  layCompetitions.getConstraints(1, -2));
	    groupCompetitions.add(edDeadline, layCompetitions.getConstraints(2, -2, 3, -2));
	    groupCompetitions.add(lDeadlineUnit,  layCompetitions.getConstraints(4, -2));
	    groupCompetitions.add(lActive,  layCompetitions.getConstraints(1, -1));
	    groupCompetitions.add(edActive, layCompetitions.getConstraints(2, -1));
	    groupCompetitions.add(edImage, layCompetitions.getConstraints(-1, 0, -1, -1));
	    groupCompetitions.add(butImageUpload, layCompetitions.getConstraints(-2, -1, -2, -1));

		// Center Area Tab
		// First Tab Stages, Groups, Groupteams, Matches. 
		navStages.setDataBook(rdbStages);
		navStages.setShowGridLines(false);
		navStages.setSortOrderColumnName("SORTORDER_ID");
		navStages.setPreferredSize(150, 139);
		
		navGroups.setDataBook(rdbGroups);
		navGroups.setShowGridLines(false);
		navGroups.setPreferredSize(150, 152);
		
		navGroupteams.setDataBook(rdbGroupteams);
		navGroupteams.setShowGridLines(false);
		navGroupteams.setPreferredSize(250, 107);

		layGroupteams.setMargins(5, 5, 5, 5);
	    groupGroupteams.setText("Teams in Group");
	    groupGroupteams.setLayout(layGroupteams);
	    groupGroupteams.add(navGroupteams, layGroupteams.getConstraints(0, 0, -1, -1));
	    
		navMatches.setDataBook(rdbMatches);
		navMatches.setShowGridLines(false);
		navMatches.setPreferredSize(250, 148);

		layMatches.setMargins(5, 5, 5, 5);
	    groupMatches.setText("Matches in Group");
	    groupMatches.setLayout(layMatches);
	    groupMatches.add(navMatches, layMatches.getConstraints(0, 0, -1, -1));
	    
		layStagesGroups.setMargins(5, 5, 5, 5);
		layStagesGroups.setVerticalGap(15);
	    groupStagesGroups.setText("Stages & Groups");
		groupStagesGroups.setLayout(layStagesGroups);
	    groupStagesGroups.add(navStages, layStagesGroups.getConstraints(0, 0));
	    groupStagesGroups.add(navGroups, layStagesGroups.getConstraints(0, 1, 0, -1));

	    layStagesGroupTeamsMatches.setMargins(5, 5, 3, 5);
	    panStagesGroupTeamsMatches.setLayout(layStagesGroupTeamsMatches);
	    panStagesGroupTeamsMatches.add(groupStagesGroups, layStagesGroupTeamsMatches.getConstraints(0, 0, 0, -1));
	    panStagesGroupTeamsMatches.add(groupGroupteams, layStagesGroupTeamsMatches.getConstraints(1, 0, -1, 0));
	    panStagesGroupTeamsMatches.add(groupMatches, layStagesGroupTeamsMatches.getConstraints(1, 1, -1, -1));
	    
		// Second Tab Matchplaces, Matchdays. 
	    navMatchplaces.setDataBook(rdbMatchplaces);
	    navMatchplaces.setShowGridLines(false);
	    navMatchplaces.setPreferredSize(400, 150);
	    
	    layMatchplaces.setMargins(5, 5, 5, 5);
	    groupMatchplaces.setText("Match Places");
	    groupMatchplaces.setLayout(layMatchplaces);
	    groupMatchplaces.add(navMatchplaces, layMatchplaces.getConstraints(0, 0, -1, -1));

	    navMatchdays.setDataBook(rdbMatchdays);
	    navMatchdays.setShowGridLines(false);
	    navMatchdays.setPreferredSize(200, 150);

	    layMatchdays.setMargins(5, 5, 5, 5);
		groupMatchdays.setText("Match Days");
	    groupMatchdays.setLayout(layMatchdays);
		groupMatchdays.add(navMatchdays, layMatchdays.getConstraints(0, 0, -1, -1));

		layMatchdayMatchplaces.setMargins(5, 5, 3, 5);
		panMatchdayMatchplaces.setLayout(layMatchdayMatchplaces);
		panMatchdayMatchplaces.add(groupMatchplaces, layMatchdayMatchplaces.getConstraints(0, 0, 0, -1));
		panMatchdayMatchplaces.add(groupMatchdays, layMatchdayMatchplaces.getConstraints(1, 0, -1, -1));

		// tab
		tabCompetitions.add(panStagesGroupTeamsMatches, "Stages, Groups, Teams, Matches");
		tabCompetitions.add(panMatchdayMatchplaces, "Matchplaces, Matchdays");


		layThis.setMargins(0, 0, 0, 0);
	    layThis.setVerticalGap(5);
		setMinimumSize(685, 0);
	    setLayout(layThis);
		add(groupCompetitions, layThis.getConstraints(0, 0, -1, 0));
	    add(tabCompetitions, layThis.getConstraints(0, 1, -1, -1));
	}
		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	
	/**
	 * Saves the image to the competition.
	 * 
	 * @param pFileHandle the file.
	 * @throws Throwable if an error occures.
	 */
	public void storeFile(IFileHandle pFileHandle) throws Throwable
	{
		pFileHandle = PackungUtil.checkAndResizeImage(pFileHandle, 110, 110);
		
		rdbCompetitions.setValue("IMAGE", FileUtil.getContent(pFileHandle.getInputStream()));
		
		rdbCompetitions.saveSelectedRow();
	}

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		
	/**
	 * Choose the image to upload for the competition image.
	 * 
	 * @throws Throwable if the File chooser dialog coulnd't opened
	 */
	public void doUpload() throws Throwable
	{
		getApplication().getLauncher().getFileHandle(this, "storeFile", "Competition Image");
	}
	
	/**
	 * Deselects, to guarantee inserting in the first row.
	 * 
	 * @throws Exception if the selection fails.
	 */
	public void doInsertingFirstRow() throws Exception
	{
		rdbCompetitions.setSelectedRow(-1);
	}

	/**
	 * Enables the fillin rank only, if the fillin group is already choosen.
	 * 
	 * @throws Exception if the enabling fails.
	 */
	public void doEnableFillinRank() throws Exception
	{
		rdbGroupteams.getRowDefinition().getColumnDefinition("FILLIN_RANK").setReadOnly(rdbGroupteams.getValue("FILLIN_GROU_ID") == null);
	}
	
	/**
	 * Checks, if Matchdays and Matchplaces exists.
	 * 
	 * @throws Exception if the enabling fails.
	 */
	public void doCheckMatchdaysMatchplaces() throws Exception
	{
		if (rdbMatchdays.getDataRow(0) == null || rdbMatchplaces.getDataRow(0) == null)
		{
			throw new Exception("You have to fill in match days and match places first!");
		}
	}
	
	/**
	 * Sets the correct table columns on the RowDefinition of the groupteams.
	 * 
	 * @throws Exception if setting the table columns fails.
	 */
	public void doHandleGroupTeamColumns() throws Exception
	{
		Object value = rdbStages.getValue("SORTORDER_ID");
		
		if (value == null)
		{
			rdbGroupteams.getRowDefinition().setTableColumnNames(defaultGroupTeamColumnNames);
		}
		else if (BigDecimal.valueOf(0).equals(value))
		{
			rdbGroupteams.getRowDefinition().setTableColumnNames(teamGroupTeamColumnNames);
		}
		else
		{
			rdbGroupteams.getRowDefinition().setTableColumnNames(groupGroupTeamColumnNames);
		}
	}
	
}	// CompetitionsWorkScreen
