/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 12.10.2009 - [JR] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIImage;
import javax.rad.genui.UIInsets;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.container.UISplitPanel;
import javax.rad.genui.container.UITabsetPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.ModelException;
import javax.rad.model.IDataBook.WriteBackIsolationLevel;
import javax.rad.model.event.DataRowEvent;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;

import com.sibvisions.apps.components.NavigationTable;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;
import com.sibvisions.util.ArrayUtil;

/**
 * The <code>ConfigurationWorkScreen</code> allows the configuration of application 
 * properties and global settings.
 * 
 * @author Ren Jahn
 */
public class ConfigurationWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the settings databook. */
	private RemoteDataBook rdbGlobalConfig = new RemoteDataBook();;
	
	/** the language databook. */
	private RemoteDataBook rdbLanguages 	= new RemoteDataBook();;
	
	/** the infotexts databook. */
	private RemoteDataBook rdbInfoTexts 	= new RemoteDataBook();;
	
	/** the rules editor. */
	private UIEditor edRules 				= new UIEditor();
	
	/** the disclaimer editor. */
	private UIEditor edDisclaimer 			= new UIEditor();
	
	/** the notice editor. */
	private UIEditor edNotice 				= new UIEditor();

	/** the registration email editor. */
	private UIEditor edRegEmail 			= new UIEditor();

	/** the lost password email editor. */
	private UIEditor edLostEmail 			= new UIEditor();

	/** the bet group email editor. */
	private UIEditor edGroupEmail 			= new UIEditor();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new instance of <code>RegistrationWorkScreen</code> for the application and a
	 * specific connectoin.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public ConfigurationWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);
		
		initializeModel();
		initializeUI();
	}
	
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{
		rdbGlobalConfig.setName("config");
		rdbGlobalConfig.setDataSource(dataSource);
		rdbGlobalConfig.setInsertEnabled(false);
		rdbGlobalConfig.setDeleteEnabled(false);
		rdbGlobalConfig.open();
		
		rdbGlobalConfig.getRowDefinition().getColumnDefinition("VALUE").setWidth(300);
		rdbGlobalConfig.getRowDefinition().getColumnDefinition("PARAMETER").setReadOnly(true);
		rdbGlobalConfig.getRowDefinition().getColumnDefinition("DESCRIPTION").setReadOnly(true);
		
		rdbGlobalConfig.getRowDefinition().setTableColumnNames(ArrayUtil.remove(rdbGlobalConfig.getRowDefinition().getTableColumnNames(), "SORT"));
		
		rdbLanguages.setName("languages");
		rdbLanguages.setDataSource(dataSource);
		rdbLanguages.setWritebackIsolationLevel(WriteBackIsolationLevel.DATASOURCE);
		rdbLanguages.open();
		
		rdbLanguages.eventAfterInserting().addListener(this, "doNewLanguage");
		rdbLanguages.eventAfterDeleting().addListener(this, "doDeleteLanguage");
		rdbLanguages.eventValuesChanged().addListener(this, "doUpdateLanguage");
		
		rdbLanguages.getRowDefinition().setTableColumnNames(new String[] {"LANGUAGE", "CODE", "STANDARD"});
		rdbLanguages.getRowDefinition().getColumnDefinition("STANDARD").getDataType().setCellEditor(PackungUtil.YESNO_EDITOR);
		
		rdbInfoTexts.setName("infoTexts");
		rdbInfoTexts.setMasterReference(new ReferenceDefinition(new String[] {"LANG_ID"}, rdbLanguages, new String[] {"ID"}));
		rdbInfoTexts.setDataSource(dataSource);
		rdbInfoTexts.setDeleteCascade(true);
		rdbInfoTexts.open();
		
		rdbInfoTexts.getRowDefinition().getColumnDefinition("RULES").getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);
		rdbInfoTexts.getRowDefinition().getColumnDefinition("DISCLAIMER").getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);
		rdbInfoTexts.getRowDefinition().getColumnDefinition("NOTICE").getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);
		rdbInfoTexts.getRowDefinition().getColumnDefinition("REGEMAIL").getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);
		rdbInfoTexts.getRowDefinition().getColumnDefinition("PWDEMAIL").getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);
		rdbInfoTexts.getRowDefinition().getColumnDefinition("GROUPEMAIL").getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);
	}
	
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		//init components/controls

		NavigationTable navGlobalConfig = new NavigationTable();
		navGlobalConfig.setDataBook(rdbGlobalConfig);
		
		NavigationTable navLanguagesk = new NavigationTable();
		navLanguagesk.setDataBook(rdbLanguages);

		UIPanel panGlobalConfig = new UIPanel();
		
		UISplitPanel spanRegister = new UISplitPanel();
		spanRegister.setDividerPosition(200);
		
		UIFormLayout folRegisterDetail = new UIFormLayout();
		folRegisterDetail.setMargins(new UIInsets(5, 3, 0, 0));
		
		UIPanel panRegisterDetail = new UIPanel();
		panRegisterDetail.setLayout(folRegisterDetail);
		
		UITabsetPanel tabRegisterDetail = new UITabsetPanel();
		
		edDisclaimer.setDataRow(rdbInfoTexts);
		edDisclaimer.setColumnName("DISCLAIMER");
		
		edRules.setDataRow(rdbInfoTexts);
		edRules.setColumnName("RULES");
		
		edNotice.setDataRow(rdbInfoTexts);
		edNotice.setColumnName("NOTICE");
		
		edRegEmail.setDataRow(rdbInfoTexts);
		edRegEmail.setColumnName("REGEMAIL");
		
		edLostEmail.setDataRow(rdbInfoTexts);
		edLostEmail.setColumnName("PWDEMAIL");
		
		edGroupEmail.setDataRow(rdbInfoTexts);
		edGroupEmail.setColumnName("GROUPEMAIL");

		// build UI

		panGlobalConfig.setLayout(new UIBorderLayout());
		panGlobalConfig.add(navGlobalConfig);
		
		tabRegisterDetail.add(edRules, "Rules");
		tabRegisterDetail.add(edDisclaimer, "Disclaimer");
		tabRegisterDetail.add(edNotice, "Notice");
		tabRegisterDetail.add(edRegEmail, "Activation Email");
		tabRegisterDetail.add(edLostEmail, "Lost Password Email");
		tabRegisterDetail.add(edGroupEmail, "Group invitation Email");
		
		panRegisterDetail.add(tabRegisterDetail, folRegisterDetail.getConstraints(0, 0, -1, -1));
		
		spanRegister.add(navLanguagesk);
		spanRegister.add(panRegisterDetail);
		
		UITabsetPanel tabMain = new UITabsetPanel();
		tabMain.add(panGlobalConfig, "Global Parameters");
		tabMain.add(spanRegister, "Registration");
		
		tabMain.setIconAt(0, UIImage.getImage("/com/sibvisions/apps/packung/images/16x16/settings.png"));
		tabMain.setIconAt(1, UIImage.getImage(UIImage.REGISTER_SMALL));
		
		setLayout(new UIBorderLayout());
		add(tabMain);
		
		setPreferredSize(860, 500);
	}	
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Creates a new language and also creates a new infotext.
	 * 
	 * @throws ModelException if the creation of a new infotext fails 
	 */
	public void doNewLanguage() throws ModelException
	{
		rdbInfoTexts.insert(false);
	}
	
	/**
	 * Deletes the selected language.
	 * 
	 * @throws ModelException if the creation of a new infotext fails 
	 */
	public void doDeleteLanguage() throws ModelException
	{
		//important, because of the write-back isolation level!!
		rdbLanguages.saveSelectedRow();
	}
	
	/**
	 * Updates the standard language flag for all languages expect the 
	 * selected one.
	 * 
	 * @param pEvent the row event which triggered the value change
	 * @throws ModelException if a problem occurs during update the standard language
	 */
	public void doUpdateLanguage(DataRowEvent pEvent) throws ModelException
	{
		if ("STANDARD".equals(pEvent.getChangedColumnNames()[0]))
		{
			int iSelected = rdbLanguages.getSelectedRow();
			for (int i = 0; i < rdbLanguages.getRowCount(); i++)
			{
				rdbLanguages.setSelectedRow(i);
				
				rdbLanguages.setValue("STANDARD", "N");
			}
			
			rdbLanguages.setSelectedRow(iSelected);
			
			pEvent.getChangedDataRow().setValue("STANDARD", "Y");
			
			rdbLanguages.saveAllRows();
		}
	}
	
}	// ConfigurationWorkScreen
