/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 20.10.2009 - [JR] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIDimension;
import javax.rad.genui.UIImage;
import javax.rad.genui.component.UIButton;
import javax.rad.genui.container.UITabsetPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.condition.Equals;
import javax.rad.remote.AbstractConnection;

import com.sibvisions.apps.packung.PackungApplication;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;

/**
 * The <code>InformationWorkScreen</code> shows the disclaimer, rules, notice and 
 * other application information.
 * 
 * @author Ren Jahn
 */
public class InformationWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the language table. */
	private RemoteDataBook rdbLangInfos = new RemoteDataBook();
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new instance of <code>InformationWorkScreen</code> for the application and a
	 * specific connectoin.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public InformationWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);
		
		initializeModel();
		initializeUI();
	}

	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{
		rdbLangInfos.setDataSource(dataSource);
		rdbLangInfos.setName("languageInfos");
		rdbLangInfos.setReadOnly(true);
		rdbLangInfos.open();
		
		rdbLangInfos.setFilter(new Equals("CODE", ((PackungApplication)getApplication()).getLanguage()));
	}
	
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		//init components/controls

		UIFormLayout folMain = new UIFormLayout();
		folMain.setMargins(5, 5, 5, 5);
		
		UITabsetPanel tabMain = new UITabsetPanel();
		
		UIEditor edRules = new UIEditor();
		edRules.setCellEditor(PackungUtil.MULTILINE_EDITOR);
		edRules.setDataRow(rdbLangInfos);
		edRules.setColumnName("RULES");

		UIEditor edDisclaimer = new UIEditor();
		edDisclaimer.setCellEditor(PackungUtil.MULTILINE_EDITOR);
		edDisclaimer.setDataRow(rdbLangInfos);
		edDisclaimer.setColumnName("DISCLAIMER");

		UIEditor edNotice = new UIEditor();
		edNotice.setCellEditor(PackungUtil.MULTILINE_EDITOR);
		edNotice.setDataRow(rdbLangInfos);
		edNotice.setColumnName("NOTICE");
		
		UIButton butOK = new UIButton("OK");
		butOK.eventAction().addListener(this, "doClose");
		butOK.setPreferredSize(new UIDimension(100, 25));
		butOK.setDefaultButton(true);
		butOK.setImage(UIImage.getImage(UIImage.OK_SMALL));

		//build UI
		
		setLayout(folMain);

		tabMain.add(edRules, "Rules");
		tabMain.add(edDisclaimer, "Disclaimer");
		tabMain.add(edNotice, "Notice");

		add(butOK, folMain.getConstraints(null, 
				                          folMain.createAnchor(folMain.getLeftMarginAnchor()), 
				                          folMain.getBottomAnchor(), 
				                          folMain.createAnchor(folMain.getRightMarginAnchor())));
		add(tabMain, folMain.getConstraints(folMain.getTopMarginAnchor(),
				                            folMain.getLeftMarginAnchor(), 
				                            folMain.createAnchor(folMain.getConstraints(butOK).getTopAnchor(), -15),
				                            folMain.getRightMarginAnchor()));
		
		setPreferredSize(new UIDimension(700, 450));
	}	
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Closes the screen.
	 */
	public void doClose()
	{
		getApplication().close(this);
	}

}	// InformationWorkScreen
