/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 15.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import java.sql.Timestamp;
import java.util.Date;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.application.genui.RemoteWorkScreenApplication;
import javax.rad.genui.UIImage;
import javax.rad.genui.celleditor.UILinkedCellEditor;
import javax.rad.genui.celleditor.UITextCellEditor;
import javax.rad.genui.component.UIButton;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.container.UITabsetPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.control.UITable;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.io.IFileHandle;
import javax.rad.model.ColumnDefinition;
import javax.rad.model.IDataBook;
import javax.rad.model.RowDefinition;
import javax.rad.model.SortDefinition;
import javax.rad.model.event.DataRowEvent;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;
import javax.rad.ui.layout.IFormLayout.IConstraints;

import com.sibvisions.apps.components.NavigationTable;
import com.sibvisions.apps.packung.PackungApplication;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.mem.DataRow;
import com.sibvisions.rad.model.remote.RemoteDataBook;
import com.sibvisions.util.type.FileUtil;

/**
 * The <code>MyBetsWorkScreen</code> shows an overview and all bet results of the current user.
 * In that screen can also new bets be placed.   
 * <br>
 * 
 * @author Martin Handsteiner
 */
public class MyBetsWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** User Defaults DataBook. */
	private IDataBook		rdbUserDefaults;
	/** UserDetails DataBook. */
	private	RemoteDataBook	rdbUserDetails		= new RemoteDataBook();
	/** Match days History DataBook. */
	private	RemoteDataBook	rdbMatchdaysHistory	= new RemoteDataBook();
	/** Bets History DataBook. */
	private	RemoteDataBook	rdbBetsHistory		= new RemoteDataBook();
	/** Match days DataBook. */
	private	RemoteDataBook	rdbMatchdays		= new RemoteDataBook();
	/** Bets DataBook. */
	private	RemoteDataBook	rdbBets				= new RemoteDataBook();
	/** Top bets DataBook. */
	private	RemoteDataBook	rdbTopbets			= new RemoteDataBook();
	/** Group teams lookup DataBook. */
	private	RemoteDataBook	rdbGroupteamsLook	= new RemoteDataBook();
	/** Bet groups DataBook. */
	private	RemoteDataBook	rdbBetgroups		= new RemoteDataBook();
	/** DataRow for special Editors. */
	private	DataRow			drBetgroupsData;

	/** Match days History table. */
	private UITable			tabMatchdaysHistory	= new UITable();
	/** Bets  History table. */
	private UITable			tabBetsHistory		= new UITable();
	/** Match days table. */
	private UITable			tabMatchdays		= new UITable();
	/** Bets table. */
	private UITable			tabBets				= new UITable();
	/** Bet groups table. */
	private NavigationTable	tabBetgroups		= new NavigationTable();

	/** the competition image. */
	private UIEditor 		editCompImage		= new UIEditor();
	/** Image editor. */
	private UIEditor		editImage			= new UIEditor();
	/** User name editor. */
	private UIEditor		editUsername		= new UIEditor();
	/** Number of players label. */
	private UILabel			labPlayers			= new UILabel();
	/** Number of players editor. */
	private UIEditor		editPlayers			= new UIEditor();
	/** Overall rank Label. */
	private UILabel			labRank				= new UILabel();
	/** Overall rank editor. */
	private UIEditor		editRank			= new UIEditor();
	/** Score label. */
	private UILabel			labScore			= new UILabel();
	/** Score editor. */
	private UIEditor		editScore			= new UIEditor();
	/** the temporary editor. */
	private UIEditor		editTemp			= new UIEditor();
	/** User image upload button. */
	private UIButton		butImageUpload		= new UIButton();
	/** Score label. */
	private UILabel			labDeadline			= new UILabel();
	/** Score editor. */
	private UIEditor		editDeadline		= new UIEditor();
	/** Score label. */
	private UILabel			labTopbet			= new UILabel();
	/** Score editor. */
	private UIEditor		editTopbetImagename	= new UIEditor();
	/** Score editor. */
	private UIEditor		editTopbetTeam		= new UIEditor();
	/** User profile button. */
	private UIButton		butUserProfile		= new UIButton();

	/** the editor for groupname. */
	private UIEditor 		editGroupname		= new UIEditor();
	/** the editor for matchbegin. */
	private UIEditor 		editMatchbegin		= new UIEditor();
	/** the editor for image1. */
	private UIEditor 		editImagename1		= new UIEditor();
	/** the editor for team1. */
	private UIEditor 		editTeam1			= new UIEditor();
	/** the label for colon. */
	private UILabel 		labColon			= new UILabel();
	/** the editor for image2. */
	private UIEditor 		editImagename2		= new UIEditor();
	/** the editor for team2. */
	private UIEditor 		editTeam2			= new UIEditor();
	/** the label for goals1. */
	private UIEditor 		editGoals1			= new UIEditor();
	/** the editor for goals2. */
	private UIEditor 		editGoals2			= new UIEditor();

	/** the label for colon. */
	private UILabel 		labCode				= new UILabel();
	/** the editor for image2. */
	private UIEditor 		editCode			= new UIEditor();
	/** User image join button. */
	private UIButton		butJoin				= new UIButton();
	/** the label for colon. */
	private UILabel 		labEmails			= new UILabel();
	/** the editor for image2. */
	private UIEditor 		editEmails			= new UIEditor();
	/** User image invite button. */
	private UIButton		butInvite			= new UIButton();
	
	/** The panel for the user image. */
	private UIPanel panImage = new UIPanel();
	/** the layout for details. */
	private UIFormLayout 	layDetails			= new UIFormLayout();
	/** the panel for details. */
	private UIGroupPanel 	panDetails			= new UIGroupPanel();

	/** the layout for bet history. */
	private UIFormLayout 	layBetHistory		= new UIFormLayout();
	/** the panel for bet history. */
	private UIGroupPanel	panBetHistory		= new UIGroupPanel();

	/** the layout for bet history. */
	private UIFormLayout 	layBets				= new UIFormLayout();
	/** the panel for bet history. */
	private UIPanel			panBets				= new UIPanel();

	/** the layout for bet history. */
	private UIFormLayout 	layBetgroups		= new UIFormLayout();
	/** the panel for bet history. */
	private UIPanel			panBetgroups		= new UIPanel();

	/** the layout for bet history. */
	private UIFormLayout 	layJoin				= new UIFormLayout();
	/** the panel for bet history. */
	private UIGroupPanel	panJoin				= new UIGroupPanel();

	/** the layout for bet history. */
	private UIFormLayout 	layInvite			= new UIFormLayout();
	/** the panel for bet history. */
	private UIGroupPanel	panInvite			= new UIGroupPanel();

	/** Tabset panel for the bets. */
	private	UITabsetPanel	tpBets				= new UITabsetPanel();	
	
	/** the layout for this component. */
	private UIFormLayout folThis				= new UIFormLayout();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * It creates the <code>MyBetsWorkScreen</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public MyBetsWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}
		
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{
		rdbUserDefaults = ((PackungApplication)getApplication()).getUserDefaults();
		
		rdbUserDetails.setDataSource(dataSource);
		rdbUserDetails.setName("userDetails");
		rdbUserDetails.open();		
		rdbUserDetails.setReadOnly(true);
		
		rdbMatchdaysHistory.setDataSource(dataSource);
		rdbMatchdaysHistory.setName("matchdaysHistory");
		rdbMatchdaysHistory.open();		
		rdbMatchdaysHistory.setReadOnly(true);
		PackungUtil.configureTeamColumns(rdbMatchdaysHistory);

		rdbBetsHistory.setDataSource(dataSource);
		rdbBetsHistory.setName("betsHistory");
		rdbBetsHistory.setMasterReference(new ReferenceDefinition(
				new String[] {"MADA_ID"},
				rdbMatchdaysHistory,
				new String[] {"ID"}));
		rdbBetsHistory.open();		
		rdbBetsHistory.setReadOnly(true);
		PackungUtil.configureTeamColumns(rdbBetsHistory);

		rdbMatchdays.setDataSource(dataSource);
		rdbMatchdays.setName("matchdays");
		rdbMatchdays.open();		
		rdbMatchdays.setWritebackEnabled(false);
		rdbMatchdays.setReadOnly(true);
		PackungUtil.configureTeamColumns(rdbMatchdays);
		rdbMatchdays.getRowDefinition().getColumnDefinition("BET_OK").getDataType().setCellEditor(PackungUtil.POS_NEG_EDITOR);

		rdbBets.setDataSource(dataSource);
		rdbBets.setName("bets");
		rdbBets.setMasterReference(new ReferenceDefinition(
				new String[] {"MADA_ID"},
				rdbMatchdays,
				new String[] {"ID"}));
		rdbBets.open();
		rdbBets.eventValuesChanged().addListener(this, "doCheckDeadlineAndShowStatistic");
		PackungUtil.configureTeamColumns(rdbBets);
		rdbBets.getRowDefinition().getColumnDefinition("STAT").getDataType().setCellEditor(PackungUtil.INFO_EDITOR);

		rdbGroupteamsLook.setDataSource(dataSource);
		rdbGroupteamsLook.setName("groupteamsLook");
		rdbGroupteamsLook.open();
		PackungUtil.configureTeamColumns(rdbGroupteamsLook);
		UILinkedCellEditor groupteamsLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"GRTE_ID", "IMAGENAME", "TEAM"},
				rdbGroupteamsLook,
				new String[] {"ID", "IMAGENAME", "TEAM"}));
		
		rdbTopbets.setDataSource(dataSource);
		rdbTopbets.setName("topbets");
		rdbTopbets.open();		
		rdbTopbets.eventValuesChanged().addListener(this, "doCheckDeadline");
		rdbTopbets.eventAfterRowSelected().addListener(this, "doCheckDeadline");
		rdbTopbets.getRowDefinition().getColumnDefinition("TEAM").getDataType().setCellEditor(groupteamsLook);
		rdbTopbets.getRowDefinition().getColumnDefinition("BET_DEADLINE").getDataType().setCellEditor(new UITextCellEditor());
		rdbTopbets.getRowDefinition().getColumnDefinition("BET_DEADLINE").setReadOnly(true);
		
		rdbBetgroups.setDataSource(dataSource);
		rdbBetgroups.setName("betgroups");
		rdbBetgroups.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID", "USER_ID"},
				rdbUserDetails,
				new String[] {"COMP_ID", "ID"}));
		rdbBetgroups.open();
		rdbBetgroups.eventAfterRowSelected().addListener(this, "doSetReadOnly");
		rdbBetgroups.eventAfterInserted().addListener(this, "doSetReadOnly");
		PackungUtil.saveImmediate(rdbBetgroups);
		rdbBetgroups.getRowDefinition().getColumnDefinition("MASTER").setReadOnly(true);
		
		ColumnDefinition cdEMails  = new ColumnDefinition("EMAILS");
		cdEMails.getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);
		RowDefinition rdBetgroupsData = new RowDefinition();
		rdBetgroupsData.addColumnDefinition(new ColumnDefinition("CODE"));
		rdBetgroupsData.addColumnDefinition(cdEMails);
		drBetgroupsData = new DataRow(rdBetgroupsData);
	}
    
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		editCompImage.setCellEditor(PackungUtil.COMPETITION_IMAGEVIEWER);
		editCompImage.setDataRow(rdbUserDefaults);
		editCompImage.setColumnName("COMP_IMAGE");

		editUsername.setDataRow(rdbUserDetails);
		editUsername.setColumnName("USERNAME");
   
		labPlayers.setText("Total Players:");
		editPlayers.setDataRow(rdbUserDetails);
		editPlayers.setColumnName("PLAYERS");

		labRank.setText("Overall Rank:");
		editRank.setDataRow(rdbUserDetails);
		editRank.setColumnName("RANK");

		labScore.setText("Score:");
		editScore.setDataRow(rdbUserDetails);
		editScore.setColumnName("SCORE");

		editTemp.setCellEditor(PackungUtil.TEMP_EDITOR);
		editTemp.setDataRow(rdbUserDetails);
		editTemp.setColumnName("TEMP");

		editImage.setCellEditor(PackungUtil.USER_IMAGEVIEWER);
		editImage.setDataRow(rdbUserDefaults);
		editImage.setColumnName("IMAGE");

		butImageUpload.setText("Change");
		butImageUpload.setMargins(1, 2, 1, 2);
		butImageUpload.setImage(UIImage.getImage(UIImage.IMPORT_SMALL));
		butImageUpload.eventAction().addListener(this, "doUpload");
		
		panImage.setLayout(new UIBorderLayout());
		panImage.add(editImage, UIBorderLayout.CENTER);
		panImage.add(butImageUpload, UIBorderLayout.SOUTH);

		butUserProfile.setText("Profile");
		butUserProfile.setImage(UIImage.getImage(PackungUtil.PROFILE));
		butUserProfile.eventAction().addListener(this, "doUserProfile");
		
		labDeadline.setText("Deadline for Topbet:");
		labDeadline.setMaximumSize(0, Integer.MAX_VALUE);
		labDeadline.setVisible(false);
		editDeadline.setDataRow(rdbTopbets);
		editDeadline.setColumnName("BET_DEADLINE");
		editDeadline.setMaximumSize(0, Integer.MAX_VALUE);
		editDeadline.setVisible(false);

		labTopbet.setText("Topbet: Winner of Competition");
		labTopbet.setMaximumSize(0, Integer.MAX_VALUE);
		editTopbetImagename.setCellEditor(PackungUtil.IMAGEVIEWER);
		editTopbetImagename.setDataRow(rdbTopbets);
		editTopbetImagename.setColumnName("IMAGENAME");
		editTopbetTeam.setDataRow(rdbTopbets);
		editTopbetTeam.setColumnName("TEAM");
		editTopbetTeam.setMinimumSize(160, 0);

		layDetails.setMargins(5, 5, 5, 5);
		layDetails.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		layDetails.setHorizontalGap(10);
		panDetails.setText("User Details");
		panDetails.setLayout(layDetails);
		panDetails.add(editCompImage, layDetails.getConstraints(0, 0, 0, -1));
		panDetails.add(panImage, layDetails.getConstraints(1, 0, 1, -1));
		layDetails.setHorizontalGap(20);
		panDetails.add(editUsername, layDetails.getConstraints(2, 0));
		layDetails.setHorizontalGap(5);
		IConstraints cons = layDetails.getConstraints(3, 0);
		cons.setTopAnchor(layDetails.createAnchor(cons.getTopAnchor()));
		cons.setBottomAnchor(layDetails.createAnchor(cons.getBottomAnchor()));
		cons.setRightAnchor(layDetails.createAnchor(cons.getLeftAnchor()));
		panDetails.add(butUserProfile, cons);
		panDetails.add(labPlayers, layDetails.getConstraints(2, 1));
		panDetails.add(editPlayers, layDetails.getConstraints(3, 1));
		panDetails.add(labRank, layDetails.getConstraints(2, 2));
		panDetails.add(editRank, layDetails.getConstraints(3, 2));
		panDetails.add(labScore, layDetails.getConstraints(2, 3));
		panDetails.add(editScore, layDetails.getConstraints(3, 3));
		panDetails.add(editTemp, layDetails.getConstraints(4, 3));
		panDetails.add(labDeadline, layDetails.getConstraints(-2, 0, -1, 0));
		panDetails.add(editDeadline, layDetails.getConstraints(-2, 1, -1, 1));
		panDetails.add(labTopbet, layDetails.getConstraints(-2, 2, -1, 2));
		panDetails.add(editTopbetImagename, layDetails.getConstraints(-2, 3));
		panDetails.add(editTopbetTeam, layDetails.getConstraints(-1, 3));

		tabMatchdaysHistory.setDataBook(rdbMatchdaysHistory);
		tabMatchdaysHistory.setShowGridLines(false);
		tabMatchdaysHistory.setPreferredSize(135, 107);
		
		tabBetsHistory.setDataBook(rdbBetsHistory);
		tabBetsHistory.setShowGridLines(false);
		tabBetsHistory.setPreferredSize(550, 107);
		
		layBetHistory.setMargins(5, 5, 5, 5);
		layBetHistory.setHorizontalGap(10);
		panBetHistory.setText("Bet History");
		panBetHistory.setLayout(layBetHistory);
		panBetHistory.add(tabMatchdaysHistory, layBetHistory.getConstraints(0, 0, 0, -1));
		panBetHistory.add(tabBetsHistory, layBetHistory.getConstraints(1, 0, -1, -1));

		editGroupname.setDataRow(rdbBets);
		editGroupname.setColumnName("GROUPNAME");
		editMatchbegin.setMaximumSize(0, Integer.MAX_VALUE);
		editMatchbegin.setDataRow(rdbBets);
		editMatchbegin.setColumnName("MATCHBEGIN");

		editTeam1.setDataRow(rdbBets);
		editTeam1.setColumnName("TEAM1");
		editImagename1.setDataRow(rdbBets);
		editImagename1.setColumnName("IMAGENAME1");
		editGoals1.setDataRow(rdbBets);
		editGoals1.setColumnName("BET_GOALS1");
		
		labColon.setText(":");
		editTeam2.setDataRow(rdbBets);
		editTeam2.setColumnName("TEAM2");
		editTeam2.setMinimumSize(150, 0);
		editImagename2.setDataRow(rdbBets);
		editImagename2.setColumnName("IMAGENAME2");
		editGoals2.setDataRow(rdbBets);
		editGoals2.setColumnName("BET_GOALS2");

		tabMatchdays.setDataBook(rdbMatchdays);
		tabMatchdays.setShowGridLines(false);
		tabMatchdays.setPreferredSize(135, 190);
		
		tabBets.setDataBook(rdbBets);
		tabBets.setShowGridLines(false);
		tabBets.setPreferredSize(250, 107);
		PackungUtil.formatMatches(tabBets);

		layBets.setMargins(7, 7, 7, 7);
		layBets.setHorizontalGap(10);
		layBets.setVerticalGap(10);
		panBets.setLayout(layBets);
		panBets.add(tabMatchdays, layBets.getConstraints(0, 0, 0, -1));
		panBets.add(editGroupname, layBets.getConstraints(1, 0));
		layBets.setHorizontalGap(5);
		panBets.add(editMatchbegin, layBets.getConstraints(3, 0, 7, 0));
		panBets.add(editTeam1, layBets.getConstraints(1, 1, 2, 1));
		panBets.add(editImagename1, layBets.getConstraints(3, 1));
		panBets.add(editGoals1, layBets.getConstraints(4, 1));
		panBets.add(labColon, layBets.getConstraints(5, 1));
		panBets.add(editGoals2, layBets.getConstraints(6, 1));
		panBets.add(editImagename2, layBets.getConstraints(7, 1));
		panBets.add(editTeam2, layBets.getConstraints(8, 1));
		panBets.add(tabBets, layBets.getConstraints(1, 2, -1, -1));

		tabBetgroups.setDataBook(rdbBetgroups);
		tabBetgroups.setShowGridLines(false);
		tabBetgroups.setMaximumSize(Integer.MAX_VALUE, 0);
		
		labCode.setText("Code:");
		labCode.setMinimumSize(40, 0);
		editCode.setDataRow(drBetgroupsData);
		editCode.setColumnName("CODE");
		butJoin.setText("Join");
		butJoin.setImage(UIImage.getImage(UIImage.IMPORT_SMALL));
		butJoin.eventAction().addListener(this, "doJoinBetgroup");
		butJoin.setMinimumSize(100, 0);

		layJoin.setMargins(5, 5, 5, 5);
		layJoin.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		panJoin.setText("Join to bet group");
		panJoin.setLayout(layJoin);
		panJoin.add(labCode, layJoin.getConstraints(0, 0));
		panJoin.add(editCode, layJoin.getConstraints(1, 0));
			
		IConstraints conEdit = layJoin.getConstraints(2, 0);
		conEdit.setTopAnchor(layJoin.createAnchor(conEdit.getTopAnchor()));
		conEdit.setBottomAnchor(layJoin.createAnchor(conEdit.getBottomAnchor()));
		
		panJoin.add(butJoin, conEdit); 
		
		labEmails.setText("Emails:");
		labEmails.setMinimumSize(40, 0);
		editEmails.setDataRow(drBetgroupsData);
		editEmails.setColumnName("EMAILS");
		editEmails.setMinimumSize(200, 0);
		butInvite.setText("Invite");
		butInvite.setImage(UIImage.getImage(UIImage.IMPORT_SMALL));
		butInvite.eventAction().addListener(this, "doInviteBetgroup");
		butInvite.setMinimumSize(100, 0);

		layInvite.setMargins(5, 5, 5, 5);
		panInvite.setText("Invite to bet group");
		panInvite.setLayout(layInvite);
		panInvite.add(labEmails, layInvite.getConstraints(0, 0));
		panInvite.add(editEmails, layInvite.getConstraints(1, 0, -2, -1));
		panInvite.add(butInvite, layInvite.getConstraints(-1, -1));
		
		layBetgroups.setMargins(5, 5, 5, 5);
		layBetgroups.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		panBetgroups.setLayout(layBetgroups);
		panBetgroups.add(tabBetgroups, layBetgroups.getConstraints(0, 0, 0, -1));
		panBetgroups.add(panJoin, layBetgroups.getConstraints(1, 0, -1, 0));
		panBetgroups.add(panInvite, layBetgroups.getConstraints(1, 1, -1, -1));
		
		tpBets.add(panBets, "Bets");
		tpBets.add(panBetgroups, "Bet Groups");
		
		folThis.setMargins(0, 0, 0, 0);
		setMinimumSize(650, 0);
		setLayout(folThis);
		add(panDetails, folThis.getConstraints(0, 0, -1, 0));
		add(panBetHistory, folThis.getConstraints(0, 1, -1, 1));
		add(tpBets, folThis.getConstraints(0, 2, -1, -1));
	}
		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	
	/**
	 * Saves the image to the user.
	 * 
	 * @param pFileHandle the file.
	 * @throws Throwable if an error occures.
	 */
	public void storeFile(IFileHandle pFileHandle) throws Throwable
	{
		pFileHandle = PackungUtil.checkAndResizeImage(pFileHandle, 90, 90);
		
		rdbUserDefaults.setValue("IMAGE", FileUtil.getContent(pFileHandle.getInputStream()));
		
		rdbUserDefaults.saveSelectedRow();
	}

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Choose the image to upload for the user image.
	 * 
	 * @throws Throwable if the File chooser dialog coulnd't opened
	 */
	public void doUpload() throws Throwable
	{
		getApplication().getLauncher().getFileHandle(this, "storeFile", "User Image");
	}
	
	/**
	 * Shows the user profile dialog.
	 * 
	 * @throws Throwable if the user profile dialog coulnd't opened
	 */
	public void doUserProfile() throws Throwable
	{
		getApplication().openContent(this, "User Profile", true, new UserProfileDialog(getApplication(), ((PackungApplication)getApplication()).getConnection()));
	}
	
	/**
	 * Join private bet group.
	 * 
	 * @throws Throwable if the E-Mail couldn't sent
	 */
	public void doJoinBetgroup() throws Throwable
	{
		String sGroupname = (String)getConnection().call("registration", "joinPrivateGroup",	
				rdbUserDetails.getValue("ID"), drBetgroupsData.getValue("CODE"));
		
		if (sGroupname == null)
		{
			showError(this, "Unknown code!");
		}
		else
		{
			showInformation(this, "Welcome to bet group '" + sGroupname + "'");
			
			drBetgroupsData.setValue("CODE", null);
			rdbBetgroups.reload();
		}
	}
	
	/**
	 * Invite to private bet group through an E-Mail with the code.
	 * 
	 * @throws Throwable if the E-Mail couldn't sent
	 */
	public void doInviteBetgroup() throws Throwable
	{
		getConnection().call("registration", "sendPrivateGroup", 
				             rdbBetgroups.getValue("ID"), ((PackungApplication)getApplication()).getLanguage(), drBetgroupsData.getValue("EMAILS"));

		showInformation(this, "Invitation successful sent");
		
		drBetgroupsData.setValue("EMAILS", null);
	}

	/**
	 * Checks the deadline, and opens the statistic work screen.
	 * Stores the bets and reloads the match days to correct the state column.
	 * 
	 * @param pDataRowEvent	the DatRowEvent
	 * @throws Throwable if the store or reload failed.
	 */
	public void doCheckDeadlineAndShowStatistic(DataRowEvent pDataRowEvent) throws Throwable
	{
		if (pDataRowEvent.isChangedColumnName("STAT"))
		{
			CompetitionOverviewWorkScreen wsCompOver = (CompetitionOverviewWorkScreen)((RemoteWorkScreenApplication)getApplication())
			                                                                          .openWorkScreen(CompetitionOverviewWorkScreen.class.getName());
			wsCompOver.search(rdbBets.getValue("STAG_ID"), rdbBets.getValue("GROU_ID"), null);
		}
		else
		{
			Timestamp tsDeadline = (Timestamp)rdbBets.getValue("BET_DEADLINE_ID");
			
			if ("Y".equals(rdbBets.getValue("FINISHED_ID")) || tsDeadline.getTime() < new Date().getTime())
			{
				rdbBets.restoreSelectedRow();
				rdbMatchdays.reload();
				rdbMatchdaysHistory.reload();
				
				throw new Exception("Bet change not possible! - Deadline ended.");
			}
			else
			{
				rdbBets.saveSelectedRow();
				if (rdbMatchdays.compareTo(rdbMatchdaysHistory, new SortDefinition("ID")) == 0)
				{
					rdbBetsHistory.reload();
				}
				
				// Reduce Reloads.
				rdbMatchdays.setReadOnly(false);
				if (rdbBets.getValue("BET_GOALS1") != null && rdbBets.getValue("BET_GOALS2") != null)
				{
					rdbMatchdays.setValue("BET_OK", "Y");
				}
				else
				{
					rdbMatchdays.setValue("BET_OK", "N");
				}
				rdbMatchdays.setReadOnly(true);
			}
		}
	}

	/**
	 * Checks the deadline of the top bet.
	 * 
	 * @throws Throwable if the store or reload failed.
	 */
	public void doCheckDeadline() throws Throwable
	{
		Timestamp tsDeadline = (Timestamp)rdbTopbets.getValue("BET_DEADLINE");
		
		if (tsDeadline == null || tsDeadline.getTime() < new Date().getTime())
		{
			rdbTopbets.restoreSelectedRow();
			
			rdbTopbets.setReadOnly(true);
			
			labDeadline.setVisible(false);
			editDeadline.setVisible(false);
		}
		else
		{
			rdbTopbets.saveSelectedRow();
			
			rdbTopbets.setReadOnly(false);
			
			labDeadline.setVisible(true);
			editDeadline.setVisible(true);
		}
	}

	/**
	 * Sets the bet groups read only, if the user is not master.
	 * 
	 * @throws Throwable if setting read only failes.
	 */
	public void doSetReadOnly() throws Throwable
	{
		if (rdbBetgroups.getSelectedRow() >= 0 && rdbBetgroups.getValue("MASTER") == null)
		{
			rdbBetgroups.setValue("MASTER", rdbUserDetails.getValue("USERNAME"));
		}
		
		boolean isMyBetgroup = rdbBetgroups.getSelectedRow() < 0
			|| rdbUserDetails.getValue("USERNAME").equals(rdbBetgroups.getValue("MASTER"));
		
		rdbBetgroups.setUpdateEnabled(isMyBetgroup);
		rdbBetgroups.setDeleteEnabled(isMyBetgroup);

		panInvite.setVisible(rdbBetgroups.getSelectedRow() >= 0 && isMyBetgroup && !rdbBetgroups.isInserting());
	}
	
} 	// MyBetsWorkScreen
