/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 08.10.2009 - [JR] - creation
 */
package com.sibvisions.apps.packung.screens;

import java.util.Hashtable;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIColor;
import javax.rad.genui.UIDimension;
import javax.rad.genui.UIImage;
import javax.rad.genui.UIInsets;
import javax.rad.genui.celleditor.UILinkedCellEditor;
import javax.rad.genui.component.UIButton;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.component.UIPasswordField;
import javax.rad.genui.component.UITextField;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.container.UITabsetPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.IDataRow;
import javax.rad.model.condition.Equals;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;
import javax.rad.ui.component.ITextField;
import javax.rad.ui.event.Key;
import javax.rad.ui.event.UIKeyEvent;
import javax.rad.ui.layout.IFormLayout.IAnchor;
import javax.rad.ui.layout.IFormLayout.IConstraints;

import com.sibvisions.apps.packung.PackungApplication;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.auth.ActivationAuthenticator;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;

/**
 * The <code>RegistrationWorkScreen</code> enables the application to register new users.
 * 
 * @author Ren Jahn
 */
public class RegistrationWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the input fiels for the activation key. */
	private UITextField tfActivationKey 		= new UITextField();
	
	/** the input field for the first name. */ 
	private UITextField tfFirstName 			= new UITextField();

	/** the input field for the last name. */ 
	private UITextField tfLastName 				= new UITextField();

	/** the input field for the username. */ 
	private UITextField tfUserName 				= new UITextField();
	
	/** the input field for the password. */ 
	private UIPasswordField tfPassword 			= new UIPasswordField();

	/** the input field for the password confirmation. */ 
	private UIPasswordField tfPasswordConfirm 	= new UIPasswordField();

	/** the input field for the email address. */ 
	private UITextField tfEmail 				= new UITextField();
	
	/** the input field for the language. */ 
	private UIEditor edLanguage 				= new UIEditor();
	
	/** the disclaimer editor. */
	private UIEditor edDisclaimer 				= new UIEditor();
	
	/** the rules editor. */
	private UIEditor edRules					= new UIEditor();
	
	/** the language table. */
	private RemoteDataBook rdbLangInfos 		= new RemoteDataBook();
	
	/** the language selection row. */
	private IDataRow rowLanguage;
	
	/** the label for user errors. */
	private UILabel lblErrUser 					= new UILabel();
	
	/** the label for password errors. */
	private UILabel lblErrPwd 					= new UILabel();

	/** the label for password confirm errors. */
	private UILabel lblErrPwdConfirm 			= new UILabel();

	/** the label for email errors. */
	private UILabel lblErrEmail 				= new UILabel();
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new instance of <code>RegistrationWorkScreen</code> for the application and a
	 * specific connectoin.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public RegistrationWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);
		
		initializeModel();
		initializeUI();
	}

	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{		
		rdbLangInfos.setDataSource(dataSource);
		rdbLangInfos.setName("languageInfos");
		rdbLangInfos.setWritebackEnabled(false);
		rdbLangInfos.setMemFilter(true);
		rdbLangInfos.open();
		
		rdbLangInfos.getRowDefinition().setTableColumnNames(new String[] {"LANGUAGE"});
		rdbLangInfos.getRowDefinition().getColumnDefinition("DISCLAIMER").getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);
		rdbLangInfos.getRowDefinition().getColumnDefinition("RULES").getDataType().setCellEditor(PackungUtil.MULTILINE_EDITOR);

		rdbLangInfos.setFilter(new Equals("CODE", ((PackungApplication)getApplication()).getLanguage()));
		
		//use standard language instead of preferred language, if not available!
		if (rdbLangInfos.getRowCount() == 0)
		{
			rdbLangInfos.setFilter(new Equals("STANDARD", "Y"));
		}

		rowLanguage = rdbLangInfos.createDataRow(null);
		
		ReferenceDefinition refLanguage = new ReferenceDefinition();
		refLanguage.setColumnNames(new String[] {"ID", "LANGUAGE", "DISCLAIMER", "RULES"});
		refLanguage.setReferencedDataBook(rdbLangInfos);
		refLanguage.setReferencedColumnNames(new String[] {"ID", "LANGUAGE", "DISCLAIMER", "RULES"});

		UILinkedCellEditor lceLanguage = new UILinkedCellEditor(refLanguage);
		lceLanguage.setTableHeaderVisible(false);
		
		rdbLangInfos.getRowDefinition().getColumnDefinition("LANGUAGE").getDataType().setCellEditor(lceLanguage);
		rdbLangInfos.getRowDefinition().getColumnDefinition("DISCLAIMER").setReadOnly(true);
		rdbLangInfos.getRowDefinition().getColumnDefinition("RULES").setReadOnly(true);
	}
	
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		//init components/controls
		
		UILabel lblActivationKey = new UILabel("Activation key:");

		UIButton butActivate = new UIButton("Activate");
		butActivate.setImage(UIImage.getImage(UIImage.OK_SMALL));
		butActivate.eventAction().addListener(this, "doActivate");
		
		tfActivationKey.setColumns(30);
		
		edLanguage.setDataRow(rowLanguage);
		edLanguage.setColumnName("LANGUAGE");
		
		edDisclaimer.setDataRow(rowLanguage);
		edDisclaimer.setColumnName("DISCLAIMER");

		edRules.setDataRow(rowLanguage);
		edRules.setColumnName("RULES");
		
		UIButton butRegister = new UIButton("Register");
		butRegister.eventAction().addListener(this, "doRegister");
		butRegister.setPreferredSize(new UIDimension(110, 25));
		butRegister.setDefaultButton(true);
		butRegister.setImage(UIImage.getImage(UIImage.OK_SMALL));
		
		UIButton butCancel = new UIButton("Cancel");
		butCancel.eventAction().addListener(this, "doCancel");
		butCancel.setPreferredSize(new UIDimension(100, 25));
		butCancel.setAccelerator(Key.getKeyOnPressed(UIKeyEvent.VK_ESCAPE));
		
		UILabel lblUserReq = new UILabel("(required)");
		UILabel lblPwdReq = new UILabel("(required)");
		UILabel lblEmailReq = new UILabel("(required)");
		
		lblUserReq.setForeground(UIColor.gray);
		lblPwdReq.setForeground(UIColor.gray);
		lblEmailReq.setForeground(UIColor.gray);
		
		lblErrUser.setForeground(UIColor.red);
		lblErrPwd.setForeground(UIColor.red);
		lblErrPwdConfirm.setForeground(UIColor.red);
		lblErrEmail.setForeground(UIColor.red);
		
		hideErrors();
		
		// build UI
		
		//Activation
		UIGroupPanel gpActivation = new UIGroupPanel("Manual Activation");

		UIFormLayout folActivation = new UIFormLayout();
		folActivation.setMargins(5, 5, 5, 5);
		folActivation.setNewlineCount(2);
		folActivation.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		
		gpActivation.setLayout(folActivation);
		
		gpActivation.add(lblActivationKey);
		gpActivation.add(tfActivationKey);
		
		IConstraints con = folActivation.getConstraints(tfActivationKey);
		
		gpActivation.add(butActivate, folActivation.getConstraints(folActivation.createAnchor(con.getTopAnchor()), 
				                                                   folActivation.createAnchor(con.getRightAnchor(), folActivation.getHorizontalGap()),
				                                                   folActivation.createAnchor(con.getBottomAnchor()),
				                                                   null));
		
		
		//Registration
		UIFormLayout folMain = new UIFormLayout();
		folMain.setMargins(new UIInsets(5, 5, 5, 5));
		folMain.setNewlineCount(1);

		UIGroupPanel gpRegister = new UIGroupPanel("New User Registration");

		UIFormLayout folRegister = new UIFormLayout();
		folRegister.setNewlineCount(3);
		gpRegister.setLayout(folRegister);

		gpRegister.add(new UILabel("First name:"));
		gpRegister.add(tfFirstName);
		
		gpRegister.add(new UILabel("Last name:"), UIFormLayout.NEWLINE);
		gpRegister.add(tfLastName);

		gpRegister.add(new UILabel("Username:"), folRegister.getConstraints(0, 4, 0, 4));
		gpRegister.add(tfUserName);
		gpRegister.add(lblUserReq);
		gpRegister.add(lblErrUser, UIFormLayout.NEWLINE);
		
		gpRegister.add(new UILabel("Password:"), UIFormLayout.NEWLINE);
		gpRegister.add(tfPassword);
		gpRegister.add(lblPwdReq);
		gpRegister.add(lblErrPwd, UIFormLayout.NEWLINE);

		gpRegister.add(new UILabel("Password (confirm):"), UIFormLayout.NEWLINE);
		gpRegister.add(tfPasswordConfirm);
		gpRegister.add(lblErrPwdConfirm, UIFormLayout.NEWLINE);
		
		gpRegister.add(new UILabel("Email:"), UIFormLayout.NEWLINE);
		gpRegister.add(tfEmail);
		gpRegister.add(lblEmailReq);
		gpRegister.add(lblErrEmail, UIFormLayout.NEWLINE);

		gpRegister.add(new UILabel("Language:"), UIFormLayout.NEWLINE);
		gpRegister.add(edLanguage);
		
		UITabsetPanel tabDetails = new UITabsetPanel();
		
		tabDetails.add(edDisclaimer, "Disclaimer");
		tabDetails.add(edRules, "Rules");
		
		gpRegister.add(tabDetails, folRegister.getConstraints(0, 16, -1, -1));
		
		//change form
		IConstraints consUserName = folRegister.getConstraints(tfUserName); 
		
		folRegister.getConstraints(edLanguage).setRightAnchor(folRegister.createAnchor(consUserName.getLeftAnchor(), 100));

		IAnchor ancEmailRight = folRegister.createAnchor(consUserName.getLeftAnchor(), 200);
		IAnchor ancEmailReqLeft = folRegister.createAnchor(ancEmailRight, 4);
		
		folRegister.getConstraints(tfEmail).setRightAnchor(ancEmailRight);
		folRegister.getConstraints(lblEmailReq).setLeftAnchor(ancEmailReqLeft);
		folRegister.getConstraints(lblEmailReq).setRightAnchor(folRegister.createAnchor(ancEmailReqLeft));

		updateConstraints(folRegister, tfUserName, lblErrUser);
		updateConstraints(folRegister, tfPassword, lblErrPwd);
		updateConstraints(folRegister, tfPasswordConfirm, lblErrPwdConfirm);
		updateConstraints(folRegister, tfEmail, lblErrEmail);
		
				
		setLayout(folMain);

		add(gpRegister, folMain.getConstraints(0, 2, -1, -2));

		folMain.setVerticalGap(8);

		add(butCancel, folMain.getConstraints(0, -1, 0, -1));
		add(butRegister, folMain.getConstraints(-1, -1, -1, -1));

		folMain.setVerticalGap(15);

		add(gpActivation, folMain.getConstraints(0, 0, -1, 1));
		
		setPreferredSize(new UIDimension(600, 580));
	}	
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Updates the constraints of an error label to be aligned under a textfield.
	 * 
	 * @param pLayout the form layout where the label was added
	 * @param pField the textfield as alignment base
	 * @param pLabel the label where the constraints should be changed
	 */
	private void updateConstraints(UIFormLayout pLayout, ITextField pField, UILabel pLabel)
	{
		IAnchor ancRight = pLayout.getRightMarginAnchor();
		
		IConstraints consTf = pLayout.getConstraints(pField);
		
		IConstraints conLbl = pLayout.getConstraints(pLabel);
		
		conLbl.setLeftAnchor(consTf.getLeftAnchor());
		conLbl.setRightAnchor(ancRight);
	}
	
	/**
	 * Hides all error messages.
	 */
	private void hideErrors()
	{
		lblErrUser.setVisible(false);
		lblErrPwd.setVisible(false);
		lblErrPwdConfirm.setVisible(false);
		lblErrEmail.setVisible(false);
	}
	
	/**
	 * Shows an error in a specific label.
	 * 
	 * @param pLabel the label where the error should be shown
	 * @param pText the error text
	 */
	private void showError(UILabel pLabel, String pText)
	{
		if (pText == null)
		{
			pLabel.setVisible(false);
		}
		else
		{
			pLabel.setText(pText);
			pLabel.setVisible(true);
		}
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Performs a manual activation with activation key.
	 * 
	 * @throws Throwable if the activation failed
	 */
	public void doActivate() throws Throwable
	{
		((PackungApplication)getApplication()).getLauncher().setParameter(ActivationAuthenticator.PARAM_ACTIVATIONKEY, tfActivationKey.getText());
		
		if (!((PackungApplication)getApplication()).checkActivation())
		{
			showError(this, "Invalid or expired activation key!");
		}
		else
		{
			((PackungApplication)getApplication()).close(this);
		}
	}
	
	/**
	 * Cancels the registration and closes the screen.
	 */
	public void doCancel()
	{
		getApplication().close(this);
	}
	
	/**
	 * Sends the registration activation to the users email.
	 * 
	 * @throws Throwable if the registration activation was not sent
	 */
	@SuppressWarnings("unchecked")
	public void doRegister() throws Throwable
	{
		Hashtable<String, String> htError = (Hashtable<String, String>)getConnection().call("registration",
																				            "create", 
																				            tfFirstName.getText(),
																				            tfLastName.getText(),
																				            tfUserName.getText(), 
																				            tfEmail.getText(), 
																				            tfPassword.getText(), 
																				            tfPasswordConfirm.getText(), 
																				            rdbLangInfos.getValue("ID"));
	
		//Error Handling
		if (htError != null && htError.size() > 0)
		{
			showError(lblErrUser, htError.get("USER"));
			showError(lblErrPwd, htError.get("PWD"));
			showError(lblErrPwdConfirm, htError.get("PWD_CONFIRM"));
			showError(lblErrEmail, htError.get("EMAIL"));
		}
		else
		{
			hideErrors();
			
			showInformation(this, "An email was sent to the given address!\n\nPlease check your mailbox and follow the instructions in the mail!", "doCancel");
		}
	}

}	// RegistrationWorkScreen
