/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIImage;
import javax.rad.genui.component.UIButton;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.container.UITabsetPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.control.UITable;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.IDataBook;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;

import com.sibvisions.apps.packung.PackungApplication;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;
import com.sibvisions.util.ArrayUtil;

/**
 * The <code>ResultsWorkScreen</code> manages all results for matches and show the current standings. 
 * <br>
 * 
 * @author Martin Handsteiner
 */
public class ResultsWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the user defaults. */
	private IDataBook 		rdbUserDefaults;

	/** Matches DataBook. */
	private	RemoteDataBook	rdbMatches			= new RemoteDataBook();
	/** Matches History DataBook. */
	private	RemoteDataBook	rdbMatchesHistory	= new RemoteDataBook();

	/** Matches navigation table. */
	private UITable navMatches 			= new UITable();
	/** Matches navigation table. */
	private UITable navMatchesHistory 	= new UITable();

	/** the competition image. */
	private UIEditor editCompImage		= new UIEditor();
	/** the editor for matchday. */
	private UIEditor editMatchday		= new UIEditor();
	/** the editor for groupname. */
	private UIEditor editGroupname		= new UIEditor();
	/** the editor for matchbegin. */
	private UIEditor editMatchbegin		= new UIEditor();
	/** the editor for image1. */
	private UIEditor editImagename1		= new UIEditor();
	/** the editor for team1. */
	private UIEditor editTeam1			= new UIEditor();
	/** the label for colon. */
	private UILabel labColon			= new UILabel();
	/** the editor for image2. */
	private UIEditor editImagename2		= new UIEditor();
	/** the editor for team2. */
	private UIEditor editTeam2			= new UIEditor();
	/** the label for goals1. */
	private UIEditor editGoals1			= new UIEditor();
	/** the editor for goals2. */
	private UIEditor editGoals2			= new UIEditor();
	/** the label for finished. */
	private UIButton butFinished		= new UIButton();

	/** the layout for this component. */
	private UIFormLayout layDetails		= new UIFormLayout();
	/** the layout for this component. */
	private UIPanel panDetails			= new UIPanel();

	/** the layout for this component. */
	private UIFormLayout layNextResult	= new UIFormLayout();
	/** the layout for this component. */
	private UIPanel panNextResult		= new UIPanel();

	/** the layout for this component. */
	private UIFormLayout layHistory		= new UIFormLayout();
	/** the layout for this component. */
	private UIPanel panHistory			= new UIPanel();

	/** the layout for this component. */
	private UITabsetPanel tabMain		= new UITabsetPanel();

	/** the layout for this component. */
	private UIFormLayout layThis		= new UIFormLayout();
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * It creates the <code>ResultsWorkScreen</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public ResultsWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}
		
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{		
		rdbUserDefaults = ((PackungApplication)getApplication()).getUserDefaults();
		
		rdbMatches.setDataSource(dataSource);
		rdbMatches.setName("matches");
		rdbMatches.open();
		PackungUtil.saveImmediate(rdbMatches);
		rdbMatches.eventAfterUpdated().addListener(this, "doConfigureMatches");
		rdbMatches.eventAfterRowSelected().addListener(this, "doConfigureMatches");
		
		configureMatchesDataBook(rdbMatches, "MATCHDAY");
		
		rdbMatchesHistory.setDataSource(dataSource);
		rdbMatchesHistory.setName("matchesHistory");
		rdbMatchesHistory.open();
		PackungUtil.saveImmediate(rdbMatchesHistory);
		
		configureMatchesDataBook(rdbMatchesHistory, "FINISHED");
	}
    
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		editCompImage.setCellEditor(PackungUtil.COMPETITION_IMAGEVIEWER);
		editCompImage.setDataRow(rdbUserDefaults);
		editCompImage.setColumnName("COMP_IMAGE");
	
		editMatchday.setDataRow(rdbMatches);
		editMatchday.setColumnName("MATCHDAY");

		editGroupname.setDataRow(rdbMatches);
		editGroupname.setColumnName("GROUPNAME");
		
		editMatchbegin.setMaximumSize(0, Integer.MAX_VALUE);
		editMatchbegin.setDataRow(rdbMatches);
		editMatchbegin.setColumnName("MATCHBEGIN");
		
		editTeam1.setDataRow(rdbMatches);
		editTeam1.setColumnName("TEAM1");
		editImagename1.setDataRow(rdbMatches);
		editImagename1.setColumnName("IMAGENAME1");
		editGoals1.setDataRow(rdbMatches);
		editGoals1.setColumnName("GOALS1");
		
		labColon.setText(":");
		editTeam2.setDataRow(rdbMatches);
		editTeam2.setColumnName("TEAM2");
		editTeam2.setMinimumSize(150, 0);
		editImagename2.setDataRow(rdbMatches);
		editImagename2.setColumnName("IMAGENAME2");
		editGoals2.setDataRow(rdbMatches);
		editGoals2.setColumnName("GOALS2");
		
		butFinished.setImage(UIImage.getImage(PackungUtil.IMAGE_WHISTLE));
		butFinished.setText("Match Finished");
		butFinished.eventAction().addListener(this, "doMatchFinished");

		navMatches.setDataBook(rdbMatches);
		navMatches.setShowGridLines(false);
		PackungUtil.formatMatches(navMatches);

		layDetails.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		layDetails.setMargins(0, 0, 0, 0);
		layDetails.setHorizontalGap(20);
		panDetails.setLayout(layDetails);
		panDetails.add(editCompImage, layDetails.getConstraints(0, 0, 0, -1));
		panDetails.add(editMatchday, layDetails.getConstraints(1, 0));
		layDetails.setHorizontalGap(5);
		panDetails.add(editGroupname, layDetails.getConstraints(1, 1));
		panDetails.add(editMatchbegin, layDetails.getConstraints(3, 1, 7, 1));
		panDetails.add(editTeam1, layDetails.getConstraints(1, -2, 2, -2));
		panDetails.add(editImagename1, layDetails.getConstraints(3, -2));
		panDetails.add(editGoals1, layDetails.getConstraints(4, -2));
		panDetails.add(labColon, layDetails.getConstraints(5, -2));
		panDetails.add(editGoals2, layDetails.getConstraints(6, -2));
		panDetails.add(editImagename2, layDetails.getConstraints(7, -2));
		panDetails.add(editTeam2, layDetails.getConstraints(8, -2));
		panDetails.add(butFinished, layDetails.getConstraints(8, -1));
		
		layNextResult.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);
		layNextResult.setVerticalGap(20);
		panNextResult.setLayout(layNextResult);
		panNextResult.add(panDetails, layNextResult.getConstraints(0, 0, -1, 0));
		panNextResult.add(navMatches, layNextResult.getConstraints(0, 1, -1, 1));

		navMatchesHistory.setDataBook(rdbMatchesHistory);
		navMatchesHistory.setShowGridLines(false);
		navMatchesHistory.setMaximumSize(Integer.MAX_VALUE, 400);

		panHistory.setLayout(layHistory);
		panHistory.add(navMatchesHistory, layHistory.getConstraints(0, 0, -1, -1));
		
		tabMain.add(panNextResult, "Enter Next Results");
		tabMain.add(panHistory, "History of Results");
		
		layThis.setMargins(0, 0, 0, 0);
		setMinimumSize(633, 0);
		setLayout(layThis);
		add(tabMain, layThis.getConstraints(0, 0, -1, -1));
	}
	
	/**
	 * Configures the matches DataBook.
	 * 
	 * @param pDataBook the data book.
	 * @param pRemoveColumn the columns to remove.
	 * @throws Exception if an exception occurs.
	 */
	private void configureMatchesDataBook(IDataBook pDataBook, String pRemoveColumn) throws Exception
	{
		PackungUtil.configureTeamColumns(pDataBook);

		pDataBook.getRowDefinition().setTableColumnNames(ArrayUtil.remove(pDataBook.getRowDefinition().getTableColumnNames(), pRemoveColumn));
	}
		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		
	/**
	 * Sets editors read only and enabled or disables the Button.
	 * 
	 * @throws Exception if an exception occurs.
	 */
	public void doConfigureMatches() throws Exception
	{
		rdbMatches.setReadOnly("Y".equals(rdbMatches.getValue("FINISHED")));
		butFinished.setEnabled("N".equals(rdbMatches.getValue("FINISHED")) && rdbMatches.getValue("GOALS1") != null && rdbMatches.getValue("GOALS2") != null);
	}

	/**
	 * Sets editors read only and enabled or disables the Button.
	 * 
	 * @throws Throwable if an exception occurs.
	 */
	public void doMatchFinished() throws Throwable
	{
		rdbMatches.setValue("FINISHED", "Y");
		
		getConnection().callAction("fillInGroupteams");
		
		reload();
	}

}	// ResultsWorkScreen
