/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 18.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIInsets;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.control.UITable;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.IDataBook;
import javax.rad.model.IDataRow;
import javax.rad.model.ModelException;
import javax.rad.model.condition.ICondition;
import javax.rad.model.condition.LikeIgnoreCase;
import javax.rad.model.event.DataRowEvent;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;

import com.sibvisions.apps.packung.PackungApplication;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;

/**
 * The <code>StatisticsWorkScreen</code> shows all available stats of all players. 
 * 
 * @author Roland Hrmann
 */
public class StatisticsWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** UserDefaults DataBook. */
	private	IDataBook		rdbUserDefaults;	
	/** Bet groups Lookup DataBook. */
	private	RemoteDataBook	rdbBetgroups 		= new RemoteDataBook();
	/** Players DataBook. */
	private	RemoteDataBook	rdbPlayers			= new RemoteDataBook();
	/** Match day history DataBook. */
	private	RemoteDataBook	rdbMatchdaysHistory	= new RemoteDataBook();
	/** Bets history DataBook. */
	private	RemoteDataBook	rdbBetsHistory		= new RemoteDataBook();
	/** User images DataBook. */
	private	RemoteDataBook	rdbUserImages		= new RemoteDataBook();

	/** Overview Filter DataBook. */
	private	IDataRow		drFilter;

	/** Rankings navigation table. */
	private UITable			tabBetgroups		= new UITable();
	/** Matches navigation table. */
	private UITable 		tabPlayers 			= new UITable();
	/** Results navigation table. */
	private UITable 		tabMatchdaysHistory 	= new UITable();
	/** Results navigation table. */
	private UITable 		tabBetsHistory 	= new UITable();

	/** the competition image. */
	private UIEditor 		editCompImage		= new UIEditor();
	/** the competition image. */
	private UIEditor 		editUserImage		= new UIEditor();
	/** User name filter Label. */
	private UILabel			labUsername			= new UILabel();
	/** User name filter editor. */
	private UIEditor		editUsername		= new UIEditor();
	
	/** Images layout. */
	private	UIFormLayout	layImages			= new UIFormLayout();
	/** Images panel. */
	private	UIPanel			panImages			= new UIPanel();

	/** Overview group layout. */
	private	UIFormLayout	layOverview			= new UIFormLayout();
	/** Overview group panel. */
	private	UIGroupPanel	panOverview			= new UIGroupPanel();

	/** Overview group layout. */
	private	UIFormLayout	layDetails			= new UIFormLayout();
	/** Overview group panel. */
	private	UIGroupPanel	panDetails			= new UIGroupPanel();

	/** Overview group panel. */
	private	UIFormLayout	layThis				= new UIFormLayout();
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * It creates the <code>StatisticsWorkScreen</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public StatisticsWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}
		
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{		
		rdbUserDefaults = ((PackungApplication)getApplication()).getUserDefaults();
		
		rdbBetgroups.setDataSource(dataSource);
		rdbBetgroups.setName("betgroups");
		rdbBetgroups.open();
		rdbBetgroups.setReadOnly(true);
			
		rdbPlayers.setDataSource(dataSource);
		rdbPlayers.setName("players");
		rdbPlayers.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID", "BEGR_ID"},
				rdbBetgroups,
				new String[] {"COMP_ID", "ID"}));
		rdbPlayers.open();	
		rdbPlayers.setReadOnly(true);
		rdbPlayers.setMemFilter(true);
		rdbPlayers.setMemSort(true);
		PackungUtil.configureTeamColumns(rdbPlayers);
		
		rdbMatchdaysHistory.setDataSource(dataSource);
		rdbMatchdaysHistory.setName("matchdaysHistory");
		rdbMatchdaysHistory.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID", "USER_ID"},
				rdbPlayers,
				new String[] {"COMP_ID", "ID"}));
		rdbMatchdaysHistory.open();		
		rdbMatchdaysHistory.setReadOnly(true);
		PackungUtil.configureTeamColumns(rdbMatchdaysHistory);

		rdbBetsHistory.setDataSource(dataSource);
		rdbBetsHistory.setName("betsHistory");
		rdbBetsHistory.setMasterReference(new ReferenceDefinition(
				new String[] {"COMP_ID", "USER_ID", "MADA_ID"},
				rdbMatchdaysHistory,
				new String[] {"COMP_ID", "USER_ID", "ID"}));
		rdbBetsHistory.open();		
		rdbBetsHistory.setReadOnly(true);
		PackungUtil.configureTeamColumns(rdbBetsHistory);

		rdbUserImages.setDataSource(dataSource);
		rdbUserImages.setName("userImages");
		rdbUserImages.setMasterReference(new ReferenceDefinition(
				new String[] {"ID"},
				rdbPlayers,
				new String[] {"ID"}));
		rdbUserImages.open();		
		rdbUserImages.setReadOnly(true);

		// Overview Filter
		drFilter = rdbPlayers.createEmptyRow(new String[] {"USERNAME"});
		drFilter.eventValuesChanged().addListener(this, "doFilterOverview");
	}
    
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{		
		editCompImage.setCellEditor(PackungUtil.COMPETITION_IMAGEVIEWER);
		editCompImage.setDataRow(rdbUserDefaults);
		editCompImage.setColumnName("COMP_IMAGE");
		editCompImage.setMinimumSize(135, 0);

		editUserImage.setCellEditor(PackungUtil.USER_IMAGEVIEWER);
		editUserImage.setDataRow(rdbUserImages);
		editUserImage.setColumnName("IMAGE");

		layImages.setMargins(0, 0, 0, 0);
		panImages.setLayout(layImages);
		panImages.add(editCompImage, layImages.getConstraints(0, 0));
		panImages.add(editUserImage, layImages.getConstraints(0, 1, 0, -1));
		
		labUsername.setText("Search Player:");
		editUsername.setColumnName("USERNAME");
		editUsername.setDataRow(drFilter);
		editUsername.setSavingImmediate(true);
				
		tabBetgroups.setDataBook(rdbBetgroups);
		tabBetgroups.setShowGridLines(false);
		tabBetgroups.setPreferredSize(135, 120);

		tabPlayers.setDataBook(rdbPlayers);
		tabPlayers.setShowGridLines(false);	

		layOverview.setMargins(new UIInsets(5, 5, 5, 5));
		layOverview.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		layOverview.setHorizontalGap(10);
		layOverview.setVerticalGap(10);
		panOverview.setText("Players Overview");
		panOverview.setLayout(layOverview);
		panOverview.add(panImages, layOverview.getConstraints(0, 0, 0, -1));
		panOverview.add(labUsername, layOverview.getConstraints(1, 0));
		panOverview.add(editUsername, layOverview.getConstraints(2, 0, 3, 0));
		panOverview.add(tabBetgroups, layOverview.getConstraints(1, 1, 2, -1));
		panOverview.add(tabPlayers, layOverview.getConstraints(3, 1, -1, -1));
				
		tabMatchdaysHistory.setDataBook(rdbMatchdaysHistory);
		tabMatchdaysHistory.setShowGridLines(false);
		tabMatchdaysHistory.setPreferredSize(135, 120);
		
		tabBetsHistory.setDataBook(rdbBetsHistory);
		tabBetsHistory.setShowGridLines(false);
		tabBetsHistory.setPreferredSize(550, 120);

		layDetails.setMargins(new UIInsets(5, 5, 5, 5));
		layDetails.setHorizontalGap(10);
		panDetails.setLayout(layDetails);
		panDetails.setText("Player Details");
		panDetails.add(tabMatchdaysHistory, layDetails.getConstraints(0, 0, 0, -1));
		panDetails.add(tabBetsHistory, layDetails.getConstraints(1, 0, -1, -1));
				
		layThis.setMargins(0, 0, 0, 0);
		setLayout(layThis);
		add(panOverview, layThis.getConstraints(0, 0, -1, -2));
		add(panDetails, layThis.getConstraints(0, -1, -1, -1));
	}
			
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Searches the Overview over ranking in bet group.
	 * 
	 * @param pDataRowEvent the dataRow event.
	 * 
	 * @throws ModelException if the search fails
	 */
	public void doFilterOverview(DataRowEvent pDataRowEvent) throws ModelException
	{
		if (!drFilter.equals(pDataRowEvent.getOriginalDataRow()))
		{
			if (drFilter.getValue("USERNAME") != null)
			{
				ICondition filter = new LikeIgnoreCase("USERNAME", "*" + drFilter.getValue("USERNAME") + "*");
				rdbPlayers.setFilter(filter);
			}
			else
			{
				rdbPlayers.setFilter(null);
			}
		}
	}
	
}	// StatisticsWorkScreen
