/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [HM] - creation
 */
package com.sibvisions.apps.packung.screens;

import java.net.URL;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIDimension;
import javax.rad.genui.celleditor.UILinkedCellEditor;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.container.UISplitPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.IDataRow;
import javax.rad.model.condition.Equals;
import javax.rad.model.event.DataRowEvent;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;

import com.sibvisions.apps.components.NavigationTable;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.mem.MemDataBook;
import com.sibvisions.rad.model.remote.RemoteDataBook;

/**
 * The <code>TeamsWorkScreen</code> manages the teams and the competition types. 
 *
 * @author Martin Handsteiner
 */
public class TeamsWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** Teams DataBook. */
	private	RemoteDataBook	rdbTeams				= new RemoteDataBook();
	/** Competitiontypes DataBook. */
	private	RemoteDataBook	rdbCompetitiontypes		= new RemoteDataBook();
	/** CotyTeam DataBook. */
	private	RemoteDataBook	rdbCotyTeam				= new RemoteDataBook();
	/** CompetitiontypesLook DataBook. */
	private	RemoteDataBook	rdbCompetitiontypesLook	= new RemoteDataBook();
	/** ImagesLook DataBook. */
	private	MemDataBook	mdbImagesLook				= new MemDataBook();

	/** CompetitionType filter for teams. */
	private	IDataRow		drDefaultCompetitionType;
	
	/** Teams navigation table. */
	private NavigationTable navTeams 				= new NavigationTable();
	/** Competitiontypes navigation table. */
	private NavigationTable navCompetitiontypes 	= new NavigationTable();

	/** Label for competition type. */
	private UILabel         labCompetitionType		= new UILabel();
	/** Editor for competition type. */
	private UIEditor		editCompetitionType 	= new UIEditor();
	
	/** Label for team. */
	private UILabel         labTeam					= new UILabel();
	/** Editor for team. */
	private UIEditor		editTeam			 	= new UIEditor();
	/** Label for code. */
	private UILabel         labCode					= new UILabel();
	/** Editor for code. */
	private UIEditor		editCode			 	= new UIEditor();
	/** Label for image. */
	private UILabel         labImage				= new UILabel();
	/** Editor for image. */
	private UIEditor		editImage			 	= new UIEditor();
	/** Label for image name. */
	private UILabel         labImagename			= new UILabel();
	/** Editor for image name. */
	private UIEditor		editImagename		 	= new UIEditor();
	
	/** The border layout for this container. */
	private UIBorderLayout	layoutThis				= new UIBorderLayout();
	/** Main split panel. */
	private UISplitPanel	spanMain				= new UISplitPanel();
	
	/** The layout for teams container. */
	private UIFormLayout    layTeams				= new UIFormLayout();
	/** The panel for the teams. */
	private	UIPanel			panTeams				= new UIPanel();

	/** The layout for details. */
	private UIFormLayout    layDetails				= new UIFormLayout();
	/** The panel for the details. */
	private	UIPanel			panDetails				= new UIPanel();
	/** The layout for detail. */
	private UIFormLayout    layDetail				= new UIFormLayout();
	/** The panel for the detail. */
	private	UIGroupPanel	groupDetail				= new UIGroupPanel();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * It creates the <code>TeamsWorkScreen</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public TeamsWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}
		
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{
		rdbTeams.setDataSource(dataSource);
		rdbTeams.setName("teams");
		rdbTeams.open();
		rdbTeams.eventAfterInserted().addListener(this, "doReloadTeams");
		
		rdbTeams.getRowDefinition().setTableColumnNames(new String[] {"TEAM", "CODE", "IMAGE"});
		PackungUtil.configureImageColumn(rdbTeams.getRowDefinition().getColumnDefinition("IMAGE"));

		mdbImagesLook.setName("Images");
		mdbImagesLook.setRowDefinition(rdbTeams.getRowDefinition().createRowDefinition(new String[] {"IMAGE", "IMAGENAME"}));
		mdbImagesLook.open();
		
		findImages();
		
		UILinkedCellEditor imageLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"IMAGE", "IMAGENAME"},
				mdbImagesLook,
				new String[] {"IMAGE", "IMAGENAME"}));
		rdbTeams.getRowDefinition().getColumnDefinition("IMAGENAME").getDataType().setCellEditor(imageLook);
		
		rdbCompetitiontypes.setDataSource(dataSource);
		rdbCompetitiontypes.setName("competitiontypes");
		rdbCompetitiontypes.setMasterReference(
				new ReferenceDefinition(
						new String[] {"TEAM_ID"},
						rdbTeams,
						new String[] {"ID"}));
		rdbCompetitiontypes.open();
		rdbCompetitiontypes.eventValuesChanged().addListener(this, "doRelateTeams");
		rdbCompetitiontypes.eventAfterInserted().addListener(this, "doReloadCompetitiontypes");
		
		rdbCompetitiontypes.getRowDefinition().getColumnDefinition("DEDICATED").getDataType().setCellEditor(PackungUtil.YESNO_EDITOR);

		rdbCotyTeam.setDataSource(dataSource);
		rdbCotyTeam.setName("cotyTeam");
		rdbCotyTeam.setMasterReference(
				new ReferenceDefinition(
						new String[] {"TEAM_ID", "COTY_ID"},
						rdbCompetitiontypes,
						new String[] {"TEAM_ID", "ID"}));
		rdbCotyTeam.open();
		
		rdbCompetitiontypesLook.setDataSource(dataSource);
		rdbCompetitiontypesLook.setName("competitiontypesLook");
		rdbCompetitiontypesLook.open();
		
		UILinkedCellEditor competitiontypesLook = new UILinkedCellEditor();
		competitiontypesLook.setLinkReference(
				new ReferenceDefinition(
						new String[] {"ID", "COMPETITION_TYPE"},
						rdbCompetitiontypesLook,
						new String[] {"ID", "COMPETITION_TYPE"}));
		
		drDefaultCompetitionType = rdbCompetitiontypesLook.createDataRow(new String[] {"ID", "COMPETITION_TYPE"});
		
		drDefaultCompetitionType.getRowDefinition().getColumnDefinition("COMPETITION_TYPE").getDataType().setCellEditor(competitiontypesLook);
		drDefaultCompetitionType.eventValuesChanged().addListener(this, "doSetTeamFilter");
		
		doSetTeamFilter();
	}
    
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		labCompetitionType.setText("Teams with Competition Type:");
		
		editCompetitionType.setDataRow(drDefaultCompetitionType);
		editCompetitionType.setColumnName("COMPETITION_TYPE");
		
		navTeams.setDataBook(rdbTeams);
		navTeams.setShowGridLines(false);
		navTeams.setMaximumSize(new UIDimension(Integer.MAX_VALUE, 400));
		
		layTeams.setVerticalGap(10);
		panTeams.setLayout(layTeams);
		panTeams.add(labCompetitionType, layTeams.getConstraints(0, 0));
		panTeams.add(editCompetitionType, layTeams.getConstraints(1, 0, -1, 0));
		panTeams.add(navTeams, layTeams.getConstraints(0, 1, -1, -1));
		
		labTeam.setText("Team:");
		editTeam.setDataRow(rdbTeams);
		editTeam.setColumnName("TEAM");
		
		labCode.setText("Code:");
		editCode.setDataRow(rdbTeams);
		editCode.setColumnName("CODE");
		
		labImagename.setText("Imagename:");
		editImagename.setDataRow(rdbTeams);
		editImagename.setColumnName("IMAGENAME");
		labImage.setText("Image:");
		editImage.setDataRow(rdbTeams);
		editImage.setColumnName("IMAGE");
		
		navCompetitiontypes.setDataBook(rdbCompetitiontypes);
		navCompetitiontypes.setShowGridLines(false);
		
		groupDetail.setText("Team Details");
		groupDetail.setLayout(layDetail);
		groupDetail.add(labTeam, layDetail.getConstraints(0, 0));
		groupDetail.add(editTeam, layDetail.getConstraints(1, 0, -1, 0));
		groupDetail.add(labCode, layDetail.getConstraints(0, 1));
		groupDetail.add(editCode, layDetail.getConstraints(1, 1, 5, 1));
		groupDetail.add(labImagename, layDetail.getConstraints(0, 2));
		groupDetail.add(editImagename, layDetail.getConstraints(1, 2, -1, 2));
		groupDetail.add(labImage, layDetail.getConstraints(0, 3));
		groupDetail.add(editImage, layDetail.getConstraints(1, 3));
		
		layDetail.setVerticalGap(20);
		groupDetail.add(navCompetitiontypes, layDetail.getConstraints(0, 4, -1, -1));

		panDetails.setLayout(layDetails);
		panDetails.add(groupDetail, layDetails.getConstraints(0, 0, -1, -1));
		
		panTeams.setMinimumSize(260, 0);
		panDetails.setMinimumSize(227, 0);
		spanMain.add(panTeams, UISplitPanel.FIRST_COMPONENT);
		spanMain.add(panDetails, UISplitPanel.SECOND_COMPONENT);
		
		layoutThis.setMargins(0, 0, 0, 0);
		setLayout(layoutThis);
		add(spanMain);
	}
		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	
	/**
	 * Searches for an image library.
	 */
	private void findImages()
	{
		try
		{
			String imageLib = getClass().getResource(PackungUtil.IMAGE_NO_FLAG).toString();
			
			if (imageLib.startsWith("jar:"))
			{
				imageLib = imageLib.substring(4);
			}
			int index = imageLib.indexOf('!');
			if (index >= 0)
			{
				imageLib = imageLib.substring(0, index);
				
				ZipInputStream zipStream = new ZipInputStream(new URL(imageLib).openStream());
				
				ZipEntry entry = zipStream.getNextEntry();
				
				while (entry != null)
				{
					mdbImagesLook.insert(false);
					mdbImagesLook.setValue("IMAGE", entry.getName());
					mdbImagesLook.setValue("IMAGENAME", entry.getName());
					
					entry = zipStream.getNextEntry();
				}
				
				zipStream.close();
			}
			mdbImagesLook.saveAllRows();
		}
		catch (Exception ex)
		{
			error("Image library was not found!");
		}
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Sets the Team Filter with the current competition type.
	 * 
	 * @throws Exception if an exception occurs.
	 */
	public void doSetTeamFilter() throws Exception
	{
		rdbTeams.setFilter(new Equals("COTY_ID", drDefaultCompetitionType.getValue("ID")));
	}
	
	/**
	 * Reloads the competitions types.
	 * 
	 * @throws Exception if an exception occurs.
	 */
	public void doReloadCompetitiontypes() throws Exception
	{
		rdbCompetitiontypes.reload();
	}
	
	/**
	 * Reloads the teams.
	 * 
	 * @throws Exception if an exception occurs.
	 */
	public void doReloadTeams() throws Exception
	{
		if (drDefaultCompetitionType.getValue("ID") != null)
		{
			rdbTeams.reload();
		}
	}
	
	/**
	 * Saves the correct relation between teams and competition types.
	 * 
	 * @param pDataRowEvent the dataRow event.
	 * @throws Exception if an exception occurs.
	 */
	public void doRelateTeams(DataRowEvent pDataRowEvent) throws Exception
	{
		String[] changedColumnNames = pDataRowEvent.getChangedColumnNames();
		if ("DEDICATED".equals(changedColumnNames[0]))
		{
			if ("N".equals(rdbCompetitiontypes.getValue("DEDICATED")))
			{
				rdbCotyTeam.delete();
			}
			else
			{
				rdbCotyTeam.insert(false);
				rdbCotyTeam.saveSelectedRow();
			}
		}
		else
		{
			rdbCompetitiontypes.saveSelectedRow();
		}
		doReloadTeams();
	}
	
}	// TeamsWorkScreen
