/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [HM] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIDimension;
import javax.rad.genui.celleditor.UILinkedCellEditor;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.container.UISplitPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.control.UITable;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.IDataRow;
import javax.rad.model.condition.Equals;
import javax.rad.model.event.DataRowEvent;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;

import com.sibvisions.apps.components.NavigationTable;
import com.sibvisions.apps.packung.PackungApplication;
import com.sibvisions.apps.packung.PackungUtil;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;

/**
 * The <code>UserAdministrationWorkScreen</code> manages the registered users and role grants. 
 *
 * @author Martin Handsteiner
 */
public class UserAdministrationWorkScreen extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** Users DataBook. */
	private	RemoteDataBook	rdbUsers			= new RemoteDataBook();
	/** Roles DataBook. */
	private	RemoteDataBook	rdbRoles			= new RemoteDataBook();
	/** UserRole DataBook. */
	private	RemoteDataBook	rdbUserRole			= new RemoteDataBook();
	/** RolesLook DataBook. */
	private	RemoteDataBook	rdbRolesLook		= new RemoteDataBook();
	/** LanguagesLook DataBook. */
	private	RemoteDataBook	rdbLanguagesLook	= new RemoteDataBook();

	/** Role filter for users. */
	private	IDataRow		drDefaultRole;
	
	/** Users navigation table. */
	private NavigationTable navUsers 		= new NavigationTable();
	/** Roles navigation table. */
	private UITable navRoles 				= new UITable();

	/** Label for role. */
	private UILabel         labRole			= new UILabel();
	/** Editor for role. */
	private UIEditor		editRole 		= new UIEditor();
	
	/** Label for username. */
	private UILabel         labUsername		= new UILabel();
	/** Editor for username. */
	private UIEditor		editUsername	= new UIEditor();
	/** Label for active. */
	private UILabel         labActive		= new UILabel();
	/** Editor for active. */
	private UIEditor		editActive		= new UIEditor();
	/** Label for password. */
	private UILabel         labPassword		= new UILabel();
	/** Editor for password. */
	private UIEditor		editPassword	= new UIEditor();
	/** Label for firstname. */
	private UILabel         labFirstName	= new UILabel();
	/** Editor for firstname. */
	private UIEditor		editFirstName 	= new UIEditor();
	/** Label for lastname. */
	private UILabel         labLastName		= new UILabel();
	/** Editor for lastname. */
	private UIEditor		editLastName	= new UIEditor();
	/** Label for email. */
	private UILabel         labEmail		= new UILabel();
	/** Editor for email. */
	private UIEditor		editEmail	 	= new UIEditor();
	/** Label for language. */
	private UILabel         labLanguage		= new UILabel();
	/** Editor for language. */
	private UIEditor		editLanguage 	= new UIEditor();
	
	/** The border layout for this container. */
	private UIBorderLayout	layoutThis		= new UIBorderLayout();
	/** Main split panel. */
	private UISplitPanel	spanMain		= new UISplitPanel();
	
	/** The layout for users container. */
	private UIFormLayout    layUsers		= new UIFormLayout();
	/** The panel for the users. */
	private	UIPanel			panUsers		= new UIPanel();

	/** The layout for details. */
	private UIFormLayout    layDetails		= new UIFormLayout();
	/** The panel for the screen. */
	private	UIPanel			panDetails		= new UIPanel();
	/** The layout for details. */
	private UIFormLayout    layDetail		= new UIFormLayout();
	/** The panel for the detail. */
	private	UIGroupPanel	groupDetail		= new UIGroupPanel();
	/** The layout for roles. */
	private UIFormLayout    layRoles		= new UIFormLayout();
	/** The panel for the roles. */
	private	UIGroupPanel	groupRoles		= new UIGroupPanel();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * It creates the <code>UserAdministrationWorkScreen</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public UserAdministrationWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)  throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}
		
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{
		
		rdbUsers.setDataSource(dataSource);
		rdbUsers.setName("users");
		rdbUsers.open();
		PackungUtil.saveImmediate(rdbUsers);
		rdbUsers.eventValuesChanged().addListener(this, "doChangeLanguage");
		rdbUsers.eventAfterInserted().addListener(this, "doReloadUsers");
		rdbUsers.eventAfterRowSelected().addListener(this, "doCheckIsOwnUser");
		
		rdbUsers.getRowDefinition().setTableColumnNames(new String[] {"USERNAME", "ACTIVE", "EMAIL"});
		rdbUsers.getRowDefinition().getColumnDefinition("ACTIVE").getDataType().setCellEditor(PackungUtil.YESNO_EDITOR);

		rdbLanguagesLook.setDataSource(dataSource);
		rdbLanguagesLook.setName("languagesLook");
		rdbLanguagesLook.open();
		UILinkedCellEditor languagesLook = new UILinkedCellEditor(new ReferenceDefinition(
				new String[] {"LANG_ID", "LANG_CODE", "LANGUAGE"},
				rdbLanguagesLook,
				new String[] {"ID", "CODE_ID", "LANGUAGE"}));
		rdbUsers.getRowDefinition().getColumnDefinition("LANGUAGE").getDataType().setCellEditor(languagesLook);
		
		rdbRoles.setDataSource(dataSource);
		rdbRoles.setName("roles");
		rdbRoles.setMasterReference(
				new ReferenceDefinition(
						new String[] {"USER_ID"},
						rdbUsers,
						new String[] {"ID"}));
		rdbRoles.open();
		rdbRoles.setInsertEnabled(false);
		rdbRoles.setDeleteEnabled(false);
		rdbRoles.eventValuesChanged().addListener(this, "doRelateUsers");
		
		rdbRoles.getRowDefinition().getColumnDefinition("NAME").setLabel("Role");
		rdbRoles.getRowDefinition().getColumnDefinition("GRANTED").getDataType().setCellEditor(PackungUtil.YESNO_EDITOR);
		
		rdbUserRole.setDataSource(dataSource);
		rdbUserRole.setName("userRole");
		rdbUserRole.setMasterReference(
				new ReferenceDefinition(
						new String[] {"USER_ID", "ROLE_ID"},
						rdbRoles,
						new String[] {"USER_ID", "ID"}));
		rdbUserRole.open();
		
		rdbRolesLook.setDataSource(dataSource);
		rdbRolesLook.setName("rolesLook");
		rdbRolesLook.open();

		UILinkedCellEditor rolesLook = new UILinkedCellEditor();
		rolesLook.setLinkReference(
				new ReferenceDefinition(
						new String[] {"ID", "NAME"},
						rdbRolesLook,
						new String[] {"ID", "NAME"}));
		
		drDefaultRole = rdbRolesLook.createDataRow(new String[] {"ID", "NAME"});
		
		drDefaultRole.getRowDefinition().getColumnDefinition("NAME").getDataType().setCellEditor(rolesLook);
		drDefaultRole.eventValuesChanged().addListener(this, "doSetUserFilter");
		
		doSetUserFilter();
	}
    
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		labRole.setText("Users with Role:");
		
		editRole.setDataRow(drDefaultRole);
		editRole.setColumnName("NAME");
		
		navUsers.setDataBook(rdbUsers);
		navUsers.setShowGridLines(false);
		navUsers.setMaximumSize(new UIDimension(Integer.MAX_VALUE, 400));
		
		layUsers.setVerticalGap(10);
		panUsers.setLayout(layUsers);
		panUsers.add(labRole, layUsers.getConstraints(0, 0));
		panUsers.add(editRole, layUsers.getConstraints(1, 0, -1, 0));
		panUsers.add(navUsers, layUsers.getConstraints(0, 1, -1, -1));
		
		labUsername.setText("Username:");
		editUsername.setDataRow(rdbUsers);
		editUsername.setColumnName("USERNAME");
		
		labActive.setText("Active:");
		editActive.setDataRow(rdbUsers);
		editActive.setColumnName("ACTIVE");
		
		labPassword.setText("Password:");
		editPassword.setCellEditor(PackungUtil.PASSWORD_EDITOR);
		editPassword.setDataRow(rdbUsers);
		editPassword.setColumnName("PASSWORD");
		
		labFirstName.setText("First name:");
		editFirstName.setDataRow(rdbUsers);
		editFirstName.setColumnName("FIRST_NAME");
		
		labLastName.setText("Last name:");
		editLastName.setDataRow(rdbUsers);
		editLastName.setColumnName("LAST_NAME");
		
		labEmail.setText("Email:");
		editEmail.setDataRow(rdbUsers);
		editEmail.setColumnName("EMAIL");
		
		labLanguage.setText("Language:");
		editLanguage.setDataRow(rdbUsers);
		editLanguage.setColumnName("LANGUAGE");
		
		navRoles.setDataBook(rdbRoles);
		navRoles.setShowGridLines(false);
		navRoles.setTableHeaderVisible(false);
		
		groupDetail.setText("User Details");
		groupDetail.setLayout(layDetail);
		groupDetail.add(labUsername, layDetail.getConstraints(0, 0));
		groupDetail.add(editUsername, layDetail.getConstraints(1, 0, 2, 0));
		groupDetail.add(labActive, layDetail.getConstraints(-2, 0));
		groupDetail.add(editActive, layDetail.getConstraints(-1, 0));
		groupDetail.add(labPassword, layDetail.getConstraints(0, 1));
		groupDetail.add(editPassword, layDetail.getConstraints(1, 1, 2, 1));
		groupDetail.add(labLanguage, layDetail.getConstraints(0, 2));
		groupDetail.add(editLanguage, layDetail.getConstraints(1, 2, 2, 2));
		groupDetail.add(labEmail, layDetail.getConstraints(0, 3));
		groupDetail.add(editEmail, layDetail.getConstraints(1, 3, -1, 3));
		groupDetail.add(labFirstName, layDetail.getConstraints(0, 4));
		groupDetail.add(editFirstName, layDetail.getConstraints(1, 4));
		groupDetail.add(labLastName, layDetail.getConstraints(2, 4));
		groupDetail.add(editLastName, layDetail.getConstraints(3, 4, -1, 4));
		
		groupRoles.setText("Role Grants");
		groupRoles.setLayout(layRoles);
		groupRoles.add(navRoles, layRoles.getConstraints(0, 0, -1, -1));
		
		layDetails.setVerticalAlignment(IAlignmentConstants.ALIGN_TOP);
		panDetails.setLayout(layDetails);
		panDetails.add(groupDetail, layDetails.getConstraints(0, 0, -1, 0));
		panDetails.add(groupRoles, layDetails.getConstraints(0, 1, -1, 1));
		
		spanMain.add(panUsers, UISplitPanel.FIRST_COMPONENT);
		spanMain.add(panDetails, UISplitPanel.SECOND_COMPONENT);
		
		setLayout(layoutThis);
		add(spanMain);
	}
		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Sets the Team Filter with the current comptition type.
	 * 
	 * @throws Exception if an exception occurs.
	 */
	public void doSetUserFilter() throws Exception
	{
		rdbUsers.setFilter(new Equals("ROLE_ID", drDefaultRole.getValue("ID")));
	}
	
	/**
	 * Reloads.
	 * 
	 * @throws Exception if an exception occurs.
	 */
	public void doReloadUsers() throws Exception
	{
		if (drDefaultRole.getValue("ID") != null)
		{
			rdbUsers.reload();
		}
	}
	
	/**
	 * Checks, if the own user is the current user.
	 * 
	 * @throws Exception if an exception occurs.
	 */
	public void doCheckIsOwnUser() throws Exception
	{
		boolean isOwnUser = getConnection().getUserName().equals(rdbUsers.getValue("USERNAME"));
			
		rdbUsers.getRowDefinition().getColumnDefinition("ACTIVE").setReadOnly(isOwnUser);
		rdbUsers.setDeleteEnabled(!isOwnUser);
	}
	
	/**
	 * Checks, if the own user is the current user.
	 * 
	 * @param pDataRowEvent the data row event.
	 * @throws Exception if an exception occurs.
	 */
	public void doChangeLanguage(DataRowEvent pDataRowEvent) throws Exception
	{
		if (pDataRowEvent.isChangedColumnName("LANG_CODE")
				&& getConnection().getUserName().equals(rdbUsers.getValue("USERNAME")))
		{
			((PackungApplication)getApplication()).setLanguage((String)rdbUsers.getValue("LANG_CODE"));
		}
	}
	
	/**
	 * Saves the correct relation between teams and competition types.
	 * 
	 * @param pDataRowEvent the dataRow event.
	 * @throws Exception if an exception occurs.
	 */
	public void doRelateUsers(DataRowEvent pDataRowEvent) throws Exception
	{
		String[] changedColumnNames = pDataRowEvent.getChangedColumnNames();
		if ("GRANTED".equals(changedColumnNames[0]))
		{
			if ("N".equals(rdbRoles.getValue("GRANTED")))
			{
				if ("admin".equals(rdbRoles.getValue("NAME")) && getConnection().getUserName().equals(rdbUsers.getValue("USERNAME")))
				{
					rdbRoles.restoreSelectedRow();
					
					throw new Exception("You are not allowed to revoke your own admin role!");
				}
				else
				{
					rdbUserRole.delete();
				}
			}
			else
			{
				rdbUserRole.insert(false);
				rdbUserRole.saveSelectedRow();
			}
		}
		doReloadUsers();
	}
	
}	// UserAdministrationWorkScreen
