/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 15.10.2009 - [RH] - creation
 * 08.11.2009 - [JR] - doChangeLanguage implemented
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.UIImage;
import javax.rad.genui.component.UIButton;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.model.IDataBook;
import javax.rad.model.event.DataRowEvent;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;

import com.sibvisions.apps.packung.PackungApplication;
import com.sibvisions.apps.packung.components.AbstractStyledWorkScreen;

/**
 * The <code>UserProfileDialog</code> allows the user to change its personal information.   
 * 
 * @author Martin Handsteiner
 */
public class UserProfileDialog extends AbstractStyledWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** User Defaults DataBook. */
	private IDataBook		rdbUserDefaults;

	/** the label user name. */
	private UILabel 		labUsername			= new UILabel();
	/** the editor user name. */
	private UIEditor 		editUsername		= new UIEditor();
	/** the label first name. */
	private UILabel 		labFirstName		= new UILabel();
	/** the editor first name. */
	private UIEditor 		editFirstName		= new UIEditor();
	/** the label last name. */
	private UILabel 		labLastName			= new UILabel();
	/** the editor last name. */
	private UIEditor 		editLastName		= new UIEditor();
	/** the label email. */
	private UILabel 		labEmail			= new UILabel();
	/** the editor email. */
	private UIEditor 		editEmail			= new UIEditor();
	/** the label email. */
	private UILabel 		labLanguage			= new UILabel();
	/** the editor email. */
	private UIEditor 		editLanguage		= new UIEditor();

	/** the ok button. */
	private UIButton		btnOk				= new UIButton();
	/** the ok button. */
	private UIButton		btnCancel			= new UIButton();
	
	/** the layout for the user profile settings. */
	private UIFormLayout 	folUserProfile		= new UIFormLayout();
	/** the panel for the user profile settings. */
	private UIGroupPanel	panUserProfile		= new UIGroupPanel();

	/** the layout for this component. */
	private UIFormLayout 	folThis				= new UIFormLayout();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * It creates the <code>UserProfileDialog</code> with the current <code>Application</code> 
	 * and <code>AbstractConnection</code>.
	 * 
	 * @param pApplication 	the application
	 * @param pConnection 	the current connection to the server
	 * @throws Exception if initialization fails.
	 */
	public UserProfileDialog(IWorkScreenApplication pApplication, AbstractConnection pConnection) throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}
		
	/**
	 * Initializes the model.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeModel() throws Exception
	{
		rdbUserDefaults = ((PackungApplication)getApplication()).getUserDefaults();
		
		rdbUserDefaults.eventValuesChanged().addListener(this, "doChangeLanguage");		
	}
    
	/**
	 * Initializes the UI.
	 * 
	 * @throws Exception if the initialization throws an error
	 */
	private void initializeUI() throws Exception
	{
		labUsername.setText("Username:");
		editUsername.setDataRow(rdbUserDefaults);
		editUsername.setColumnName("USERNAME");
   
		labFirstName.setText("First name:");
		editFirstName.setDataRow(rdbUserDefaults);
		editFirstName.setColumnName("FIRST_NAME");
   
		labLastName.setText("Last name:");
		editLastName.setDataRow(rdbUserDefaults);
		editLastName.setColumnName("LAST_NAME");
   
		labEmail.setText("Email:");
		editEmail.setDataRow(rdbUserDefaults);
		editEmail.setColumnName("EMAIL");
		editEmail.setMinimumSize(250, 0);
   
		labLanguage.setText("Language:");
		editLanguage.setDataRow(rdbUserDefaults);
		editLanguage.setColumnName("LANGUAGE");
   
		btnOk.setText("OK");
		btnOk.setImage(UIImage.getImage(UIImage.OK_SMALL));
		btnOk.eventAction().addListener(this, "doOk");

		btnCancel.setText("Cancel");
		btnCancel.eventAction().addListener(this, "doCancel");
   
		folUserProfile.setMargins(5, 5, 5, 5);
		folUserProfile.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		panUserProfile.setText("Change User Profile");
		panUserProfile.setLayout(folUserProfile);
		panUserProfile.add(labUsername, folUserProfile.getConstraints(0, 0));
		panUserProfile.add(editUsername, folUserProfile.getConstraints(1, 0));
		panUserProfile.add(labFirstName, folUserProfile.getConstraints(0, 1));
		panUserProfile.add(editFirstName, folUserProfile.getConstraints(1, 1));
		panUserProfile.add(labLastName, folUserProfile.getConstraints(0, 2));
		panUserProfile.add(editLastName, folUserProfile.getConstraints(1, 2));
		panUserProfile.add(labEmail, folUserProfile.getConstraints(0, 3));
		panUserProfile.add(editEmail, folUserProfile.getConstraints(1, 3, 2, 3));
		panUserProfile.add(labLanguage, folUserProfile.getConstraints(0, 4));
		panUserProfile.add(editLanguage, folUserProfile.getConstraints(1, 4));
		
		folThis.setMargins(5, 5, 5, 5);
		folThis.setVerticalGap(10);
		setLayout(folThis);
		add(panUserProfile, folThis.getConstraints(0, 0, -1, -2));
		add(btnCancel, folThis.getConstraints(0, -1));
		add(btnOk, folThis.getConstraints(-1, -1));
	}
		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Overwritten methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	
	/**
	 * {@inheritDoc}
	 */
	public void notifyDestroy()
	{
		rdbUserDefaults.eventValuesChanged().removeListener(this);

		try
		{
			rdbUserDefaults.restoreSelectedRow();

			if (!((PackungApplication)getApplication()).getLanguage().equals((String)rdbUserDefaults.getValue("LANG_CODE")))
			{
				((PackungApplication)getApplication()).setLanguage((String)rdbUserDefaults.getValue("LANG_CODE"));
			}
		}
		catch (Exception ex)
		{
			// silent restore.
		}
		
		super.notifyDestroy();
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Saves all changes.
	 * 
	 * @throws Throwable if it's not possible to change the profile
	 */
	public void doOk() throws Throwable
	{
		rdbUserDefaults.saveSelectedRow();
		
		getApplication().close(this);
	}
	
	/**
	 * Closes the screen without saving.
	 */
	public void doCancel()
	{
		getApplication().close(this);
	}
	
	/**
	 * Checks, if the own user is the current user.
	 * 
	 * @param pDataRowEvent the data row event.
	 * @throws Exception if an exception occurs.
	 */
	public void doChangeLanguage(DataRowEvent pDataRowEvent) throws Exception
	{
		if (pDataRowEvent.isChangedColumnName("LANGUAGE"))
		{
			((PackungApplication)getApplication()).setLanguage((String)rdbUserDefaults.getValue("LANG_CODE"));
		}
	}
	
} 	// UserProfileDialog
