/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.model.SortDefinition;
import javax.rad.model.condition.Equals;

import com.sibvisions.apps.packung.Session;
import com.sibvisions.rad.persist.jdbc.DBStorage;

/**
 * The <code>CompetitionOverview</code> class is the lifecycle object for the 
 * {@link CompetitionOverviewWorkScreen}. It manages all business objects.
 * 
 * @author Roland Hrmann
 */
public class CompetitionOverview extends Session
{	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Returns the competition details DBStorage.
	 * 
	 * @return the competition details DBStorage.
	 * @throws Exception if the competition details DBStorage couldn't be initialized.
	 */
	public DBStorage getCompetitionDetails() throws Exception
	{
		DBStorage competitionDetails = (DBStorage)get("competitionDetails");
		if (competitionDetails == null)
		{
			competitionDetails = new DBStorage();
			competitionDetails.setDBAccess(getDBAccess());
			competitionDetails.setFromClause("V_COOV_COMPETITION_DETAILS");
			competitionDetails.setRestrictCondition(new Equals("USER_ID", getUserId()));
			competitionDetails.open();
			
			put("competitionDetails", competitionDetails);
		}
		return competitionDetails;
	}
	
	/**
	 * Returns the stages DBStorage.
	 * 
	 * @return the stages DBStorage.
	 * @throws Exception if the stages DBStorage couldn't be initialized.
	 */
	public DBStorage getStages() throws Exception
	{
		DBStorage stages = (DBStorage)get("stages");
		if (stages == null)
		{
			stages = new DBStorage();
			stages.setDBAccess(getDBAccess());
			stages.setFromClause("V_COOV_STAGES");
			stages.setWritebackTable("STAGES");
			stages.setDefaultSort(new SortDefinition("SORTORDER_ID"));
			stages.open();
			
			put("stages", stages);
		}
		return stages;
	}
	
	/**
	 * Returns the groups DBStorage.
	 * 
	 * @return the groups DBStorage.
	 * @throws Exception if the groups DBStorage couldn't be initialized.
	 */
	public DBStorage getGroups() throws Exception
	{
		DBStorage groups = (DBStorage)get("groups");
		if (groups == null)
		{
			groups = new DBStorage();
			groups.setDBAccess(getDBAccess());
			groups.setFromClause("V_COOV_GROUPS");
			groups.setWritebackTable("GROUPS");
			groups.setDefaultSort(new SortDefinition("GROUPNAME"));
			groups.open();
			
			put("groups", groups);
		}
		return groups;
	}

	/**
	 * Returns the groupteams DBStorage.
	 * 
	 * @return the groupteams DBStorage.
	 * @throws Exception if the groupteams DBStorage couldn't be initialized.
	 */
	public DBStorage getGroupteams() throws Exception
	{
		DBStorage groupteams = (DBStorage)get("groupteams");
		if (groupteams == null)
		{
			groupteams = new DBStorage();
			groupteams.setDBAccess(getDBAccess());
			groupteams.setFromClause("V_COOV_GROUPTEAMS");
			groupteams.setWritebackTable("GROUPTEAMS");
			groupteams.setDefaultSort(new SortDefinition("RANK_ID", "SORT_ID"));
			groupteams.open();
			
			put("groupteams", groupteams);
		}
		return groupteams;
	}

	/**
	 * Returns the matches DBStorage.
	 * 
	 * @return the matches DBStorage.
	 * @throws Exception if the matches DBStorage couldn't be initialized.
	 */
	public DBStorage getMatches() throws Exception
	{
		DBStorage matches = (DBStorage)get("matches");
		if (matches == null)
		{
			matches = new DBStorage();
			matches.setDBAccess(getDBAccess());
			matches.setFromClause("V_COOV_MATCHES");
			matches.setWritebackTable("MATCHES");
			matches.setDefaultSort(new SortDefinition("MATCHBEGIN"));
			matches.open();
			
			put("matches", matches);
		}
		return matches;
	}

}	// CompetitionOverview
