/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.model.SortDefinition;

import com.sibvisions.apps.packung.Session;
import com.sibvisions.rad.persist.jdbc.DBStorage;

/**
 * The <code>Competitions</code> class is the lifecycle object for the 
 * {@link CompetitionsWorkScreen}. It manages all business objects.
 * 
 * @author Roland Hrmann
 */
public class Competitions extends Session
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Returns the competitions DBStorage.
	 * 
	 * @return the competitions DBStorage.
	 * @throws Exception if the competitions DBStorage couldn't be initialized.
	 */
	public DBStorage getCompetitions() throws Exception
	{
		DBStorage competitions = (DBStorage)get("competitions");
		if (competitions == null)
		{
			competitions = new DBStorage();
			competitions.setDBAccess(getDBAccess());
			competitions.setFromClause("V_COMP_COMPETITIONS");
			competitions.setWritebackTable("COMPETITIONS");
			competitions.setDefaultSort(new SortDefinition(false, "ID"));
			competitions.open();
			
			put("competitions", competitions);
		}
		return competitions;
	}

	/**
	 * Returns the stages DBStorage.
	 * 
	 * @return the stages DBStorage.
	 * @throws Exception if the stages DBStorage couldn't be initialized.
	 */
	public DBStorage getStages() throws Exception
	{
		DBStorage stages = (DBStorage)get("stages");
		if (stages == null)
		{
			stages = new DBStorage();
			stages.setDBAccess(getDBAccess());
			stages.setFromClause("V_COMP_STAGES");
			stages.setWritebackTable("STAGES");
			stages.setDefaultSort(new SortDefinition("SORTORDER_ID"));
			stages.open();
			
			put("stages", stages);
		}
		return stages;
	}
	
	/**
	 * Returns the groups DBStorage.
	 * 
	 * @return the groups DBStorage.
	 * @throws Exception if the groups DBStorage couldn't be initialized.
	 */
	public DBStorage getGroups() throws Exception
	{
		DBStorage groups = (DBStorage)get("groups");
		if (groups == null)
		{
			groups = new DBStorage();
			groups.setDBAccess(getDBAccess());
			groups.setFromClause("V_COMP_GROUPS");
			groups.setWritebackTable("GROUPS");
			groups.setDefaultSort(new SortDefinition("GROUPNAME"));
			groups.open();
			
			put("groups", groups);
		}
		return groups;
	}

	/**
	 * Returns the groupteams DBStorage.
	 * 
	 * @return the groupteams DBStorage.
	 * @throws Exception if the groupteams DBStorage couldn't be initialized.
	 */
	public DBStorage getGroupteams() throws Exception
	{
		DBStorage groupteams = (DBStorage)get("groupteams");
		if (groupteams == null)
		{
			groupteams = new DBStorage();
			groupteams.setDBAccess(getDBAccess());
			groupteams.setFromClause("V_COMP_GROUPTEAMS");
			groupteams.setWritebackTable("GROUPTEAMS");
			groupteams.setDefaultSort(new SortDefinition("TEAM_PLACEHOLDER"));
			groupteams.open();
			
			put("groupteams", groupteams);
		}
		return groupteams;
	}

	/**
	 * Returns the matches DBStorage.
	 * 
	 * @return the matches DBStorage.
	 * @throws Exception if the matches DBStorage couldn't be initialized.
	 */
	public DBStorage getMatches() throws Exception
	{
		DBStorage matches = (DBStorage)get("matches");
		if (matches == null)
		{
			matches = new DBStorage();
			matches.setDBAccess(getDBAccess());
			matches.setFromClause("V_COMP_MATCHES");
			matches.setWritebackTable("MATCHES");
			matches.setDefaultSort(new SortDefinition("MATCHBEGIN"));
			matches.open();
			
			put("matches", matches);
		}
		return matches;
	}

	/**
	 * Returns the match days DBStorage.
	 * 
	 * @return the match days DBStorage.
	 * @throws Exception if the match days DBStorage couldn't be initialized.
	 */
	public DBStorage getMatchdays() throws Exception
	{
		DBStorage matchdays = (DBStorage)get("matchdays");
		if (matchdays == null)
		{
			matchdays = new DBStorage();
			matchdays.setDBAccess(getDBAccess());
			matchdays.setFromClause("V_COMP_MATCHDAYS");
			matchdays.setWritebackTable("MATCHDAYS");
			matchdays.setDefaultSort(new SortDefinition("MATCHDAY"));
			matchdays.open();
			
			put("matchdays", matchdays);
		}
		return matchdays;
	}
	
	/**
	 * Returns the match places DBStorage.
	 * 
	 * @return the match places DBStorage.
	 * @throws Exception if the match places DBStorage couldn't be initialized.
	 */
	public DBStorage getMatchplaces() throws Exception
	{
		DBStorage matchplaces = (DBStorage)get("matchplaces");
		if (matchplaces == null)
		{
			matchplaces = new DBStorage();
			matchplaces.setDBAccess(getDBAccess());
			matchplaces.setFromClause("V_COMP_MATCHPLACES");
			matchplaces.setWritebackTable("MATCHPLACES");
			matchplaces.setDefaultSort(new SortDefinition("TOWN", "PLACE"));
			matchplaces.open();
			
			put("matchplaces", matchplaces);
		}
		return matchplaces;
	}
	
	/**
	 * Returns the competitiontypes DBStorage.
	 * 
	 * @return the competitiontypes DBStorage.
	 * @throws Exception if the competitiontypes DBStorage couldn't be initialized.
	 */
	public DBStorage getCompetitionTypesLook() throws Exception
	{
		DBStorage competitiontypes = (DBStorage)get("competitiontypesLook");
		if (competitiontypes == null)
		{
			competitiontypes = new DBStorage();
			competitiontypes.setDBAccess(getDBAccess());
			competitiontypes.setFromClause("V_COMP_COMPETITIONTYPES_LOOK");
			competitiontypes.setDefaultSort(new SortDefinition("COMPETITION_TYPE"));
			competitiontypes.open();
			
			put("competitiontypesLook", competitiontypes);
		}
		return competitiontypes;
	}
	
	/**
	 * Returns the teams DBStorage.
	 * 
	 * @return the teams DBStorage.
	 * @throws Exception if the teams DBStorage couldn't be initialized.
	 */
	public DBStorage getTeamsLook() throws Exception
	{
		DBStorage teamsLook = (DBStorage)get("teamsLook");
		if (teamsLook == null)
		{
			teamsLook = new DBStorage();
			teamsLook.setDBAccess(getDBAccess());
			teamsLook.setFromClause("V_COMP_TEAMS_LOOK");
			teamsLook.setDefaultSort(new SortDefinition("TEAM"));
			teamsLook.open();
			
			put("teamsLook", teamsLook);
		}
		return teamsLook;
	}
	
	/**
	 * Returns the fill in ranks DBStorage.
	 * 
	 * @return the fill in ranks DBStorage.
	 * @throws Exception if the fill in ranks DBStorage couldn't be initialized.
	 */
	public DBStorage getFillinRanksLook() throws Exception
	{
		DBStorage fillinRanksLook = (DBStorage)get("fillinRanksLook");
		if (fillinRanksLook == null)
		{
			fillinRanksLook = new DBStorage();
			fillinRanksLook.setDBAccess(getDBAccess());
			fillinRanksLook.setFromClause("V_COMP_FILLIN_RANKS_LOOK");
			fillinRanksLook.setDefaultSort(new SortDefinition("ID"));
			fillinRanksLook.open();
			
			put("fillinRanksLook", fillinRanksLook);
		}
		return fillinRanksLook;
	}
	
	/**
	 * Returns the fill in groups DBStorage.
	 * 
	 * @return the fill in groups DBStorage.
	 * @throws Exception if the fill in groups DBStorage couldn't be initialized.
	 */
	public DBStorage getFillinGroupsLook() throws Exception
	{
		DBStorage fillinGroupsLook = (DBStorage)get("fillinGroupsLook");
		if (fillinGroupsLook == null)
		{
			fillinGroupsLook = new DBStorage();
			fillinGroupsLook.setDBAccess(getDBAccess());
			fillinGroupsLook.setFromClause("V_COMP_FILLIN_GROUPS_LOOK");
			fillinGroupsLook.setDefaultSort(new SortDefinition("GROUPNAME"));
			fillinGroupsLook.open();
			
			put("fillinGroupsLook", fillinGroupsLook);
		}
		return fillinGroupsLook;
	}
	
}	// Competitions
