/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 12.10.2009 - [JR] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.model.SortDefinition;

import com.sibvisions.apps.packung.Session;
import com.sibvisions.rad.persist.jdbc.DBStorage;

/**
 * The <code>Configuration</code> class is the lifecycle object for the 
 * {@link ConfigurationWorkScreen}. It manages all business objects.
 * 
 * @author Ren Jahn
 */
public class Configuration extends Session
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Returns the languages DBStorage.
	 * 
	 * @return the languages DBStorage.
	 * @throws Exception if the languages DBStorage couldn't be initialized.
	 */
	public DBStorage getLanguages() throws Exception
	{
		DBStorage dbsInfoTexts = (DBStorage)get("languages");
		
		if (dbsInfoTexts == null)
		{
			dbsInfoTexts = new DBStorage();
			dbsInfoTexts.setDBAccess(getDBAccess());
			dbsInfoTexts.setFromClause("V_COFG_LANGUAGES");
			dbsInfoTexts.setWritebackTable("LANGUAGES");
			dbsInfoTexts.setDefaultSort(new SortDefinition("LANGUAGE"));
			dbsInfoTexts.open();
			
			put("languages", dbsInfoTexts);
		}
		
		return dbsInfoTexts;
	}

	/**
	 * Returns the infotexts DBStorage.
	 * 
	 * @return the infotexts DBStorage.
	 * @throws Exception if the infotexts DBStorage couldn't be initialized.
	 */
	public DBStorage getInfoTexts() throws Exception
	{
		DBStorage dbsInfoTexts = (DBStorage)get("infoTexts");
		
		if (dbsInfoTexts == null)
		{
			dbsInfoTexts = new DBStorage();
			dbsInfoTexts.setDBAccess(getDBAccess());
			dbsInfoTexts.setFromClause("V_COFG_INFOTEXTS");
			dbsInfoTexts.setWritebackTable("INFOTEXTS");
			dbsInfoTexts.open();
			
			put("infoTexts", dbsInfoTexts);
		}
		
		return dbsInfoTexts;
	}
	
}	// Configuration
