/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 15.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.model.SortDefinition;
import javax.rad.model.condition.Equals;

import com.sibvisions.apps.packung.Session;
import com.sibvisions.rad.persist.jdbc.DBStorage;

/**
 * The <code>MyBets</code> class is the lifecycle object for the 
 * {@link MyBetsWorkScreen}. It manages all business objects.
 * 
 * @author Roland Hrmann
 */
public class MyBets extends Session
{	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Returns the user details DBStorage.
	 * 
	 * @return the user details DBStorage.
	 * @throws Exception if the user details DBStorage couldn't be initialized.
	 */
	public DBStorage getUserDetails() throws Exception
	{
		DBStorage userDetails = (DBStorage)get("userDetails");
		if (userDetails == null)
		{
			userDetails = new DBStorage();
			userDetails.setDBAccess(getDBAccess());
			userDetails.setFromClause("V_MYBE_USER_DETAILS");
			userDetails.setWritebackTable("USERS");
			userDetails.setRestrictCondition(new Equals("ID", getUserId()));
			userDetails.open();
			
			put("userDetails", userDetails);
		}
		return userDetails;
	}
	
	/**
	 * Returns the match days history DBStorage.
	 * 
	 * @return the match days history DBStorage.
	 * @throws Exception if the match days history DBStorage couldn't be initialized.
	 */
	public DBStorage getMatchdaysHistory() throws Exception
	{
		DBStorage matchdaysHistory = (DBStorage)get("matchdaysHistory");
		if (matchdaysHistory == null)
		{
			matchdaysHistory = new DBStorage();
			matchdaysHistory.setDBAccess(getDBAccess());
			matchdaysHistory.setFromClause("V_MYBE_MATCHDAYS_HISTORY");
			matchdaysHistory.setRestrictCondition(new Equals("USER_ID", getUserId()));
			matchdaysHistory.setDefaultSort(new SortDefinition(false, "SORT_ID"));
			matchdaysHistory.open();
			
			put("matchdaysHistory", matchdaysHistory);
		}
		return matchdaysHistory;
	}
	
	/**
	 * Returns the bets history DBStorage.
	 * 
	 * @return the bets history DBStorage.
	 * @throws Exception if the bets history DBStorage couldn't be initialized.
	 */
	public DBStorage getBetsHistory() throws Exception
	{
		DBStorage betsHistory = (DBStorage)get("betsHistory");
		if (betsHistory == null)
		{
			betsHistory = new DBStorage();
			betsHistory.setDBAccess(getDBAccess());
			betsHistory.setFromClause("V_MYBE_BETS_HISTORY");
			betsHistory.setRestrictCondition(new Equals("USER_ID", getUserId()));
			betsHistory.setDefaultSort(new SortDefinition("MATCHBEGIN"));
			betsHistory.open();
			
			put("betsHistory", betsHistory);
		}
		return betsHistory;
	}
	
	/**
	 * Returns the match days DBStorage.
	 * 
	 * @return the match days DBStorage.
	 * @throws Exception if the match days DBStorage couldn't be initialized.
	 */
	public DBStorage getMatchdays() throws Exception
	{
		DBStorage matchdays = (DBStorage)get("matchdays");
		if (matchdays == null)
		{
			matchdays = new DBStorage();
			matchdays.setDBAccess(getDBAccess());
			matchdays.setFromClause("V_MYBE_MATCHDAYS");
			matchdays.setWritebackTable("MATCHDAYS");
			matchdays.setRestrictCondition(new Equals("USER_ID", getUserId()));
			matchdays.setDefaultSort(new SortDefinition("SORT_ID"));
			matchdays.open();
			
			put("matchdays", matchdays);
		}
		return matchdays;
	}
	
	/**
	 * Returns the bets DBStorage.
	 * 
	 * @return the bets DBStorage.
	 * @throws Exception if the bets DBStorage couldn't be initialized.
	 */
	public DBStorage getBets() throws Exception
	{
		DBStorage bets = (DBStorage)get("bets");
		if (bets == null)
		{
			bets = new DBStorage();
			bets.setDBAccess(getDBAccess());
			bets.setFromClause("V_MYBE_BETS");
			bets.setWritebackTable("BETS");
			bets.setRestrictCondition(new Equals("USER_ID", getUserId()));
			bets.setDefaultSort(new SortDefinition("MATCHBEGIN"));
			bets.open();
			
			put("bets", bets);
		}
		return bets;
	}
		
	/**
	 * Returns the top bets DBStorage.
	 * 
	 * @return the top bets DBStorage.
	 * @throws Exception if the top bets DBStorage couldn't be initialized.
	 */
	public DBStorage getTopbets() throws Exception
	{
		DBStorage topbets = (DBStorage)get("topbets");
		if (topbets == null)
		{
			topbets = new DBStorage();
			topbets.setDBAccess(getDBAccess());
			topbets.setFromClause("V_MYBE_TOPBETS");
			topbets.setWritebackTable("TOPBETS");
			topbets.setRestrictCondition(new Equals("USER_ID", getUserId()));
			topbets.open();
			
			put("topbets", topbets);
		}
		return topbets;
	}
		
	/**
	 * Returns the group teams look DBStorage.
	 * 
	 * @return the group teams look DBStorage.
	 * @throws Exception if the group teams look DBStorage couldn't be initialized.
	 */
	public DBStorage getGroupteamsLook() throws Exception
	{
		DBStorage groupteamsLook = (DBStorage)get("groupteamsLook");
		if (groupteamsLook == null)
		{
			groupteamsLook = new DBStorage();
			groupteamsLook.setDBAccess(getDBAccess());
			groupteamsLook.setFromClause("V_MYBE_GROUPTEAMS_LOOK");
			groupteamsLook.open();
			
			put("groupteamsLook", groupteamsLook);
		}
		return groupteamsLook;
	}
		
	/**
	 * Returns the bet groups DBStorage.
	 * 
	 * @return the bet groups DBStorage.
	 * @throws Exception if the bet groups DBStorage couldn't be initialized.
	 */
	public DBStorage getBetgroups() throws Exception
	{
		DBStorage betgroups = (DBStorage)get("betgroups");
		if (betgroups == null)
		{
			betgroups = new DBStorage();
			betgroups.setDBAccess(getDBAccess());
			betgroups.setFromClause("V_MYBE_BETGROUPS");
			betgroups.setWritebackTable("BETGROUPS");
			betgroups.open();
			
			put("betgroups", betgroups);
		}
		return betgroups;
	}
		
}	// MyBets
