/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.model.SortDefinition;
import javax.rad.model.condition.Equals;

import com.sibvisions.apps.packung.Session;
import com.sibvisions.rad.persist.jdbc.DBStorage;

/**
 * The <code>Results</code> class is the lifecycle object for the 
 * {@link ResultsWorkScreen}. It manages all business objects.
 * 
 * @author Roland Hrmann
 */
public class Results extends Session
{	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Returns the matches DBStorage.
	 * 
	 * @return the matches DBStorage.
	 * @throws Exception if the matches DBStorage couldn't be initialized.
	 */
	public DBStorage getMatches() throws Exception
	{
		DBStorage matches = (DBStorage)get("matches");
		if (matches == null)
		{
			matches = new DBStorage();
			matches.setDBAccess(getDBAccess());
			matches.setFromClause("V_RESU_MATCHES");
			matches.setWritebackTable("MATCHES");
			matches.setDefaultSort(new SortDefinition("MATCHBEGIN"));
			matches.setRestrictCondition(new Equals("USER_ID", getUserId()));
			matches.open();
			
			put("matches", matches);
		}
		return matches;
	}

	/**
	 * Returns the matches history DBStorage.
	 * 
	 * @return the matches history DBStorage.
	 * @throws Exception if the matches history DBStorage couldn't be initialized.
	 */
	public DBStorage getMatchesHistory() throws Exception
	{
		DBStorage matchesHistory = (DBStorage)get("matchesHistory");
		if (matchesHistory == null)
		{
			matchesHistory = new DBStorage();
			matchesHistory.setDBAccess(getDBAccess());
			matchesHistory.setFromClause("V_RESU_MATCHES_HISTORY");
			matchesHistory.setWritebackTable("MATCHES");
			matchesHistory.setDefaultSort(new SortDefinition(false, "MATCHBEGIN"));
			matchesHistory.setRestrictCondition(new Equals("USER_ID", getUserId()));
			matchesHistory.open();
			
			put("matchesHistory", matchesHistory);
		}
		return matchesHistory;
	}
	
	/**
	 * Fills in missing group teams, if a stage is finished.
	 * 
	 * @throws Exception if the teams cannot be filled in.
	 */
	public void fillInGroupteams() throws Exception
	{
		getDBAccess().getConnection().createStatement().executeUpdate(
				"update groupteams gt " +
                   "set gt.team_id = (select max(y.team_id) " +
                                       "from v_coov_groupteams y " +
                                      "where y.grou_id = gt.fillin_grou_id " +
                                        "and y.rank_id = gt.fillin_rank_id) " +
                 "where gt.team_id is null " +
                   "and gt.stag_id in (select x.id " +
                                        "from v_global_stages_finished x)");
	}

}	// Results
