/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 18.10.2009 - [RH] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.model.SortDefinition;
import javax.rad.model.condition.Equals;

import com.sibvisions.apps.packung.Session;
import com.sibvisions.rad.persist.jdbc.DBStorage;

/**
 * The <code>Statistics</code> class is the lifecycle object for the 
 * {@link StatisticsWorkScreen}. It manages all business objects.
 * 
 * @author Roland Hrmann
 */
public class Statistics extends Session
{	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Returns the bet groups days DBStorage.
	 * 
	 * @return the bet groups days DBStorage.
	 * @throws Exception if the bet groups days DBStorage couldn't be initialized.
	 */
	public DBStorage getBetgroups() throws Exception
	{
		DBStorage betgroups = (DBStorage)get("betgroups");
		if (betgroups == null)
		{
			betgroups = new DBStorage();
			betgroups.setDBAccess(getDBAccess());
			betgroups.setFromClause("V_STAT_BETGROUPS");
			betgroups.setWritebackTable("BETGROUPS");
			betgroups.setRestrictCondition(new Equals("USER_ID", getUserId()));
			betgroups.open();
			
			put("betgroups", betgroups);
		}
		return betgroups;
	}
		
	/**
	 * Returns the players DBStorage.
	 * 
	 * @return the players DBStorage.
	 * @throws Exception if the players DBStorage couldn't be initialized.
	 */
	public DBStorage getPlayers() throws Exception
	{
		DBStorage players = (DBStorage)get("players");
		if (players == null)
		{
			players = new DBStorage();
			players.setDBAccess(getDBAccess());
			players.setFromClause("V_STAT_PLAYERS");
			players.setWritebackTable("USERS");
			players.setDefaultSort(new SortDefinition("RANK"));
			players.open();
			
			put("players", players);
		}
		return players;
	}

	/**
	 * Returns the match days history DBStorage.
	 * 
	 * @return the match days history DBStorage.
	 * @throws Exception if the match days history DBStorage couldn't be initialized.
	 */
	public DBStorage getMatchdaysHistory() throws Exception
	{
		DBStorage matchdaysHistory = (DBStorage)get("matchdaysHistory");
		if (matchdaysHistory == null)
		{
			matchdaysHistory = new DBStorage();
			matchdaysHistory.setDBAccess(getDBAccess());
			matchdaysHistory.setFromClause("V_STAT_MATCHDAYS_HISTORY");
			matchdaysHistory.setDefaultSort(new SortDefinition(false, "SORT_ID"));
			matchdaysHistory.open();
			
			put("matchdaysHistory", matchdaysHistory);
		}
		return matchdaysHistory;
	}
	
	/**
	 * Returns the bets history DBStorage.
	 * 
	 * @return the bets history DBStorage.
	 * @throws Exception if the bets history DBStorage couldn't be initialized.
	 */
	public DBStorage getBetsHistory() throws Exception
	{
		DBStorage betsHistory = (DBStorage)get("betsHistory");
		if (betsHistory == null)
		{
			betsHistory = new DBStorage();
			betsHistory.setDBAccess(getDBAccess());
			betsHistory.setFromClause("V_STAT_BETS_HISTORY");
			betsHistory.setDefaultSort(new SortDefinition("MATCHBEGIN"));
			betsHistory.open();
			
			put("betsHistory", betsHistory);
		}
		return betsHistory;
	}
	
	/**
	 * Returns the userImages DBStorage.
	 * 
	 * @return the userImages DBStorage.
	 * @throws Exception if the userImages DBStorage couldn't be initialized.
	 */
	public DBStorage getUserImages() throws Exception
	{
		DBStorage userImages = (DBStorage)get("userImages");
		if (userImages == null)
		{
			userImages = new DBStorage();
			userImages.setDBAccess(getDBAccess());
			userImages.setFromClause("V_STAT_USER_IMAGES");
			userImages.open();
			
			put("userImages", userImages);
		}
		return userImages;
	}
	
}	// Statistics
