/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [HM] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.model.SortDefinition;

import com.sibvisions.apps.packung.Session;
import com.sibvisions.rad.persist.jdbc.DBStorage;

/**
 * The <code>Teams</code> class is the lifecycle object for the 
 * {@link TeamsWorkScreen}. It manages all business objects.
 * 
 * @author Martin Handsteiner
 */
public class Teams extends Session
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Returns the teams DBStorage.
	 * 
	 * @return the teams DBStorage.
	 * @throws Exception if the teams DBStorage couldn't be initialized.
	 */
	public DBStorage getTeams() throws Exception
	{
		DBStorage teams = (DBStorage)get("teams");
		if (teams == null)
		{
			teams = new DBStorage();
			teams.setDBAccess(getDBAccess());
			teams.setFromClause("V_TEAM_TEAMS");
			teams.setWritebackTable("TEAMS");
			teams.setDefaultSort(new SortDefinition("TEAM"));
			teams.open();
			
			put("teams", teams);
		}
		return teams;
	}

	/**
	 * Returns the competitiontypes DBStorage.
	 * 
	 * @return the competitiontypes DBStorage.
	 * @throws Exception if the competitiontypes DBStorage couldn't be initialized.
	 */
	public DBStorage getCompetitiontypes() throws Exception
	{
		DBStorage competitiontypes = (DBStorage)get("competitiontypes");
		if (competitiontypes == null)
		{
			competitiontypes = new DBStorage();
			competitiontypes.setDBAccess(getDBAccess());
			competitiontypes.setFromClause("V_TEAM_COMPETITIONTYPES");
			competitiontypes.setWritebackTable("COMPETITIONTYPES");
			competitiontypes.setDefaultSort(new SortDefinition("COMPETITION_TYPE"));
			competitiontypes.open();
			
			put("competitiontypes", competitiontypes);
		}
		return competitiontypes;
	}

	/**
	 * Returns the cotyTeam DBStorage.
	 * 
	 * @return the cotyTeam DBStorage.
	 * @throws Exception if the cotyTeam DBStorage couldn't be initialized.
	 */
	public DBStorage getCotyTeam() throws Exception
	{
		DBStorage cotyTeam = (DBStorage)get("cotyTeam");
		if (cotyTeam == null)
		{
			cotyTeam = new DBStorage();
			cotyTeam.setDBAccess(getDBAccess());
			cotyTeam.setWritebackTable("COTY_TEAM");
			cotyTeam.open();
			
			put("cotyTeam", cotyTeam);
		}
		return cotyTeam;
	}

	/**
	 * Returns the competitiontypesLook DBStorage.
	 * 
	 * @return the competitiontypesLook DBStorage.
	 * @throws Exception if the competitiontypesLook DBStorage couldn't be initialized.
	 */
	public DBStorage getCompetitiontypesLook() throws Exception
	{
		DBStorage competitiontypesLook = (DBStorage)get("competitiontypesLook");
		if (competitiontypesLook == null)
		{
			competitiontypesLook = new DBStorage();
			competitiontypesLook.setDBAccess(getDBAccess());
			competitiontypesLook.setFromClause("V_TEAM_COMPETITIONTYPES_LOOK");
			competitiontypesLook.open();
			
			put("competitiontypesLook", competitiontypesLook);
		}
		return competitiontypesLook;
	}
	
}	// Teams
