/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 06.10.2009 - [HM] - creation
 */
package com.sibvisions.apps.packung.screens;

import javax.rad.model.SortDefinition;

import com.sibvisions.apps.packung.Session;
import com.sibvisions.rad.persist.jdbc.DBStorage;

/**
 * The <code>UserAdministration</code> class is the lifecycle object for the 
 * {@link UserAdministrationWorkScreen}. It manages all business objects.
 * 
 * @author Martin Handsteiner
 */
public class UserAdministration extends Session
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Returns the users DBStorage.
	 * 
	 * @return the users DBStorage.
	 * @throws Exception if the users DBStorage couldn't be initialized.
	 */
	public DBStorage getUsers() throws Exception
	{
		DBStorage users = (DBStorage)get("users");
		if (users == null)
		{
			users = new DBStorage();
			users.setDBAccess(getDBAccess());
			users.setFromClause("V_USER_USERS");
			users.setWritebackTable("USERS");
			users.setDefaultSort(new SortDefinition("USERNAME"));
			users.open();
			
			put("users", users);
		}
		return users;
	}

	/**
	 * Returns the roles DBStorage.
	 * 
	 * @return the roles DBStorage.
	 * @throws Exception if the roles DBStorage couldn't be initialized.
	 */
	public DBStorage getRoles() throws Exception
	{
		DBStorage roles = (DBStorage)get("roles");
		if (roles == null)
		{
			roles = new DBStorage();
			roles.setDBAccess(getDBAccess());
			roles.setFromClause("V_USER_ROLES");
			roles.setWritebackTable("ROLES");
			roles.open();
			
			put("roles", roles);
		}
		return roles;
	}

	/**
	 * Returns the userRole DBStorage.
	 * 
	 * @return the userRole DBStorage.
	 * @throws Exception if the userRole DBStorage couldn't be initialized.
	 */
	public DBStorage getUserRole() throws Exception
	{
		DBStorage userRole = (DBStorage)get("userRole");
		if (userRole == null)
		{
			userRole = new DBStorage();
			userRole.setDBAccess(getDBAccess());
			userRole.setWritebackTable("USER_ROLE");
			userRole.open();
			
			put("userRole", userRole);
		}
		return userRole;
	}

	/**
	 * Returns the rolesLook DBStorage.
	 * 
	 * @return the rolesLook DBStorage.
	 * @throws Exception if the rolesLook DBStorage couldn't be initialized.
	 */
	public DBStorage getRolesLook() throws Exception
	{
		DBStorage rolesLook = (DBStorage)get("rolesLook");
		if (rolesLook == null)
		{
			rolesLook = new DBStorage();
			rolesLook.setDBAccess(getDBAccess());
			rolesLook.setFromClause("V_USER_ROLES_LOOK");
			rolesLook.open();
			
			put("rolesLook", rolesLook);
		}
		return rolesLook;
	}
	
}	// UserAdministration
